/* 
 * auxapp_main.c
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <dlfcn.h>
#include <assert.h>
#include <jni.h>
#include <link.h>

#include <auxapp.h>
#include <jnienv32.h>

struct ia32_runtime_fptr_table g_fptr_table;
void * table_p;

/* JNIEnv interface function table for the IA32 world.  */
#define DECLARE_INTERFACE_METHOD(_type, _name, _args) \
    extern _type _name _args;

/* JNIEnv function declarations */
#define JNIENVFN(_idx, _type, _name, _args) \
    DECLARE_INTERFACE_METHOD(_type, JNIENV_IA32_STUB(_name), _args)

#include "jnifns.h"

void *jnienv32_fns[] = {
#define JNIENVFN(_idx, _type, _name, _args) \
	[_idx] = (void *)JNIENV_IA32_STUB(_name),

#include "jnifns.h"
};

#define JAVAVMFN(_idx, _type, _name, _args) \
    DECLARE_INTERFACE_METHOD(_type, JAVAVM_IA32_STUB(_name), _args)

#include "jvmfns.h"

void *javavm32_fns[] = {
#define JAVAVMFN(_idx, _type, _name, _args) \
	[_idx] = (void *)JAVAVM_IA32_STUB(_name),

#include "jvmfns.h"
};

extern void auxapp_break (void);
extern void dummy_thread_func (void);
extern void phdr_iter_callback (void);
extern uint32_t binary_desc_init (const char *binary_full_path, uint32_t base_address);
extern int32_t binary_desc_finish (uint32_t);
extern int32_t get_symbol_info_str (uint32_t bd, uint32_t eip, char *info, uint32_t len);

int main ()
{
	/* get function addresses */
	g_fptr_table.dlopen = (uintptr_t)dlopen;
	g_fptr_table.dlsym = (uintptr_t)dlsym;
	g_fptr_table.dlerror = (uintptr_t)dlerror;
	g_fptr_table.dlclose = (uintptr_t)dlclose;
	g_fptr_table.pthread_create = (uintptr_t)pthread_create;
	g_fptr_table.pthread_attr_setstack = (uintptr_t)pthread_attr_setstack;
	g_fptr_table.pthread_attr_setstacksize = (uintptr_t)pthread_attr_setstacksize;
	g_fptr_table.pthread_attr_setdetachstate = (uintptr_t)pthread_attr_setdetachstate;
	g_fptr_table.pthread_attr_init = (uintptr_t)pthread_attr_init;
	g_fptr_table.pthread_exit = (uintptr_t)pthread_exit;
	g_fptr_table.malloc = (uintptr_t)malloc;
	g_fptr_table.realloc = (uintptr_t)realloc;
	g_fptr_table.free = (uintptr_t)free;
	g_fptr_table.dl_iterate_phdr = (uintptr_t)dl_iterate_phdr;
	g_fptr_table.jnienv32_fns = (uintptr_t)jnienv32_fns;
	g_fptr_table.javavm32_fns = (uintptr_t)javavm32_fns;
	g_fptr_table.dummy_thread_func = (uintptr_t)dummy_thread_func;
	g_fptr_table.phdr_iter_callback = (uintptr_t)phdr_iter_callback;
	g_fptr_table.binary_desc_init = (uintptr_t)binary_desc_init;
	g_fptr_table.binary_desc_finish = (uintptr_t)binary_desc_finish;
	g_fptr_table.get_symbol_info_str = (uintptr_t)get_symbol_info_str;

#if 0
	fprintf (stderr, "g_fptr_table=%p\n", &g_fptr_table);
	fprintf (stderr, "\tdlopen=%x\n"
			"\tdlsym=%x\n"
			"\tdlerror=%x\n"
			"\tdlclose=%x\n"
			"\tpthread_create=%x\n"
			"\tpthread_attr_setstack=%x\n"
			"\tpthread_attr_setstacksize=%x\n"
			"\tpthread_attr_setdetachstate=%x\n"
			"\tpthread_attr_init=%x\n"
			"\tpthread_exit=%x\n"
			"\tmalloc=%x\n"
			"\trealloc=%x\n"
			"\tfree=%x\n"
			"\tjnienv32_fns=%x\n"
			"\tjavavm32_fns=%x\n"
			"\tdummy_thread_func=%x\n",
			g_fptr_table.dlopen,
			g_fptr_table.dlsym,
			g_fptr_table.dlerror,
			g_fptr_table.dlclose,
			g_fptr_table.pthread_create,
			g_fptr_table.pthread_attr_setstack,
			g_fptr_table.pthread_attr_setstacksize,
			g_fptr_table.pthread_attr_setdetachstate,
			g_fptr_table.pthread_attr_init,
			g_fptr_table.pthread_exit,
			g_fptr_table.malloc,
			g_fptr_table.realloc,
			g_fptr_table.free,
			g_fptr_table.jnienv32_fns,
			g_fptr_table.javavm32_fns,
			g_fptr_table.dummy_thread_func);
#endif

	table_p = &g_fptr_table;
	auxapp_break();

	assert ( ! "should not get here" );
	return 0;
}

