/* 
 * btmix.h
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Stanislav Polevic <stanislav.v.polevic@intel.com>
 * Copyright (C) 2005       Vasily Levchenko <vasily.v.levchenko@intel.com>
 * Copyright (C) 2005       Alexander Alexeev <alexander.n.alexeev@intel.com>
 * Copyright (C) 2005       Valery Ushakov <valery.e.ushakov@intel.com>
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
#ifndef _BTMIX_H_
#define _BTMIX_H_

#include <stdint.h>
#include <string.h>
#include <jni.h>

#include <ia32el_mme_api.h>
#include <mme_internal.h>
#include <auxapp.h>

static inline uint32_t
btmix_get_upcall_argp(ia32el_context32 * ec32)
{
	return ec32->esp + MME_UPCALL_ARGS_OFFSET;
}

#define PRINTF_SPECIFIER_int8_t		"%x"
#define PRINTF_SPECIFIER_int16_t	"%x"
#define PRINTF_SPECIFIER_int32_t	"%x"
#define PRINTF_SPECIFIER_int64_t	"%lx"
#define PRINTF_SPECIFIER_uint8_t	"%x"
#define PRINTF_SPECIFIER_uint16_t	"%x"
#define PRINTF_SPECIFIER_uint32_t	"%x"
#define PRINTF_SPECIFIER_uint64_t	"%lx"
#define PRINTF_SPECIFIER_float		"%f"
#define PRINTF_SPECIFIER_double		"%e"



#define DEF_BTMIX_SET_RETVAL(_suffix,_ret_type_t,_treat_as) \
static inline void __attribute__ ((always_inline, unused))\
btmix_set_retval_##_suffix(ia32el_context32 * ec32, _ret_type_t value)\
{\
	union {\
		uint32_t esp;\
		void * ptr;\
	} pc;\
	void * retvalp;\
	const size_t size = sizeof (_treat_as);\
	_treat_as val = (_treat_as)value;\
\
	pc.ptr = NULL;\
	pc.esp = ec32->esp;\
	retvalp = pc.ptr - sizeof(_treat_as);\
	DBPRINT (0, "esp=%x retvalp=%p value="\
			PRINTF_SPECIFIER_##_ret_type_t\
			" val="\
			PRINTF_SPECIFIER_##_treat_as\
			" size=%ld\n",\
			ec32->esp, retvalp, value, val, size);\
	copy_to_ia32_stack (retvalp, &val, size);\
}

DEF_BTMIX_SET_RETVAL(char,	int8_t,		int32_t)
DEF_BTMIX_SET_RETVAL(uchar,	uint8_t,	uint32_t)
DEF_BTMIX_SET_RETVAL(short,	int16_t,	int32_t)
DEF_BTMIX_SET_RETVAL(ushort,	uint16_t,	uint32_t)
DEF_BTMIX_SET_RETVAL(int,	int32_t,	int32_t)
DEF_BTMIX_SET_RETVAL(uint,	uint32_t,	uint32_t)
DEF_BTMIX_SET_RETVAL(long,	int32_t,	int32_t)	/* ia32 longs are 32-bit */
DEF_BTMIX_SET_RETVAL(ulong,	uint32_t,	uint32_t)
DEF_BTMIX_SET_RETVAL(ptr,	uint32_t,	uint32_t)
DEF_BTMIX_SET_RETVAL(llong,	int64_t,	int64_t)
DEF_BTMIX_SET_RETVAL(ullong,	uint64_t,	uint64_t)
DEF_BTMIX_SET_RETVAL(float,	float,		float)
DEF_BTMIX_SET_RETVAL(double,	double,		double)


#define DEF_BTMIX_POP(_suffix, _arg_type_t, _size)\
static inline _arg_type_t \
btmix_pop_##_suffix(uint32_t *)\
	__attribute__ ((always_inline, unused));\
\
static inline _arg_type_t \
btmix_pop_##_suffix(uint32_t *psp32)\
{\
	union {\
		uint32_t esp;\
		void * ptr;\
	} pcs;\
	union {\
		_arg_type_t val;\
		uint32_t u;\
		uint64_t lu;\
	} pcv;\
	pcs.ptr = NULL;\
	pcs.esp = *psp32;\
	if (_size == sizeof (uint32_t)) {\
		copy_from_ia32_stack (&pcv.u, pcs.ptr, _size);\
	} else {\
		assert (_size == sizeof (uint64_t));\
		copy_from_ia32_stack (&pcv.lu, pcs.ptr, _size);\
	}\
	*psp32 += _size;\
	return (pcv.val);\
}

DEF_BTMIX_POP(char,	int8_t,		4)
DEF_BTMIX_POP(uchar,	uint8_t,	4)
DEF_BTMIX_POP(short,	int16_t,	4)
DEF_BTMIX_POP(ushort,	uint16_t,	4)
DEF_BTMIX_POP(int,	int32_t,	4)
DEF_BTMIX_POP(uint,	uint32_t,	4)
DEF_BTMIX_POP(long,	int32_t,	4)	/* ia32 longs are 32-bit */
DEF_BTMIX_POP(ulong,	uint32_t,	4)
DEF_BTMIX_POP(ptr,	uint32_t,	4)
DEF_BTMIX_POP(llong,	uint64_t,	8)
DEF_BTMIX_POP(ullong,	uint64_t,	8)
DEF_BTMIX_POP(float,	float,		4)
DEF_BTMIX_POP(double,	double,		8)


#endif /* _BTMIX_H_ */
