/* 
 * jni32_array.h
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Stanislav Polevic <stanislav.v.polevic@intel.com>
 * Copyright (C) 2005       Vasily Levchenko <vasily.v.levchenko@intel.com>
 * Copyright (C) 2005       Alexander Alexeev <alexander.n.alexeev@intel.com>
 * Copyright (C) 2005       Valery Ushakov <valery.e.ushakov@intel.com>
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
#include <jni.h>
#include <jni32.h>
#include <jni32_refmap.h>
#include <jni32_jnienv.h>
#include <jni32_args.h>
#include <jni32_ptrmap.h>
#include <jni32_mth_parse.h>
#include <jni32_refmap.h>
#include <jni32_upcall.h>
#include <jni32_trampolines.h>

#define DEFINE_NEW_ARRAY_TRAMPOLINE(__type, __name)\
DEFINE_JNIENV_IA64_TRAMPOLINE(__name)\
{\
	JNIEnv	*env;\
	__type res;\
	jsize len;\
	uint32_t sp32;\
\
	sp32 = btmix_get_upcall_argp(ec32);\
	assert(sp32 != 0);\
\
	env	= jni32_jnienv_demarshal(&sp32);\
	assert(env != NULL);\
	len	= btmix_pop_jint(&sp32);\
	res = (*env)->__name(env, len);\
\
	__SET_RETVAL_OBJECT_LOCAL(jobject, res);\
	return 0;\
}

#define DEFINE_GET_ARRAY_ELEMENTS_TRAMPOLINE(__type, __name)\
DEFINE_JNIENV_IA64_TRAMPOLINE(__name)\
{\
	JNIEnv	*env;\
	__type *buf64, *buf32;\
	jarray array;\
	jboolean *isCopy;\
	uint32_t sp32;\
	jsize length;\
\
	sp32 = btmix_get_upcall_argp(ec32);\
	assert(sp32 != 0);\
\
	env	= jni32_jnienv_demarshal(&sp32);\
	assert(env != NULL);\
	array	= jni32_demarshal_ref(&sp32);\
	isCopy = demarshal_ptr(&sp32);\
\
        if (array == NULL) {\
	        buf64 = (*env)->__name(env, NULL, isCopy);\
                __SET_RETVAL_PTR(__type, NULL);\
                return 0;\
	}\
\
	length = GET_ARRAY_LENGTH_ ## __type(env, array, __type);\
	assert(length >= 0);\
	buf64 = (*env)->__name(env, array, isCopy);\
	assert(buf64 != NULL);\
	buf32 = jni32_add_ptrmap(buf64, length);\
	assert(buf32 != NULL);\
\
	memcpy(buf32, buf64, length);\
	if (__PREDICT_FALSE(isCopy != NULL)) {\
		*isCopy = JNI_TRUE;\
	}\
\
	__SET_RETVAL_PTR(__type, buf32);\
	return 0;\
}

#define DEFINE_RELEASE_ARRAY_ELEMENTS_TRAMPOLINE(__type, __name)\
DEFINE_JNIENV_IA64_TRAMPOLINE(__name)\
{\
	JNIEnv	*env;\
	__type *elems64, *elems32;\
	jarray array;\
	jint mode, mode64;\
	uint32_t sp32;\
	jsize length;\
	int32_t free_elems32;\
\
	sp32 = btmix_get_upcall_argp(ec32);\
	assert(sp32 != 0);\
\
	env	= jni32_jnienv_demarshal(&sp32);\
	assert(env != NULL);\
	array	= jni32_demarshal_ref(&sp32);\
	elems32= demarshal_ptr(&sp32);\
	mode	= btmix_pop_jint(&sp32);\
	mode64 = mode == JNI_ABORT ? mode : 0;\
	free_elems32 = mode != JNI_COMMIT;\
	elems64= jni32_get_ptrmap(elems32);\
\
	if (elems64 == NULL) {\
                __SET_RETVAL_VOID(void, (*env)->__name(env, array, elems64, mode64));\
                return 0;\
        }\
\
	length = GET_ARRAY_LENGTH_ ## __type(env, array, __type);\
	assert(length >= 0);\
	if (mode != JNI_ABORT) {\
		memcpy(elems64, elems32, length);\
	}\
	jni32_del_ptrmap(elems32, free_elems32);\
\
	__SET_RETVAL_VOID(void, (*env)->__name(env, array, elems64, mode64));\
	return 0;\
}

#define DEFINE_ARRAY_REGION_TRAMPOLINE(__type, __name, __prefix)\
DEFINE_JNIENV_IA64_TRAMPOLINE(__name)\
{\
	JNIEnv	*env;\
	__type *buf;\
	__type##Array array;\
	/*CONCAT3(__type, Array, EMPTY) array;*/\
	jsize start, len;\
	uint32_t sp32;\
\
	sp32 = btmix_get_upcall_argp(ec32);\
	assert(sp32 != 0);\
\
	env	= jni32_jnienv_demarshal(&sp32);\
	assert(env != NULL);\
	array	= jni32_demarshal_ref(&sp32);\
	start	= btmix_pop_jint(&sp32);\
	len	= btmix_pop_jint(&sp32);\
	buf	= demarshal_ptr(&sp32);\
\
	__SET_RETVAL_VOID(void,\
	(*env)->CONCAT3(__prefix, __UPCASE_TYPE(__type), ArrayRegion)\
					(env, array, start, len, buf));\
	return 0;\
}

#define GET_ARRAY_LENGTH_PRIMITIVE(env, array, jtype)\
({\
	(*env)->GetArrayLength(env, array) * sizeof(jtype);\
})


#define GET_ARRAY_LENGTH_VOID(env, array, jtype)\
({\
	size_t acsize;\
	jsize size;\
	/* int32_t ptype; */ \
\
	/*ptype = get_array_component_jtype(env, array);*/\
	/*acsize = get_jtype_size(ptype);*/\
	acsize = g_java_callbacks.get_primitive_array_element_size(env, array);\
	size = (*env)->GetArrayLength(env, array);\
	size * acsize;\
 })


#define GET_ARRAY_LENGTH_void		GET_ARRAY_LENGTH_VOID
#define GET_ARRAY_LENGTH_jboolean	GET_ARRAY_LENGTH_PRIMITIVE
#define GET_ARRAY_LENGTH_jbyte		GET_ARRAY_LENGTH_PRIMITIVE
#define GET_ARRAY_LENGTH_jchar		GET_ARRAY_LENGTH_PRIMITIVE
#define GET_ARRAY_LENGTH_jshort		GET_ARRAY_LENGTH_PRIMITIVE
#define GET_ARRAY_LENGTH_jint		GET_ARRAY_LENGTH_PRIMITIVE
#define GET_ARRAY_LENGTH_jlong		GET_ARRAY_LENGTH_PRIMITIVE
#define GET_ARRAY_LENGTH_jfloat		GET_ARRAY_LENGTH_PRIMITIVE
#define GET_ARRAY_LENGTH_jdouble	GET_ARRAY_LENGTH_PRIMITIVE


DEFINE_JNIENV_IA64_TRAMPOLINE(GetArrayLength)
{
	JNIEnv	*env;
	jint	result;
	jintArray array;
	uint32_t sp32;

	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env	= jni32_jnienv_demarshal(&sp32);
	assert(env != NULL);
	array = jni32_demarshal_ref(&sp32);

	result = (*env)->GetArrayLength(env, array);

	__SET_RETVAL_PRIMITIVE(jint, result);
	return 0;
}

DEFINE_JNIENV_IA64_TRAMPOLINE(NewObjectArray)
{
	JNIEnv	*env;
	jsize	length;
	jclass	elementClass;
	jobject initialElement;
	jobjectArray result;
	uint32_t sp32;

	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env			= jni32_jnienv_demarshal(&sp32);
	assert(env != NULL);
	length		= btmix_pop_jint(&sp32);
	elementClass	= jni32_demarshal_ref(&sp32);
	initialElement = jni32_demarshal_ref(&sp32);

	result = (*env)->NewObjectArray(env, length, elementClass, initialElement);

	__SET_RETVAL_OBJECT_LOCAL(jobject, result);
	return 0;
}

DEFINE_JNIENV_IA64_TRAMPOLINE(GetObjectArrayElement)
{
	JNIEnv	*env;
	jobjectArray array;
	jsize	idx;
	jobject result;
	uint32_t sp32;

	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env	= jni32_jnienv_demarshal(&sp32);
	assert(env != NULL);
	array	= jni32_demarshal_ref(&sp32);
	idx	= btmix_pop_jint(&sp32);

	result = (*env)->GetObjectArrayElement(env, array, idx);

	__SET_RETVAL_OBJECT_LOCAL(jobject, result);
	return 0;
}

DEFINE_JNIENV_IA64_TRAMPOLINE(SetObjectArrayElement)
{
	JNIEnv	*env;
	jobjectArray array;
	jsize	idx;
	jobject value;
	uint32_t sp32;

	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env	= jni32_jnienv_demarshal(&sp32);
	assert(env != NULL);
	array	= jni32_demarshal_ref(&sp32);
	idx	= btmix_pop_jint(&sp32);
	value	= jni32_demarshal_ref(&sp32);

	__SET_RETVAL_VOID(void, (*env)->SetObjectArrayElement(env, array, idx, value));
	return 0;
}

/* NewXXXArray */
DEFINE_NEW_ARRAY_TRAMPOLINE(jbooleanArray, NewBooleanArray)
DEFINE_NEW_ARRAY_TRAMPOLINE(jbyteArray, NewByteArray)
DEFINE_NEW_ARRAY_TRAMPOLINE(jcharArray, NewCharArray)
DEFINE_NEW_ARRAY_TRAMPOLINE(jshortArray, NewShortArray)
DEFINE_NEW_ARRAY_TRAMPOLINE(jintArray, NewIntArray)
DEFINE_NEW_ARRAY_TRAMPOLINE(jlongArray, NewLongArray)
DEFINE_NEW_ARRAY_TRAMPOLINE(jfloatArray, NewFloatArray)
DEFINE_NEW_ARRAY_TRAMPOLINE(jdoubleArray, NewDoubleArray)

/* GetXXXArrayElements */
DEFINE_GET_ARRAY_ELEMENTS_TRAMPOLINE(jboolean, GetBooleanArrayElements)
DEFINE_GET_ARRAY_ELEMENTS_TRAMPOLINE(jbyte, GetByteArrayElements)
DEFINE_GET_ARRAY_ELEMENTS_TRAMPOLINE(jchar, GetCharArrayElements)
DEFINE_GET_ARRAY_ELEMENTS_TRAMPOLINE(jshort, GetShortArrayElements)
DEFINE_GET_ARRAY_ELEMENTS_TRAMPOLINE(jint, GetIntArrayElements)
DEFINE_GET_ARRAY_ELEMENTS_TRAMPOLINE(jlong, GetLongArrayElements)
DEFINE_GET_ARRAY_ELEMENTS_TRAMPOLINE(jfloat, GetFloatArrayElements)
DEFINE_GET_ARRAY_ELEMENTS_TRAMPOLINE(jdouble, GetDoubleArrayElements)

/* ReleaseXXXArrayElements */
DEFINE_RELEASE_ARRAY_ELEMENTS_TRAMPOLINE(jboolean, ReleaseBooleanArrayElements)
DEFINE_RELEASE_ARRAY_ELEMENTS_TRAMPOLINE(jbyte, ReleaseByteArrayElements)
DEFINE_RELEASE_ARRAY_ELEMENTS_TRAMPOLINE(jchar, ReleaseCharArrayElements)
DEFINE_RELEASE_ARRAY_ELEMENTS_TRAMPOLINE(jshort, ReleaseShortArrayElements)
DEFINE_RELEASE_ARRAY_ELEMENTS_TRAMPOLINE(jint, ReleaseIntArrayElements)
DEFINE_RELEASE_ARRAY_ELEMENTS_TRAMPOLINE(jlong, ReleaseLongArrayElements)
DEFINE_RELEASE_ARRAY_ELEMENTS_TRAMPOLINE(jfloat, ReleaseFloatArrayElements)
DEFINE_RELEASE_ARRAY_ELEMENTS_TRAMPOLINE(jdouble, ReleaseDoubleArrayElements)

/* GetXXXArrayRegion */
DEFINE_ARRAY_REGION_TRAMPOLINE(jboolean, GetBooleanArrayRegion, Get)
DEFINE_ARRAY_REGION_TRAMPOLINE(jbyte, GetByteArrayRegion, Get)
DEFINE_ARRAY_REGION_TRAMPOLINE(jchar, GetCharArrayRegion, Get)
DEFINE_ARRAY_REGION_TRAMPOLINE(jshort, GetShortArrayRegion, Get)
DEFINE_ARRAY_REGION_TRAMPOLINE(jint, GetIntArrayRegion, Get)
DEFINE_ARRAY_REGION_TRAMPOLINE(jlong, GetLongArrayRegion, Get)
DEFINE_ARRAY_REGION_TRAMPOLINE(jfloat, GetFloatArrayRegion, Get)
DEFINE_ARRAY_REGION_TRAMPOLINE(jdouble, GetDoubleArrayRegion, Get)

/* SetXXXArrayRegion */
DEFINE_ARRAY_REGION_TRAMPOLINE(jboolean, SetBooleanArrayRegion, Set)
DEFINE_ARRAY_REGION_TRAMPOLINE(jbyte, SetByteArrayRegion, Set)
DEFINE_ARRAY_REGION_TRAMPOLINE(jchar, SetCharArrayRegion, Set)
DEFINE_ARRAY_REGION_TRAMPOLINE(jshort, SetShortArrayRegion, Set)
DEFINE_ARRAY_REGION_TRAMPOLINE(jint, SetIntArrayRegion, Set)
DEFINE_ARRAY_REGION_TRAMPOLINE(jlong, SetLongArrayRegion, Set)
DEFINE_ARRAY_REGION_TRAMPOLINE(jfloat, SetFloatArrayRegion, Set)
DEFINE_ARRAY_REGION_TRAMPOLINE(jdouble, SetDoubleArrayRegion, Set)

/* GetPrimitiveArrayCritical */
DEFINE_GET_ARRAY_ELEMENTS_TRAMPOLINE(void, GetPrimitiveArrayCritical)

/* ReleasePrimitiveArrayCritical */
DEFINE_RELEASE_ARRAY_ELEMENTS_TRAMPOLINE(void, ReleasePrimitiveArrayCritical)

