/* 
 * jni32_exception.c
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Stanislav Polevic <stanislav.v.polevic@intel.com>
 * Copyright (C) 2005       Vasily Levchenko <vasily.v.levchenko@intel.com>
 * Copyright (C) 2005       Alexander Alexeev <alexander.n.alexeev@intel.com>
 * Copyright (C) 2005       Valery Ushakov <valery.e.ushakov@intel.com>
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#include <jni32_jnienv.h>
#include <jni32_mth_parse.h>
#include <jni32_refmap.h>
#include <jni32_ptrmap.h>
#include <jni32_trampolines.h>
#include <jni32_upcall.h>

DEFINE_JNIENV_IA64_TRAMPOLINE(Throw)
{
	JNIEnv	*env;
	jthrowable exc;
	uint32_t sp32;
	jint result;

	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env = jni32_jnienv_demarshal(&sp32);
	exc = jni32_demarshal_ref(&sp32);

	if (exc == NULL) {
	        __SET_RETVAL_PRIMITIVE(jint, -1);
		return 0;
	}

	result = (*env)->Throw(env, exc);

	__SET_RETVAL_PRIMITIVE(jint, result);
	return 0;
}


DEFINE_JNIENV_IA64_TRAMPOLINE(ThrowNew)
{
	JNIEnv	*env;
	jclass	clazz;
	char* msg;
	uint32_t sp32;
	jint result;

	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env = jni32_jnienv_demarshal(&sp32);
	assert(env != NULL);
	clazz = jni32_demarshal_ref(&sp32);
	msg	= demarshal_ptr(&sp32);

	result = (*env)->ThrowNew(env, clazz, msg);

	__SET_RETVAL_PRIMITIVE(jint, result);
	return 0;
}


DEFINE_JNIENV_IA64_TRAMPOLINE(ExceptionOccurred)
{
	JNIEnv	*env;
	uint32_t sp32;
	jthrowable exc;

	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env = jni32_jnienv_demarshal(&sp32);
	assert(env != NULL);

	exc = (*env)->ExceptionOccurred(env);

	__SET_RETVAL_OBJECT_LOCAL(jobject, exc);
	return 0;
}


DEFINE_JNIENV_IA64_TRAMPOLINE(ExceptionDescribe)
{
	JNIEnv	*env;
	uint32_t sp32;

	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env = jni32_jnienv_demarshal(&sp32);
	assert(env != NULL);

	__SET_RETVAL_VOID(void, (*env)->ExceptionDescribe(env));
	return 0;
}


DEFINE_JNIENV_IA64_TRAMPOLINE(ExceptionClear)
{
	JNIEnv	*env;
	uint32_t sp32;

	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env = jni32_jnienv_demarshal(&sp32);
	assert(env != NULL);

	__SET_RETVAL_VOID(void, (*env)->ExceptionClear(env));
	return 0;
}


DEFINE_JNIENV_IA64_TRAMPOLINE(FatalError)
{
	JNIEnv	*env;
	char* msg;
	uint32_t sp32;

	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env = jni32_jnienv_demarshal(&sp32);
	assert(env != NULL);
	msg	= demarshal_ptr(&sp32);

	__SET_RETVAL_VOID(void, (*env)->FatalError(env, msg));
	return 0;
}


DEFINE_JNIENV_IA64_TRAMPOLINE(ExceptionCheck)
{
	JNIEnv	*env;
	uint32_t sp32;
	jboolean result;

	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env = jni32_jnienv_demarshal(&sp32);
	assert(env != NULL);

	result = (*env)->ExceptionCheck(env);

	__SET_RETVAL_PRIMITIVE(jboolean, result);
	return 0;
}

