/* 
 * jni32_field.c
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Stanislav Polevic <stanislav.v.polevic@intel.com>
 * Copyright (C) 2005       Vasily Levchenko <vasily.v.levchenko@intel.com>
 * Copyright (C) 2005       Alexander Alexeev <alexander.n.alexeev@intel.com>
 * Copyright (C) 2005       Valery Ushakov <valery.e.ushakov@intel.com>
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#include <jni32_jnienv.h>
#include <jni32_idmap.h>
#include <jni32_refmap.h>
#include <jni32_ptrmap.h>
#include <jni32_trampolines.h>
#include <jni32_upcall.h>

#define DEFINE_GET_FIELD_ID_TRAMPOLINE(_kind)\
DEFINE_JNIENV_IA64_TRAMPOLINE(Get ## _kind ## FieldID)\
{\
	JNIEnv	*env;\
	jclass	clazz;\
	char	*name;\
	char	*sig;\
	jfieldID fid;\
	uint32_t sp32;\
\
	sp32 = btmix_get_upcall_argp(ec32);\
\
	env = jni32_jnienv_demarshal(&sp32);\
	assert(env != NULL);\
\
	clazz = jni32_demarshal_ref(&sp32);\
	name	= demarshal_ptr(&sp32);\
	sig	= demarshal_ptr(&sp32);\
\
\
	fid = (*env)->Get ## _kind ## FieldID(env, clazz, name, sig);\
\
	__SET_RETVAL_FID(jfieldID, fid);\
	return 0;\
}

#define DEFINE_GET_FIELD_TRAMPOLINE(_jtype, _kind)\
DEFINE_JNIENV_IA64_TRAMPOLINE(CONCAT3(Get ## _kind, __UPCASE_TYPE(_jtype), Field))\
{\
	JNIEnv	*env;\
	jobject self;\
	_jtype res;\
	uint32_t sp32;\
	jfieldID fid;\
\
	sp32 = btmix_get_upcall_argp(ec32);\
\
	env = jni32_jnienv_demarshal(&sp32);\
	assert(env != NULL);\
\
	self	= jni32_demarshal_ref(&sp32);\
	fid	= demarshal_fieldID(&sp32);\
\
\
	res = (*env)->CONCAT3(Get ## _kind, __UPCASE_TYPE(_jtype), Field)\
				(env, self, fid);\
\
	__SET_RETVAL_ ## _jtype (_jtype, res);\
	return 0;\
}


#define DEFINE_SET_FIELD_TRAMPOLINE(_jtype, _kind)\
DEFINE_JNIENV_IA64_TRAMPOLINE(CONCAT3(Set ## _kind,\
			__UPCASE_TYPE(_jtype), Field))\
{\
	JNIEnv	*env;\
	jobject self;\
	_jtype value;\
	uint32_t sp32;\
	jfieldID fid;\
\
	sp32 = btmix_get_upcall_argp(ec32);\
\
	env = jni32_jnienv_demarshal(&sp32);\
	assert(env != NULL);\
\
	self	= jni32_demarshal_ref(&sp32);\
	fid	= demarshal_fieldID(&sp32);\
	__POP_VALUE_ ## _jtype (sp32, value, _jtype);\
\
	__SET_RETVAL_void(void,\
		(*env)->CONCAT3(Set ## _kind, __UPCASE_TYPE(_jtype), Field)\
				(env, self, fid, value));\
	return 0;\
}

#define DEFINE_FIELD_ACCESSORS(_jtype)\
	DEFINE_GET_FIELD_TRAMPOLINE(_jtype, /* */)\
	DEFINE_GET_FIELD_TRAMPOLINE(_jtype, Static)\
	DEFINE_SET_FIELD_TRAMPOLINE(_jtype, /* */)\
	DEFINE_SET_FIELD_TRAMPOLINE(_jtype, Static)

#define DEFINE_ALL_FIELD_METHODS()\
	DEFINE_GET_FIELD_ID_TRAMPOLINE(/* */)\
	DEFINE_GET_FIELD_ID_TRAMPOLINE(Static)\
	DEFINE_FIELD_ACCESSORS(jobject)\
	DEFINE_FIELD_ACCESSORS(jboolean)\
	DEFINE_FIELD_ACCESSORS(jbyte)\
	DEFINE_FIELD_ACCESSORS(jchar)\
	DEFINE_FIELD_ACCESSORS(jshort)\
	DEFINE_FIELD_ACCESSORS(jint)\
	DEFINE_FIELD_ACCESSORS(jlong)\
	DEFINE_FIELD_ACCESSORS(jfloat)\
	DEFINE_FIELD_ACCESSORS(jdouble)

DEFINE_ALL_FIELD_METHODS()

