/* 
 * jni32_javavm.c
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Stanislav Polevic <stanislav.v.polevic@intel.com>
 * Copyright (C) 2005       Vasily Levchenko <vasily.v.levchenko@intel.com>
 * Copyright (C) 2005       Alexander Alexeev <alexander.n.alexeev@intel.com>
 * Copyright (C) 2005       Valery Ushakov <valery.e.ushakov@intel.com>
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/


#include <stdlib.h>
#include <jni32_javavm.h>
#include <jni32_jnienv.h>

#include <mme_internal.h>
#include <jni32_diag.h>
#include <jni32_ptrmap.h>
#include <jni32_refmap.h>
#include <btmix.h>

/* define array of trampolines */
#define JAVAVMLAST(_idx) \
jni32_trampoline_t HIDDEN jni32_javavm_trampolines[_idx]

#include <jvmfns.h>

= {

#define JAVAVMFN(_idx, _type, _name, _args) \
	[_idx] = (void *)JAVAVM_IA64_TRAMPOLINE(_name),

#include <jvmfns.h>
}; /* jni32_trampolines definition */

/* type def for attach functions */
struct JavaVMAttachArgs32 {
	jint version;
	ptr32_t name;
	ptr32_t group;
};

/* JNIEnv interface function table in the IA32 world */
uint32_t HIDDEN g_javavm32_fns;

static int32_t attach_thread (ia32el_context32 * ec32, bool is_daemon);

/*****************************************************************************/
/*****************************************************************************/
int64_t HIDDEN
jni32_javavm_upcall(ia32el_context32 *ec32)
{
	int64_t result;

	
	result = jni32_upcall(ec32, jni32_javavm_trampolines, MAX_JAVAVM_IDX);

	
	return (result);
}
/*
 * Trampolines implementation
 */

/*****************************************************************************/
/*****************************************************************************/
DEFINE_JAVAVM_IA64_TRAMPOLINE(DestroyJavaVM)
{
	int32_t retval;
	uint32_t sp32;
	JavaVM *vm;

	assert(ec32 != NULL);
	DBPRINT (1, "esp=%x\n", ec32->esp);
	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	/* get the 32 bit params */
	vm = jni32_javavm_demarshal(&sp32);

	if (vm == NULL) {
		__SET_RETVAL_PRIMITIVE(jint, JNI_ERR);
		return 0;
	}

	retval = (*vm)->DestroyJavaVM (vm);
	if (retval == JNI_ERR) {
		DBPRINT (true, "DestroyJavaVM failed\n");
	}
	__SET_RETVAL_PRIMITIVE(jint, retval);
	return 0;
}


/*****************************************************************************/
/*****************************************************************************/
DEFINE_JAVAVM_IA64_TRAMPOLINE(AttachCurrentThread)
{
	return attach_thread (ec32, false);
}
/*****************************************************************************/
/*****************************************************************************/
static int32_t attach_thread (ia32el_context32 * ec32, bool is_daemon)
{
	uint32_t sp32;
	JavaVM *vm;
	ptr32_t args32p;
	struct JavaVMAttachArgs32 * args32;
	JavaVMAttachArgs args64, * args64p = NULL;
	int32_t retval;
	uint32_t * env32p;
	union {
		void * pv;
		JNIEnv * env64;
	} pc;

	struct JNIEnv32to64 *env32to64;


	assert(ec32 != NULL);
	DBPRINT (1, "esp=%x\n", ec32->esp);
	DBPRINT (1, "ebp=%x\n", ec32->ebp);
	DBPRINT (1, "is_daemon=%d\n", is_daemon);
	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	/* get the 32 bit params */
	vm = jni32_javavm_demarshal(&sp32);

	if (vm == NULL) {
		__SET_RETVAL_PRIMITIVE(jint, JNI_ERR);
		return 0;
	}

	env32p = demarshal_ptr(&sp32);
	args32p = btmix_pop_ptr(&sp32);

	/* convert the args parameter */
	if (args32p != 0) {
		args32 = I2P (args32p);
		args64.version = args32->version;
		args64.name = I2P (args32->name);
		if (args32->group != 0) {
			args64.group = jni32_get_ref (args32->group);
			if (args64.group == NULL) {
				DBPRINT (true, "cannot find group\n");
				MME_THROW_EXCEPTION ("cannot find group");
				return -1;
			}
		} else {
		        args64.group = NULL;
		}
		args64p = &args64;
	}

	/* call 64 bit upcall */

	if (env32p == NULL) {
	        if (is_daemon) {
		        retval = (*vm)->AttachCurrentThreadAsDaemon (vm, NULL, args64p);
		} else {
		        retval = (*vm)->AttachCurrentThread (vm, NULL, args64p);
		}
		goto out;
	}

	if (is_daemon) {
		retval = (*vm)->AttachCurrentThreadAsDaemon (vm, &pc.pv, args64p);
	} else {
		retval = (*vm)->AttachCurrentThread (vm, &pc.pv, args64p);
	}
	if (retval == JNI_ERR) {
		DBPRINT (true, "AttachCurrentThread failed\n");
		goto out;
	}

	/* marshal the jni-env */

	env32to64 = jni32_jnienv_get_marshalled();

	*env32p = jni32_jnienv_marshal (NULL, pc.env64, false);
	if (*env32p == 0) {
		retval = -1;
		DBPRINT (true, "jni32_jnienv_marshal failed\n");
		MME_THROW_EXCEPTION ("jni32_jnienv_marshal failed");
		goto detach;
	}

	if (env32to64 != NULL) {
	        goto out;
	}

	/* init the new thread */
	retval = init_thread (true);
	if (retval != 0) {
		DBPRINT (true, "init_thread failed\n");
		MME_THROW_EXCEPTION ("init_thread failed");
		goto detach;
	}

		
out:
	__SET_RETVAL_PRIMITIVE(jint, retval);
	return 0;

detach:
	{
		int32_t r;
		r = (*vm)->DetachCurrentThread (vm);
		if (r == JNI_ERR) {
			DBPRINT (true, "DetachCurrentThread failed\n");
			MME_THROW_EXCEPTION ("DetachCurrentThread failed");
		}
	}
	goto out;
}


/*****************************************************************************/
/*****************************************************************************/
DEFINE_JAVAVM_IA64_TRAMPOLINE(DetachCurrentThread)
{
	uint32_t sp32;
	JavaVM *vm;
	int32_t retval = 0;

	assert(ec32 != NULL);
	DBPRINT (1, "esp=%x, ebx=%x ebp=%x\n", ec32->esp, ec32->ebx, ec32->ebp);
	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	/* get the 64 bit jvm param */
	vm = jni32_javavm_demarshal(&sp32);

	if (vm == NULL) {
		__SET_RETVAL_PRIMITIVE(jint, JNI_ERR);
		return 0;
	}

	/* detach from jvm */
	retval = (*vm)->DetachCurrentThread (vm);
	if (retval == JNI_ERR) {
		DBPRINT (true, "DetachCurrentThread failed\n");
		goto out;
	}

	retval = jni32_uninit_local_refmap ();
	if (retval != 0) {
		DBPRINT (true, "jni32_uninit_local_refmap failed\n");
		MME_THROW_EXCEPTION ("jni32_uninit_local_refmap failed");
	}
	retval = jni32_jnienv_destroy ();
	if (retval != 0) {
		DBPRINT (true, "jni32_jnienv_destroy failed\n");
		MME_THROW_EXCEPTION ("jni32_jnienv_destroy failed");
	}

#if 0
	/* the implementation of internal_thread_term has changed. */
	/* no more access to ia32 context is allowed after this call. */
	/* so comment it out */
	/* cleanup */
	r = internal_thread_term ();
	if (r != 0) {
		DBPRINT (true, "internal_thread_term failed\n");
		MME_THROW_EXCEPTION ("internal_thread_term failed");
	}
#endif

out:
	__SET_RETVAL_PRIMITIVE(jint, retval);
	return 0;
}

/*****************************************************************************/
/*****************************************************************************/
DEFINE_JAVAVM_IA64_TRAMPOLINE(GetEnv)
{
	JavaVM *vm;
	struct JNIEnv32to64 *penv32;
	uint32_t *env32;
	jint version;
	uint32_t sp32;
	jint result;
	union {
		JNIEnv *env;
		void * ptr;
	} pc;

	assert(ec32 != NULL);
	DBPRINT (1, "esp=%x\n", ec32->esp);
	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	vm = jni32_javavm_demarshal(&sp32);

	if (vm == NULL) {
		__SET_RETVAL_PRIMITIVE(jint, JNI_ERR);
		return 0;
	}

	env32 = demarshal_ptr(&sp32);
	assert (env32 != NULL);
	version = btmix_pop_jint(&sp32);

	result = (*vm)->GetEnv(vm, &pc.ptr, version);

	if (__PREDICT_TRUE(result == JNI_OK)) {
		penv32 = jni32_jnienv_get_marshalled();
		if (__PREDICT_FALSE(penv32 == 0)) {
			result = JNI_EDETACHED;
		} else {
			if (__PREDICT_FALSE(penv32->env64 != pc.env)) {
				MME_THROW_EXCEPTION("corrupted env32 structure");
				result = JNI_ERR;
			}
		}
	} else {
	        penv32 = NULL;
	}
	*env32 = TRUNCATE_PTR(penv32);

	DBPRINT (1, "result=%x env32=%p\n", result, penv32);

	__SET_RETVAL_PRIMITIVE(jint, result);

	return 0;
}


/*****************************************************************************/
/*****************************************************************************/
DEFINE_JAVAVM_IA64_TRAMPOLINE(AttachCurrentThreadAsDaemon)
{
	return attach_thread (ec32, true);
}

