/* 
 * jni32_javavm.h
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Stanislav Polevic <stanislav.v.polevic@intel.com>
 * Copyright (C) 2005       Vasily Levchenko <vasily.v.levchenko@intel.com>
 * Copyright (C) 2005       Alexander Alexeev <alexander.n.alexeev@intel.com>
 * Copyright (C) 2005       Valery Ushakov <valery.e.ushakov@intel.com>
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
#ifndef _JNI32_JAVAVM_H_
#define _JNI32_JAVAVM_H_

#include <stdint.h>

#include <btmix.h>
#include <jni.h>

#include <ia32el_mme_api.h>
#include <jni32_ptrmap.h>
#include <jni32_stack.h>
#include <jni32_upcall.h>

/* btlib upcall handler for IA32 JavaVM calls */
extern long jni32_javavm_upcall(ia32el_context32 *);

/* JavaVM interface function table in the IA32 world */
extern uint32_t g_javavm32_fns;

struct JavaVM32to64 {
	uint32_t fns32;
	JavaVM	*vm64;
};

static inline struct JavaVM32to64 *
jni32_javavm_marshal(JavaVM *vm)
{
	struct JavaVM32to64 *vm32;

	assert(vm != NULL);

	vm32 = jni32_get_ptrmap(vm);

	if (__PREDICT_FALSE(vm32 == NULL)) {
		vm32 = jni32_add_ptrmap(vm, sizeof(struct JavaVM32to64));
		assert(vm32 != NULL);
		vm32->fns32 = g_javavm32_fns;
		vm32->vm64 = vm;
	}

	return vm32;
}



static inline JavaVM *
jni32_javavm_demarshal(uint32_t *psp32)
{
	uint32_t vm32 = btmix_pop_jobject(psp32);
	JavaVM *vm64;
	struct JavaVM32to64 * envs;


	if (__PREDICT_FALSE(vm32 == 0)) {
		vm64 = NULL;
	} else {
		envs = I2P (vm32);
		vm64 = envs->vm64;
		assert(vm64 != NULL);
	}

	return vm64;
}

#define JAVAVM_IA64_TRAMPOLINE(_name) _Jni64_##_name##_trampoline

#define DECLARE_JAVAVM_IA64_TRAMPOLINE(_name)								\
extern int64_t JAVAVM_IA64_TRAMPOLINE(_name)(ia32el_context32 *)			\
	__attribute__ ((visibility ("hidden")))

#define DEFINE_JAVAVM_IA64_TRAMPOLINE(_name)									\
int64_t																			\
JAVAVM_IA64_TRAMPOLINE(_name)(ia32el_context32 * ec32 UNUSED)

extern int64_t (*jni32_javavm_trampolines[])(ia32el_context32 *);

/* declare trampolines */
#define JAVAVMFN(_idx, _type, _name, _args)									\
	DECLARE_JAVAVM_IA64_TRAMPOLINE(_name);

#include "jvmfns.h"

#endif /* _JNI32_JAVAVM_H_ */
