/* 
 * jni32_jnienv.h
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Stanislav Polevic <stanislav.v.polevic@intel.com>
 * Copyright (C) 2005       Vasily Levchenko <vasily.v.levchenko@intel.com>
 * Copyright (C) 2005       Alexander Alexeev <alexander.n.alexeev@intel.com>
 * Copyright (C) 2005       Valery Ushakov <valery.e.ushakov@intel.com>
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
#ifndef _JNI32_JNIENV_H_
#define _JNI32_JNIENV_H_

#include <jni.h>
#include <mme_internal.h>

#include <btmix.h>
#include <jni32_stack.h>

struct JNIEnv32to64 {
	/*
	* MUST BE FIRST!!!
	* Pointer to jnienv32_fns
	*/
	uint32_t fns32;

	/* real 64 bit env for this thread */
	JNIEnv *env64;

	/* malloc info */
	bool is_malloc;
	ptr32_t malloc_addr;
};

/* per-thread env struct */
extern MME_THREAD_VAR struct JNIEnv32to64 * t_env32to64;

/*****************************************************************************/
/*****************************************************************************/
static inline struct JNIEnv32to64 *
jni32_jnienv_get_marshalled(void)
{
	return t_env32to64;
}

/*****************************************************************************/
/*****************************************************************************/
static inline JNIEnv *
jni32_jnienv_demarshal(ptr32_t * psp32)
{
	ptr32_t env32 = btmix_pop_jobject (psp32);
	JNIEnv *env64;
	struct JNIEnv32to64 * envs;

	assert(env32 != 0);
	if (env32 == 0) {
		env64 = NULL;
	} else {
		envs = (struct JNIEnv32to64*)I2P (env32);
		env64 = envs->env64;
	}
	
	return env64;
};

/*****************************************************************************/
/*****************************************************************************/
static inline JNIEnv *
jni32_jnienv_get_current(void)
{
	struct JNIEnv32to64 *env32to64;
	JNIEnv *env;

	env32to64 = jni32_jnienv_get_marshalled();
	/*
	* Special case when we do not have marshalled env yet (in JNI_OnLoad)
	*/
	if (env32to64 == NULL) {
		env = NULL;
	}
	else {
		assert(env32to64->env64 != NULL);
		env = env32to64->env64;
	}
	
	return env;
}
extern int64_t HIDDEN (*jni32_jnienv_trampolines[])(ia32el_context32 *);
extern uint32_t HIDDEN jni32_jnienv_marshal(ptr32_t * esp, JNIEnv *env, bool is_on_stack);
extern int32_t HIDDEN jni32_jnienv_destroy (void);

#endif /* _JNI32_JNIENV_H_ */
