/* 
 * jni32_jnivm.c
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Stanislav Polevic <stanislav.v.polevic@intel.com>
 * Copyright (C) 2005       Vasily Levchenko <vasily.v.levchenko@intel.com>
 * Copyright (C) 2005       Alexander Alexeev <alexander.n.alexeev@intel.com>
 * Copyright (C) 2005       Valery Ushakov <valery.e.ushakov@intel.com>
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#include <alloca.h>
#include <assert.h>
#include <jni.h>

#include <jni32_diag.h>
#include <jni32_jnivm.h>
#include <jni32_javavm.h>
#include <jni32_jnienv.h>
#include <ia32el_mme_api.h>
#include <mme_internal.h>


/* define array of trampolines for JavaVM methods */
#define JNIVMLAST(_idx)				\
jni32_trampoline_t jni32_jnivm_trampolines[_idx]

#include <jnivmfns.h>
= {
#define JNIVMFN(_idx, _type, _name, _args)		\
	[_idx] = (void *)JNIVM_IA64_TRAMPOLINE(_name),

#include <jnivmfns.h>
}; /* end of jni32_jnivm_trampolines initializer */

/* JavaVM interface function table in the IA32 world */
uint32_t jnivm32_fns;

int64_t
jni32_jnivm_upcall(ia32el_context32 *ec32)
{
	int64_t result;

	result = jni32_upcall(ec32, jni32_jnivm_trampolines, MAX_JNIVM_IDX);

	return (result);
}


/*
 * Trampolines implementation
 */

DEFINE_JNIVM_IA64_TRAMPOLINE(JNI_GetDefaultJavaVMInitArgs)
{
	void *args;
	uint32_t ap32;
	jint result;

	assert(ec32 != NULL);

	ap32	= btmix_get_upcall_argp(ec32);
	assert(ap32 != 0);

	args = demarshal_ptr(&ap32);

	result = JNI_GetDefaultJavaVMInitArgs(args);

	__SET_RETVAL_PRIMITIVE(jint, result);

	return 0;
}


DEFINE_JNIVM_IA64_TRAMPOLINE(JNI_CreateJavaVM)
{
	jint result;
	JavaVM *vm;
	union {
		void * vp;
		JNIEnv *env;
	} pc;
	uint32_t *vm32;
	uint32_t *env32;
	void *args;
	uint32_t ap32;

	ap32	= btmix_get_upcall_argp(ec32);
	assert(ap32 != 0);

	vm32 = demarshal_ptr(&ap32);
	env32 = demarshal_ptr(&ap32);
	args = demarshal_ptr(&ap32);

	if ((vm32 == NULL) && (env32 == NULL)) {
	        result = JNI_CreateJavaVM(NULL, NULL, args);
	} else if (vm32 == NULL) {
	        result = JNI_CreateJavaVM(NULL, &pc.vp, args);
	} else if (env32 == NULL) {
	        result = JNI_CreateJavaVM(&vm, NULL, args);
	} else {
	        result = JNI_CreateJavaVM(&vm, &pc.vp, args);
	}

	if (result == JNI_OK) {
	        if (vm32) {
		        *vm32 = TRUNCATE_PTR (jni32_javavm_marshal(vm));
		}
		if (env32) {
		        *env32 = jni32_jnienv_marshal(NULL, pc.env, false);
		}
	}

	__SET_RETVAL_PRIMITIVE(jint, result);

	return 0;
}


DEFINE_JNIVM_IA64_TRAMPOLINE(JNI_GetCreatedJavaVMs)
{
	JavaVM **vmBuf;
	uint32_t *vmBuf32;
	jsize bufLen;
	jsize *nVms;
	uint32_t ap32;
	jint result;

	assert(ec32 != NULL);

	ap32	= btmix_get_upcall_argp(ec32);
	assert(ap32 != 0);

	vmBuf32	= demarshal_ptr(&ap32);
	bufLen	= btmix_pop_jint(&ap32);
	nVms	= demarshal_ptr(&ap32);

	if (vmBuf32 == NULL) {
	        result = JNI_GetCreatedJavaVMs(NULL, bufLen, nVms);
		__SET_RETVAL_PRIMITIVE(jint, result);
		return 0;
	}

	vmBuf = alloca(bufLen);

	result = JNI_GetCreatedJavaVMs(vmBuf, bufLen, nVms);
	if ((result == JNI_OK) && (nVms)) {
		jsize i;
		for (i = 0; i < *nVms; ++i) {
			uint32_t vm32 = TRUNCATE_PTR (jni32_javavm_marshal(vmBuf[i]));
			vmBuf32[i] = vm32;
		}
	}

	__SET_RETVAL_PRIMITIVE(jint, result);

	return 0;
}

