/* 
 * jni32_jnivm.h
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Stanislav Polevic <stanislav.v.polevic@intel.com>
 * Copyright (C) 2005       Vasily Levchenko <vasily.v.levchenko@intel.com>
 * Copyright (C) 2005       Alexander Alexeev <alexander.n.alexeev@intel.com>
 * Copyright (C) 2005       Valery Ushakov <valery.e.ushakov@intel.com>
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
#ifndef _JNI32_JNIVM_H_
#define _JNI32_JNIVM_H_

#include <stdint.h>

#include <btmix.h>
#include <jni.h>

#include <jni32_ptrmap.h>
#include <jni32_stack.h>
#include <jni32_upcall.h>

/* btlib upcall handler for IA32 JavaVM calls */
extern int64_t jni32_jnivm_upcall(ia32el_context32 *ec32);

/* JavaVM interface function table in the IA32 world */
extern uint32_t jnivm32_fns;

/*
 * Common naming pattern for functions that implement trampolines for
 * specific JavaVM methods.
 */
#define JNIVM_IA64_TRAMPOLINE(_name) _JNIVM64_ ## _name ## _trampoline

/*
 * All trampolines have the same signature.  jni32_jnivm_upcall()
 * calls them with the current IA32 context as the argument.  The
 * return value of the trampoline is the value to put in %EAX - btlib
 * upcall code will propagate the return value of the trampoline to
 * there.  Any other registers that must be changed on return must be
 * changed by the trampoline itself.
 *
 * Hint: btmix provides a set of btmix_set_retval_* functions that
 * takes care of the details of IA32 calling conventions.
 */
#define DECLARE_JNIVM_IA64_TRAMPOLINE(_name)				\
extern int64_t JNIVM_IA64_TRAMPOLINE(_name)(ia32el_context32 *)	\
    __attribute__ ((visibility ("hidden")))

/* mass-declare trampolines */
#define JNIVMFN(_idx, _type, _name, _args)	\
    DECLARE_JNIVM_IA64_TRAMPOLINE(_name);

#include <jnivmfns.h>


/* 
 * Syntactic sugar to start the definition of the trampoline.
 * E.g., a trampoline for GetFoo might look something like:
 *
 *   DEFINE_JNIVM_IA64_TRAMPOLINE(Foo)
 *   {
 *       // ...
 *   }
 */
#define DEFINE_JNIVM_IA64_TRAMPOLINE(_name)			\
int64_t								\
JNIVM_IA64_TRAMPOLINE(_name)(ia32el_context32 *ec32)

#endif /* _JNI32_JNIVM_H_ */
