/* 
 * jni32_map.c
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Stanislav Polevic <stanislav.v.polevic@intel.com>
 * Copyright (C) 2005       Vasily Levchenko <vasily.v.levchenko@intel.com>
 * Copyright (C) 2005       Alexander Alexeev <alexander.n.alexeev@intel.com>
 * Copyright (C) 2005       Valery Ushakov <valery.e.ushakov@intel.com>
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
#include <stdlib.h>
#include <assert.h>
#include <mme_internal.h>
#include <jni32_map.h>

//XXX: add check that map index < MAX(uint32_t)

/*
 * Create map.
 */
int32_t HIDDEN
jni32_map_create(struct jni32_map *map, int32_t init_size, int32_t inc_size)
{

	int32_t free_tail = 0;

	map->data = NULL;
	map->size = 0; 
	map->free = -1;
	map->increment_size = inc_size;

	free_tail = jni32_map_grow(map, init_size);
	if (free_tail < 0) {
		return -1; // XXX: add good error code
	}

	jni32_map_freelist_add(map, free_tail);

	return 0; // XXX: add good result code
}


void HIDDEN
jni32_map_destroy(struct jni32_map *map)
{
	assert(map != NULL);

	if (map->data == NULL) {
		return;
	}

	free(map->data);
	map->data = NULL;
	map->size = 0; 
	map->free = -1;
	map->increment_size = 0;

	return;
}


int32_t HIDDEN
jni32_map_add(struct jni32_map *map, void *value64, int32_t *result_value32)
{
	int32_t free_tail = 0;

	if (map->free < 0) {

		free_tail = jni32_map_grow(map, map->increment_size);

		if (free_tail < 0) {
			return -1; //XXX: add errror code
		}

		jni32_map_freelist_add(map, free_tail);
	}

	assert(map->free >= 0);
	assert(map->free < map->size);

	/* get free slot from the head of the free list */
	*result_value32 = map->free;

	map->free = map->data[*result_value32].next_free;
	map->data[*result_value32].value = value64;

	return 0;
}

/*
 * Delete value32 from the map.
 */
HIDDEN void *
jni32_map_del(struct jni32_map *map, int32_t value32)
{
	void *value64 = 0;
	if (value32 >= map->size) {
		return NULL;
	}

	value64 = map->data[value32].value;
	map->data[value32].value = NULL;

	/* prepend to the free list */
	map->data[value32].next_free = map->free;
	map->free = value32;

	return value64;
}

HIDDEN void *
jni32_map_get(struct jni32_map *map, int32_t value32)
{
	if (value32 >= map->size) {
		return NULL;
	}

	return map->data[value32].value;
}



/*
 * Add more elements to the map.
 * Return the index of the first new element.
 * Does NOT touch free list.
 */
int32_t HIDDEN
jni32_map_grow(struct jni32_map *map, size_t more)
{
	struct jni32_map_element *new_data = NULL;
	int32_t old_size = 0, new_size = 0;

	old_size = map->size;
	new_size = old_size + more;

	new_data = realloc(map->data, new_size * sizeof(struct jni32_map_element));
	if (new_data == NULL) {
		return -1;
	}

	map->data = new_data;
	map->size = new_size;

	return old_size;		/* free tail */
}


/*
 * Add to the free list all elements from free_tail to the end of the map.
 */
void HIDDEN
jni32_map_freelist_add(struct jni32_map *map, int32_t free_tail)
{
	int32_t last = 0;
	int32_t i = 0;

	last = map->size - 1;

	for (i = free_tail; i < last; ++i) {
		map->data[i].next_free = i + 1;
		map->data[i].value = NULL;
	}

	map->data[last].next_free = map->free;
	map->data[last].value = NULL;

	map->free = free_tail;
}

