/* 
 * jni32_method.c
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Stanislav Polevic <stanislav.v.polevic@intel.com>
 * Copyright (C) 2005       Vasily Levchenko <vasily.v.levchenko@intel.com>
 * Copyright (C) 2005       Alexander Alexeev <alexander.n.alexeev@intel.com>
 * Copyright (C) 2005       Valery Ushakov <valery.e.ushakov@intel.com>
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#include <jni.h>
#include <stdlib.h>

#include <jni32_jnienv.h>
#include <jni32_idmap.h>
#include <jni32_refmap.h>
#include <jni32_ptrmap.h>
#include <jni32_args.h>
#include <jni32_upcall.h>
#include <jni32_trampolines.h>


#define _UPCASE(x) __UPCASE_TYPE(x)

#define DEFINE_GET_METHOD_ID(_name)\
DEFINE_JNIENV_IA64_TRAMPOLINE(_name)\
{\
	JNIEnv	*env;\
	jclass	clazz;\
	char	*name;\
	char	*sig;\
	jmethodID mid;\
	uint32_t sp32;\
	jni32_jmethodID methodID;\
\
	sp32 = btmix_get_upcall_argp(ec32);\
	assert(sp32 != 0);\
\
	env	= jni32_jnienv_demarshal(&sp32);\
	assert(env != NULL);\
	clazz = jni32_demarshal_ref(&sp32);\
	name	= demarshal_ptr(&sp32);\
	sig	= demarshal_ptr(&sp32);\
\
	mid = (*env)->_name(env, clazz, name, sig);\
        if (mid == NULL) { \
                btmix_set_retval_pointer(ec32, 0);\
                return 0;\
        }\
 \
	methodID = create_method_entry(mid, name, sig);\
\
        if (methodID == NULL) { /* set exception if methodID is NULL*/ \
        btmix_set_retval_pointer(ec32, 0);\
                return 0;\
        }\
	/* set exception if methodID is NULL*/\
	btmix_set_retval_pointer(ec32, marshal_methodID(methodID));\
	return 0;\
}


/*
 * Common tempalte for Call*Method* trampolines.
 * Varying parts are parametrized via somewhat hairy macros.
 */		
#define DEFINE_CALL(_jtype, _kind, _sig, _prefix)\
DEFINE_JNIENV_IA64_TRAMPOLINE(CONCAT(_prefix, Method ## _sig))\
{\
	uint32_t ap32;\
	JNIEnv *env;\
	jobject self __attribute__((unused));\
	jclass clazz __attribute__((unused));\
	jni32_jmethodID methodID;\
	jvalue *args;\
\
	ap32 = btmix_get_upcall_argp(ec32);\
	assert(ap32 != 0);\
\
	env = jni32_jnienv_demarshal(&ap32);\
	assert(env != NULL);\
	__DEMARSHAL_TARGET_ ## _kind(self, clazz);\
\
	methodID = demarshal_methodID(&ap32);\
\
        if (methodID == NULL) {\
	        __SET_RETVAL_ ## _jtype (_jtype,\
	                                 (*env)->CONCAT(_prefix, MethodA)(env,\
			                 __PASS_TARGET_ ## _kind(self, clazz),\
			                 0, NULL));\
	        return 0;\
	}\
\
	/* test value of the methodID*/\
	args = alloca(methodID->method_info.argc * sizeof(jvalue));\
	__RETRIEVE_ARGS ## _sig(ec32, &methodID->method_info,\
							ap32, args);\
\
	__SET_RETVAL_ ## _jtype (_jtype,\
	(*env)->CONCAT(_prefix, MethodA)(env,\
			__PASS_TARGET_ ## _kind(self, clazz),\
			methodID->mid64, args));\
	return 0;\
}


/* 
 * How to retrieve the target of the call.
 * Differs for Call, CallStatic, and CallNonvirtual.
 */

#define __DEMARSHAL_TARGET_(_self, _class) do {\
		_self = jni32_demarshal_ref(&ap32);\
	} while (0)

#define __DEMARSHAL_TARGET_Static(_self, _class) do {\
	_class = jni32_demarshal_ref(&ap32);\
	} while (0)

#define __DEMARSHAL_TARGET_Nonvirtual(_self, _class) do {\
	_self = jni32_demarshal_ref(&ap32);\
	_class = jni32_demarshal_ref(&ap32);\
	} while (0)


/*
 * What to pass as call target to Call*MethodA.	
 * Differs for Call, CallStatic, and CallNonvirtual.
 */

#define __PASS_TARGET_(_self, _class)		_self
#define __PASS_TARGET_Static(_self, _class)	_class
#define __PASS_TARGET_Nonvirtual(_self, _class)	_self, _class


/*
 * How to retrieve call arguments from ia32 world.
 * Differs for Method,	MethodA and MethodV.
 */

#define __RETRIEVE_ARGS			retrieve_args_stack
#define __RETRIEVE_ARGSV		retrieve_args_valist
#define __RETRIEVE_ARGSA		retrieve_args_array


#define DEFINE_ONE_CALL(_kind, _jtype, _sig)\
	DEFINE_CALL(_jtype, _kind, _sig,\
	CONCAT(Call ## _kind, _UPCASE(_jtype)))


/* loop over object, static, nonvirtual */
#define DEFINE_TYPED_CALLS(_jtype, _sig)\
	DEFINE_ONE_CALL(/* */,		_jtype, _sig)\
	DEFINE_ONE_CALL(Static,	_jtype, _sig)\
	DEFINE_ONE_CALL(Nonvirtual, _jtype, _sig)

/* loop over return types */
#define DEFINE_CALLS_FOR_SIGNATURE(_sig)\
	DEFINE_TYPED_CALLS(void,	_sig)\
	DEFINE_TYPED_CALLS(jboolean, _sig)\
	DEFINE_TYPED_CALLS(jbyte,	_sig)\
	DEFINE_TYPED_CALLS(jchar,	_sig)\
	DEFINE_TYPED_CALLS(jshort,	_sig)\
	DEFINE_TYPED_CALLS(jint,	_sig)\
	DEFINE_TYPED_CALLS(jlong,	_sig)\
	DEFINE_TYPED_CALLS(jfloat,	_sig)\
	DEFINE_TYPED_CALLS(jdouble,	_sig)\
	DEFINE_TYPED_CALLS(jobject,	_sig)

/* loop over JNI signatures */			
#define DEFINE_ALL_CALLS()\
	DEFINE_CALLS_FOR_SIGNATURE(/* vararg */	)\
	DEFINE_CALLS_FOR_SIGNATURE(/* valist */ V)\
	DEFINE_CALLS_FOR_SIGNATURE(/* array	*/ A)

/* GetXXXMethodID */
DEFINE_GET_METHOD_ID(GetMethodID)
DEFINE_GET_METHOD_ID(GetStaticMethodID)

DEFINE_ALL_CALLS()

