/* 
 * jni32_mth_parse.h
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Stanislav Polevic <stanislav.v.polevic@intel.com>
 * Copyright (C) 2005       Vasily Levchenko <vasily.v.levchenko@intel.com>
 * Copyright (C) 2005       Alexander Alexeev <alexander.n.alexeev@intel.com>
 * Copyright (C) 2005       Valery Ushakov <valery.e.ushakov@intel.com>
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
#ifndef _JNI32_MTH_PARSER_H_
#define _JNI32_MTH_PARSER_H_

#include <jni.h>
#include <stdint.h>

#include <mme_internal.h>

#define JNI_BOOL	0
#define JNI_BYTE	1
#define JNI_CHAR	2
#define JNI_SHORT	3
#define JNI_INT	4
#define JNI_LONG	5
#define JNI_FLOAT	6
#define JNI_DOUBLE	7
#define JNI_VOID	8
#define JNI_REF	9
#define JNI_REF_ARRAY	10
#define JNI_PRIM_ARRAY	11

#define JNI_ARRAY_REF_SIGN	"[L"
#define JNI_ARRAY_PRIM_SIGN	"["

#define JNI_BOOL_SIGN	"boolean"
#define JNI_BYTE_SIGN	"byte"
#define JNI_CHAR_SIGN	"char"
#define JNI_SHORT_SIGN	"short"
#define JNI_INT_SIGN	"int"
#define JNI_LONG_SIGN	"long"
#define JNI_FLOAT_SIGN	"float"
#define JNI_DOUBLE_SIGN	"double"
#define JNI_VOID_SIGN	"void"
#define JNI_REF_SIGN	"L"

#define JNI_BOOL_SYM	"Z"
#define JNI_BYTE_SYM	"B"
#define JNI_CHAR_SYM	"C"
#define JNI_SHORT_SYM	"S"
#define JNI_INT_SYM	"I"
#define JNI_LONG_SYM	"J"
#define JNI_FLOAT_SYM	"F"
#define JNI_DOUBLE_SYM	"D"
#define JNI_VOID_SYM	"V"
#define JNI_REF_SYM	"L"

#define JNI_SIG_START	'('
#define JNI_SIG_END	')'
#define JNI_REF_END	';'
#define JNI_ARRAY_CHAR	'['
#define JNI_BOOL_CHAR	'Z'
#define JNI_BYTE_CHAR	'B'
#define JNI_CHAR_CHAR	'C'
#define JNI_SHORT_CHAR	'S'
#define JNI_INT_CHAR	'I'
#define JNI_LONG_CHAR	'J'
#define JNI_FLOAT_CHAR	'F'
#define JNI_DOUBLE_CHAR	'D'
#define JNI_VOID_CHAR	'V'
#define JNI_REF_CHAR	'L'

/*
 *
 */
struct jni32_method_info {
	const char	*name;
	const char	*sig;
	char		ret;
	uint32_t	argc;
};

/*
 * Parses signature and fills specified string with type tags.
 * Skips return type.
 * Result string is a bunch of type tags, one per character.
 */
extern int HIDDEN parse_sig(const char *sig, struct jni32_method_info *method);

/*
 * Extract name of the method from Java object of class Method
 */ 
extern jsize HIDDEN get_method_sig(JNIEnv *env, jobject method_obj, char* name_buf);

/*
 * Extract method's signature from Java object of class Method
 */ 
extern jsize HIDDEN get_method_name(JNIEnv *env, jobject method_obj, char* sig_buf);

/*
 * Check if a Java object is a Method
 */ 
extern int32_t HIDDEN isMethod(JNIEnv *env, jobject tested_obj);

/*
 * Get java type size
 */
extern size_t HIDDEN get_jtype_size(int jtype);

/*
 * Get array component java type
 */
extern int HIDDEN get_array_component_jtype(JNIEnv *env, jobject array);

#endif /*_JNI32_MTH_PARSER_H_*/
