/* 
 * jni32_nio.c
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Stanislav Polevic <stanislav.v.polevic@intel.com>
 * Copyright (C) 2005       Vasily Levchenko <vasily.v.levchenko@intel.com>
 * Copyright (C) 2005       Alexander Alexeev <alexander.n.alexeev@intel.com>
 * Copyright (C) 2005       Valery Ushakov <valery.e.ushakov@intel.com>
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/


#include <jni32_refmap.h>
#include <jni32_jnienv.h>
#include <jni32_stack.h>
#include <jni32_trampolines.h>
#include <jni32_upcall.h>
#include <jni32_diag.h>

 
DEFINE_JNIENV_IA64_TRAMPOLINE(NewDirectByteBuffer)
{
	JNIEnv  *env;
	void    *address;
	jlong    capacity;
	jobject  result;
	uint32_t sp32;

	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env = jni32_jnienv_demarshal(&sp32);
	assert(env != NULL);
	address  = (void *)(unsigned long)btmix_pop_jint(&sp32);
	capacity = btmix_pop_jlong(&sp32);

	DBPRINT(1, "NewDirectByteBuffer: 0x%lx, 0x%lx, 0x%lx\n", (unsigned long)env, (unsigned long)address, (unsigned long)capacity);
	result = (*env)->NewDirectByteBuffer(env, address, capacity);

	__SET_RETVAL_OBJECT_LOCAL(jobject, result);
	return 0;
}

 
DEFINE_JNIENV_IA64_TRAMPOLINE(GetDirectBufferAddress)
{
	JNIEnv  *env;
	jobject  buf;
	void    *result;
	uint32_t sp32;

	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env = jni32_jnienv_demarshal(&sp32);
	assert(env != NULL);
	buf = jni32_get_ref(btmix_pop_jobject(&sp32));

	DBPRINT(1, "GetDirectBufferAddress: 0x%lx, 0x%lx\n", (unsigned long)env, (unsigned long)buf);
	result = (*env)->GetDirectBufferAddress(env, buf);

	assert (IS_32BIT_VALUE(result)); /* or will not working */

	__SET_RETVAL_PTR(void, result);
	return 0;
}

 
DEFINE_JNIENV_IA64_TRAMPOLINE(GetDirectBufferCapacity)
{

	JNIEnv  *env;
	jobject  buf;
	jlong    result;
	uint32_t sp32;

	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env = jni32_jnienv_demarshal(&sp32);
	assert(env != NULL);
	buf = jni32_get_ref(btmix_pop_jobject(&sp32));

	DBPRINT(1, "GetDirectBufferCapacity: 0x%lx, 0x%lx\n", (unsigned long)env, (unsigned long)buf);
	result = (*env)->GetDirectBufferCapacity(env, buf);

	btmix_set_retval_jlong(ec32, result);
	return 0;
}

