/* 
 * jni32_object.c
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Stanislav Polevic <stanislav.v.polevic@intel.com>
 * Copyright (C) 2005       Vasily Levchenko <vasily.v.levchenko@intel.com>
 * Copyright (C) 2005       Alexander Alexeev <alexander.n.alexeev@intel.com>
 * Copyright (C) 2005       Valery Ushakov <valery.e.ushakov@intel.com>
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#include <alloca.h>

#include <jni32_jnienv.h>
#include <jni32_refmap.h>
#include <jni32_idmap.h>
#include <jni32_args.h>
#include <jni32_trampolines.h>
#include <jni32_upcall.h>

#define DEFINE_NEW_OBJECT_TRAMPOLINE(__name, __args_from)\
DEFINE_JNIENV_IA64_TRAMPOLINE(__name)\
{\
	JNIEnv	*env;\
	jclass clazz;\
	jvalue *argc;\
	jobject result;\
	uint32_t sp32;\
	jni32_jmethodID methodID;\
\
	sp32 = btmix_get_upcall_argp(ec32);\
	assert(sp32 != 0);\
\
	env	= jni32_jnienv_demarshal(&sp32);\
	assert(env != NULL);\
	clazz	= jni32_demarshal_ref(&sp32);\
	methodID = demarshal_methodID(&sp32);\
\
	if (methodID == NULL) { /*XXX: throw exception if zero*/\
                result = (*env)->NewObjectA(env, clazz, NULL, NULL);\
	        __SET_RETVAL_OBJECT_LOCAL(jobject, result);\
        	return 0;\
        }\
\
	argc = alloca(methodID->method_info.argc * sizeof(jvalue));\
	__args_from(ec32, &methodID->method_info, sp32, argc);\
\
	result = (*env)->NewObjectA(env, clazz, methodID->mid64, argc);\
\
	__SET_RETVAL_OBJECT_LOCAL(jobject, result);\
	return 0;\
}


DEFINE_JNIENV_IA64_TRAMPOLINE(AllocObject)
{
	JNIEnv	*env;
	jobject result;
	jclass	clazz;
	uint32_t sp32;

	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env = jni32_jnienv_demarshal(&sp32);
	assert(env != NULL);
	clazz = jni32_demarshal_ref(&sp32);

	result = (*env)->AllocObject(env, clazz);
	
	__SET_RETVAL_OBJECT_LOCAL(jobject, result);
	return 0;
}
	
DEFINE_NEW_OBJECT_TRAMPOLINE(NewObject, retrieve_args_stack)
DEFINE_NEW_OBJECT_TRAMPOLINE(NewObjectV, retrieve_args_valist)
DEFINE_NEW_OBJECT_TRAMPOLINE(NewObjectA, retrieve_args_array)

 
DEFINE_JNIENV_IA64_TRAMPOLINE(GetObjectClass)
{
	JNIEnv	*env;
	jobject self;
	jclass	clazz;
	uint32_t sp32;

	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env = jni32_jnienv_demarshal(&sp32);
	assert(env != NULL);
	self = jni32_demarshal_ref(&sp32);

	clazz = (*env)->GetObjectClass(env, self);
 
	__SET_RETVAL_OBJECT_LOCAL(jobject, clazz);
	return 0;
}

 
DEFINE_JNIENV_IA64_TRAMPOLINE(IsInstanceOf)
{
	JNIEnv	*env;
	jobject self;
	jclass	clazz;
	jboolean result;
	uint32_t sp32;

	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env = jni32_jnienv_demarshal(&sp32);
	self = jni32_demarshal_ref(&sp32);
	clazz = jni32_demarshal_ref(&sp32);

	if (self == NULL) {
	  	__SET_RETVAL_PRIMITIVE(jboolean, JNI_TRUE);
		return 0;
	}

	result = (*env)->IsInstanceOf(env, self, clazz);

	__SET_RETVAL_PRIMITIVE(jboolean, result);
	return 0;
}

