/* 
 * jni32_ref.c
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Stanislav Polevic <stanislav.v.polevic@intel.com>
 * Copyright (C) 2005       Vasily Levchenko <vasily.v.levchenko@intel.com>
 * Copyright (C) 2005       Alexander Alexeev <alexander.n.alexeev@intel.com>
 * Copyright (C) 2005       Valery Ushakov <valery.e.ushakov@intel.com>
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/


#include <jni32_jnienv.h>
#include <jni32_refmap.h>
#include <jni32_trampolines.h>
#include <jni32_upcall.h>

#define DEFINE_NEW_REF_TRAMPOLINE(_type, _name, _ref_type)\
DEFINE_JNIENV_IA64_TRAMPOLINE(_name)\
{\
	JNIEnv	*env;\
	jobject obj;\
	jobject res;\
	uint32_t ap32;\
\
	ap32 = btmix_get_upcall_argp(ec32);\
	assert(ap32 != 0);\
\
	env	= jni32_jnienv_demarshal(&ap32);\
	obj	= jni32_demarshal_ref(&ap32);\
\
        if (obj == NULL) {\
                __DEFINE_SET_RETVAL_OBJECT(jobject, NULL, _ref_type);\
	        return 0;\
        }\
\
	res	= (*env)->_name(env, obj);\
\
	__DEFINE_SET_RETVAL_OBJECT(jobject, res, _ref_type);\
	return 0;\
}

#define DEFINE_DEL_REF_TRAMPOLINE(_type, _name, _ref_type)\
DEFINE_JNIENV_IA64_TRAMPOLINE(_name)\
{\
	JNIEnv	*env;\
	jobject obj;\
	uint32_t ap32, idx;\
\
	ap32 = btmix_get_upcall_argp(ec32);\
	assert(ap32 != 0);\
\
	env	= jni32_jnienv_demarshal(&ap32);\
	idx	= btmix_pop_jobject(&ap32);\
\
        if (idx == 0) {\
	        return 0;\
	}\
\
	obj	= jni32_get_ref(idx);\
\
	jni32_##_ref_type##_ref_del(idx);\
\
	__SET_RETVAL_VOID(void, (*env)->_name(env, obj));\
	return 0;\
}

 
DEFINE_JNIENV_IA64_TRAMPOLINE(PushLocalFrame)
{
	JNIEnv	*env;
	jint	capacity, result;
	uint32_t ap32;
	int32_t res;

	ap32 = btmix_get_upcall_argp(ec32);
	assert(ap32 != 0);

	env		= jni32_jnienv_demarshal(&ap32);
	assert(env != NULL);
	capacity = btmix_pop_jint(&ap32);
	
	res = jni32_push_local_ref_frame(capacity);

	if (res < 0) {
	  	__SET_RETVAL_PRIMITIVE(jint, res);
		return 0;
	}

	result = (*env)->PushLocalFrame(env, capacity);

	__SET_RETVAL_PRIMITIVE(jint, result);
	return 0;
}

 
DEFINE_JNIENV_IA64_TRAMPOLINE(PopLocalFrame)
{
	JNIEnv	*env;
	jobject obj;
	jobject result;
	uint32_t ap32;

	ap32 = btmix_get_upcall_argp(ec32);
	assert(ap32 != 0);

	env = jni32_jnienv_demarshal(&ap32);
	assert(env != NULL);
	obj = jni32_demarshal_ref(&ap32);

	jni32_pop_local_ref_frame();
	result = (*env)->PopLocalFrame(env, obj);

	__SET_RETVAL_OBJECT_LOCAL(jobject, result);
	return 0;
}

 /*
 * [New/Delete][Global/Local]Ref + IsSameObject
 */
DEFINE_NEW_REF_TRAMPOLINE(jobject, NewGlobalRef, global)
DEFINE_DEL_REF_TRAMPOLINE(void, DeleteGlobalRef, global)
DEFINE_DEL_REF_TRAMPOLINE(void, DeleteLocalRef, local)

DEFINE_JNIENV_IA64_TRAMPOLINE(IsSameObject)
{
	JNIEnv	*env;
	jobject obj1;
	jobject obj2;
	jboolean res;
	uint32_t ap32;

	ap32 = btmix_get_upcall_argp(ec32);
	assert(ap32 != 0);

	env	= jni32_jnienv_demarshal(&ap32);
	obj1	= jni32_demarshal_ref(&ap32);
	obj2	= jni32_demarshal_ref(&ap32);

	if ((obj1 == NULL) && (obj2 == NULL)) {
	 	__SET_RETVAL_PRIMITIVE(jboolean, JNI_TRUE);
		return 0;
	}
	/*if ((obj1 == NULL) || (obj2 == NULL)) {
	 	__SET_RETVAL_PRIMITIVE(jboolean, JNI_FALSE);
		return 0;
		}*/

	res	= (*env)->IsSameObject(env, obj1, obj2);
	__SET_RETVAL_PRIMITIVE(jboolean, res);
	return 0;
}

DEFINE_NEW_REF_TRAMPOLINE(jobject, NewLocalRef, local)

 
DEFINE_JNIENV_IA64_TRAMPOLINE(EnsureLocalCapacity)
{
	JNIEnv	*env;
	jint	capacity, result;
	uint32_t ap32;

	ap32 = btmix_get_upcall_argp(ec32);
	assert(ap32 != 0);

	env		= jni32_jnienv_demarshal(&ap32);
	assert(env != NULL);
	capacity = btmix_pop_jint(&ap32);

	result = (*env)->EnsureLocalCapacity(env, capacity);

	__SET_RETVAL_PRIMITIVE(jint, result);	
	return 0;
}

/* [New/Delete]WeakGlobalRef */
DEFINE_NEW_REF_TRAMPOLINE(jweak, NewWeakGlobalRef, weak)
DEFINE_DEL_REF_TRAMPOLINE(void, DeleteWeakGlobalRef, weak)

