/* 
 * jni32_reflect.c
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Stanislav Polevic <stanislav.v.polevic@intel.com>
 * Copyright (C) 2005       Vasily Levchenko <vasily.v.levchenko@intel.com>
 * Copyright (C) 2005       Alexander Alexeev <alexander.n.alexeev@intel.com>
 * Copyright (C) 2005       Valery Ushakov <valery.e.ushakov@intel.com>
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#include <alloca.h>

#include <jni32_jnienv.h>
#include <jni32_idmap.h>
#include <jni32_refmap.h>
#include <jni32_ptrmap.h>
#include <jni32_upcall.h>
#include <jni32_trampolines.h>


DEFINE_JNIENV_IA64_TRAMPOLINE(DefineClass)
{
	JNIEnv *env;
	const char *name;
	jobject loader;
	jbyte *buf;
	jsize bufLen;
	jclass clazz;
	uint32_t sp32;

	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env = jni32_jnienv_demarshal(&sp32);
	assert(env != NULL);
	name = demarshal_ptr(&sp32);
	loader = jni32_demarshal_ref(&sp32);
	buf = (jbyte*) demarshal_ptr(&sp32);
	bufLen = btmix_pop_jint(&sp32);

	if (name == NULL) {
	 	__SET_RETVAL_OBJECT_LOCAL(jobject, NULL);
		return 0;
	}

	clazz = (*env)->DefineClass(env, name, loader, buf, bufLen);

	__SET_RETVAL_OBJECT_LOCAL(jobject, clazz);
	return 0;
}


DEFINE_JNIENV_IA64_TRAMPOLINE(FindClass)
{
	JNIEnv *env;
	const char *name;
	jclass clazz;
	uint32_t sp32;

	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env = jni32_jnienv_demarshal(&sp32);
	assert(env != NULL);
	name = demarshal_ptr(&sp32);

	clazz = (*env)->FindClass(env, name);

	__SET_RETVAL_OBJECT_LOCAL(jobject, clazz);
	return 0;
}


DEFINE_JNIENV_IA64_TRAMPOLINE(FromReflectedMethod)
{
	JNIEnv *env;
	jobject method_obj;
	uint32_t sp32;
	jmethodID mid;
	jni32_jmethodID methodID;
	char *name = NULL;
	jsize name_len = 0;
	char *sig = NULL;
	jsize sig_len = 0;

	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env = jni32_jnienv_demarshal(&sp32);
	assert(env != NULL);
	method_obj = jni32_demarshal_ref(&sp32);

	mid = (*env)->FromReflectedMethod(env, method_obj);

	if (!isMethod(env, method_obj)) {
	        __SET_RETVAL_MID(jni32_jmethodID, NULL);
		return 0;
	}

	name_len = get_method_name(env, method_obj, NULL);
	assert(name_len != 0);
	name = (char*) alloca(name_len);
	name[0] = '\0';
	get_method_name(env, method_obj, name);

	sig_len = get_method_sig(env, method_obj, NULL);
	assert(sig_len != 0);
	sig = (char*) alloca(sig_len);
	sig[0] = '\0';
	get_method_sig(env, method_obj, sig);

	methodID = create_method_entry(mid, name, sig);
	assert(methodID != NULL);

	__SET_RETVAL_MID(jni32_jmethodID, methodID);
	return 0;
}


DEFINE_JNIENV_IA64_TRAMPOLINE(FromReflectedField)
{
	JNIEnv *env;
	jobject obj;
	jfieldID fieldID;
	uint32_t sp32;

	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env = jni32_jnienv_demarshal(&sp32);
	assert(env != NULL);
	obj = jni32_demarshal_ref(&sp32);

	fieldID = (*env)->FromReflectedField(env, obj);

	__SET_RETVAL_FID(jfieldID, fieldID);
	return 0;
}

 
DEFINE_JNIENV_IA64_TRAMPOLINE(ToReflectedMethod)
{
	JNIEnv *env;
	jclass cls;
	jni32_jmethodID methodID;
	uint32_t sp32;
	jobject method_obj;
	jboolean isStatic;

	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env = jni32_jnienv_demarshal(&sp32);
	assert(env != NULL);
	cls = jni32_demarshal_ref(&sp32);
	methodID = demarshal_methodID(&sp32);
	isStatic = btmix_pop_jboolean(&sp32);

	if (methodID == NULL) {
	  	method_obj = (*env)->ToReflectedMethod(env, cls, NULL, isStatic);
	} else {
	        method_obj = (*env)->ToReflectedMethod(env, cls, methodID->mid64, isStatic);
	}

	__SET_RETVAL_OBJECT_LOCAL(jobject, method_obj);
	return 0;
}

 
DEFINE_JNIENV_IA64_TRAMPOLINE(ToReflectedField)
{

	JNIEnv *env;
	jclass cls;
	jfieldID fid;
	uint32_t sp32;
	jobject field_obj;
	jboolean isStatic;

	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env = jni32_jnienv_demarshal(&sp32);
	assert(env != NULL);
	cls = jni32_demarshal_ref(&sp32);
	fid = demarshal_fieldID(&sp32);
	isStatic = btmix_pop_jboolean(&sp32);

	field_obj = (*env)->ToReflectedField(env, cls, fid, isStatic);

	__SET_RETVAL_OBJECT_LOCAL(jobject, field_obj);
	return 0;
}

 
DEFINE_JNIENV_IA64_TRAMPOLINE(GetSuperclass)
{
	JNIEnv *env;
	jclass arg_clazz;
	jclass ret_clazz;
	uint32_t sp32;

	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env = jni32_jnienv_demarshal(&sp32);
	assert(env != NULL);
	arg_clazz = jni32_demarshal_ref(&sp32);

	ret_clazz = (*env)->GetSuperclass(env, arg_clazz);

	__SET_RETVAL_OBJECT_LOCAL(jobject, ret_clazz);
	return 0;
}

 
DEFINE_JNIENV_IA64_TRAMPOLINE(IsAssignableFrom)
{
	JNIEnv *env;
	jclass clazz1;
	jclass clazz2;
	jboolean result;
	uint32_t sp32;

	sp32 = btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env = jni32_jnienv_demarshal(&sp32);
	assert(env != NULL);
	clazz1 = jni32_demarshal_ref(&sp32);
	clazz2 = jni32_demarshal_ref(&sp32);

	if ((clazz1 == NULL) || (clazz2 == NULL)) {
	  	__SET_RETVAL_PRIMITIVE(jboolean, JNI_FALSE);
		return 0;
	}

	result = (*env)->IsAssignableFrom(env, clazz1, clazz2);

	__SET_RETVAL_PRIMITIVE(jboolean, result);
	return 0;
}


