/* 
 * jni32_reg_natives.c
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Stanislav Polevic <stanislav.v.polevic@intel.com>
 * Copyright (C) 2005       Vasily Levchenko <vasily.v.levchenko@intel.com>
 * Copyright (C) 2005       Alexander Alexeev <alexander.n.alexeev@intel.com>
 * Copyright (C) 2005       Valery Ushakov <valery.e.ushakov@intel.com>
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
#include <pthread.h>
#include <stdlib.h>
#include <jni.h>
#include <stdint.h>

#include <mme_internal.h>
#include <jni32_trampolines.h>
#include <jni32_jnienv.h>
#include <jni32_refmap.h>
#include <jni32_ptrmap.h>
#include <jni32_upcall.h>
#include <jni32.h>
#include <jni32_diag.h>

#if 0
#include "jni32_dyn_thunk.h"
#endif

struct JNINativeMethod32 {
	ptr32_t name;
	ptr32_t signature;
	ptr32_t fnPtr;
};

/*
 * entery JNI functions 
 */
DEFINE_JNIENV_IA64_TRAMPOLINE(RegisterNatives)
{
	JNIEnv *env;
	jclass clazz;
	struct JNINativeMethod32 *methods32;
	JNINativeMethod *methods64;
	uint32_t sp32;
	jint result;
	jint nMethods, j;


	/* get 32 bit args */
	sp32 = btmix_get_upcall_argp(ec32);
	env = jni32_jnienv_demarshal(&sp32);
	clazz = jni32_demarshal_ref(&sp32);
	methods32 = demarshal_ptr(&sp32);
	nMethods  = btmix_pop_jint(&sp32);

	/* prepare 64 bit list */
	methods64 = malloc (sizeof (*methods64) * nMethods);
	if (methods64 == NULL) {
		MME_THROW_EXCEPTION("malloc failed");
		result = JNI_ERR;
		goto out;
	}

	for (j = 0; j < nMethods; j ++) {
		methods64[j].name = I2P (methods32[j].name);
		methods64[j].signature = I2P (methods32[j].signature);
		methods64[j].fnPtr = I2P (methods32[j].fnPtr);
	}

	result = g_java_callbacks.register_natives_32 (env, clazz, methods64, nMethods);

	free (methods64);
out:
	__SET_RETVAL_PRIMITIVE(jint, result);
	return 0;
}

DEFINE_JNIENV_IA64_TRAMPOLINE(UnregisterNatives)
{
	JNIEnv *env;
	jclass clazz;
	uint32_t sp32;
	jint result;


	sp32	= btmix_get_upcall_argp(ec32);
	env	= jni32_jnienv_demarshal(&sp32);
	clazz = jni32_demarshal_ref(&sp32);

	result = g_java_callbacks.unregister_natives_32 (env, clazz);

	__SET_RETVAL_PRIMITIVE(jint, result);
	return 0;
}


