/* 
 * jni32_string.c
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Stanislav Polevic <stanislav.v.polevic@intel.com>
 * Copyright (C) 2005       Vasily Levchenko <vasily.v.levchenko@intel.com>
 * Copyright (C) 2005       Alexander Alexeev <alexander.n.alexeev@intel.com>
 * Copyright (C) 2005       Valery Ushakov <valery.e.ushakov@intel.com>
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#include <stdint.h>
#include <jni.h>

#include <jni32_jnienv.h>
#include <jni32_ptrmap.h>
#include <jni32_refmap.h>
#include <jni32_upcall.h>
#include <jni32_trampolines.h>
#include <mme_internal.h>

#define DEFINE_GET_STRING_LEN_TRAMPOLINE(__name, is_UTF)\
DEFINE_JNIENV_IA64_TRAMPOLINE(__name)\
{\
	JNIEnv	*env;\
	jstring jstr;\
	uint32_t sp32;\
	jsize result;\
\
	sp32 = btmix_get_upcall_argp(ec32);\
	assert(sp32 != 0);\
\
	env	= jni32_jnienv_demarshal(&sp32);\
	assert(env != NULL);\
	jstr	= jni32_demarshal_ref(&sp32);\
\
        if ((jstr == NULL) && (is_UTF == 1)) {\
  	        __SET_RETVAL_PRIMITIVE(jint, -1);\
	        return 0;\
        }\
\
	result = (*env)->__name(env, jstr);\
\
	__SET_RETVAL_PRIMITIVE(jint, result);\
	return 0;\
}


#define DEFINE_GET_STRING_CHARS_TRAMPOLINE(__type,__string_type,__name, isCritical)\
DEFINE_JNIENV_IA64_TRAMPOLINE(__name)\
{\
\
	JNIEnv	*env;\
	__type *buf64;\
	__type *buf32;\
	jstring	jstr;\
	jboolean *isCopy;\
	uint32_t sp32;\
	jsize length;\
\
	sp32 = btmix_get_upcall_argp(ec32);\
	assert(sp32 != 0);\
\
	env	= jni32_jnienv_demarshal(&sp32);\
	jstr	= jni32_demarshal_ref(&sp32);\
	isCopy = demarshal_ptr(&sp32);\
\
        if (jstr == NULL) {\
                if (isCritical == 0) {\
                        if (isCopy != NULL) {\
		                *isCopy = JNI_TRUE;\
	                }\
                }\
                __SET_RETVAL_PTR(void, NULL);\
	        return 0;\
        }\
\
	length = (((*env)->Get##__string_type##Length)\
					(env, jstr) + 1) * sizeof(__type);\
\
	if (length < 0) {\
                if (isCritical == 0) {\
                        if (isCopy != NULL) {\
		                *isCopy = JNI_TRUE;\
	                }\
                }\
                __SET_RETVAL_PTR(void, NULL);\
	        return 0;\
        }\
\
	buf64 = (__type*)(*env)->__name(env, jstr, isCopy);\
	buf32 = jni32_add_ptrmap(buf64, length);\
\
	if (buf32 == NULL) {\
                if (isCritical == 0) {\
                        if (isCopy != NULL) {\
		                *isCopy = JNI_TRUE;\
	                }\
                }\
                __SET_RETVAL_PTR(void, NULL);\
	        return 0;\
        }\
\
	memcpy(buf32, buf64, length);\
	if (isCopy != NULL) {\
		*isCopy = JNI_TRUE;\
	}\
\
	__SET_RETVAL_PTR(void, buf32);\
	return 0;\
}

#define DEFINE_RELEASE_STRING_CHARS_TRAMPOLINE(__type,__string_type,__name)\
DEFINE_JNIENV_IA64_TRAMPOLINE(__name)\
{\
	JNIEnv	*env;\
	jstring	jstr;\
	__type *buf64;\
	void		*buf32;\
	uint32_t sp32;\
	struct JNIEnv32to64 *env32to64;\
\
	sp32 = btmix_get_upcall_argp(ec32);\
	assert(sp32 != 0);\
\
	env	= jni32_jnienv_demarshal(&sp32);\
	jstr	= jni32_demarshal_ref(&sp32);\
	buf32	= demarshal_ptr(&sp32);\
	buf64 = jni32_get_ptrmap(buf32);\
\
	if (env == NULL) {\
	        env32to64 = jni32_jnienv_get_marshalled();\
		if (env32to64 == NULL) {\
			return 0;\
		} else {\
		        env = env32to64->env64;\
                        __SET_RETVAL_VOID(void, (*env)->__name(NULL, jstr, buf64));\
		}\
	} else {\
	        __SET_RETVAL_VOID(void, (*env)->__name(env, jstr, buf64));\
        }\
\
        if (buf64 != NULL) {\
	        jni32_del_ptrmap(buf32, 1);\
        }\
\
	return 0;\
}

#define DEFINE_GET_STRING_REGION_TRAMPOLINE(__type,__name)\
DEFINE_JNIENV_IA64_TRAMPOLINE(__name)\
{\
	JNIEnv	*env;\
	jstring jstr;\
	jsize start;\
	jsize size;\
	__type *buf;\
	uint32_t sp32;\
\
	sp32 = btmix_get_upcall_argp(ec32);\
	assert(sp32 != 0);\
\
	env	= jni32_jnienv_demarshal(&sp32);\
	jstr	= jni32_demarshal_ref(&sp32);\
	start = btmix_pop_jint(&sp32);\
	size	= btmix_pop_jint(&sp32);\
	buf	= (__type*)demarshal_ptr(&sp32);\
\
        if (jstr == NULL) {\
	        return 0;\
        }\
\
	__SET_RETVAL_VOID(void, (*env)->__name(env, jstr, start, size, buf));\
	return 0;\
}


/*
 * We can use the same macros body for normal and critical functions
 */

DEFINE_GET_STRING_LEN_TRAMPOLINE(GetStringLength, 0)
DEFINE_GET_STRING_CHARS_TRAMPOLINE(jchar, String, GetStringChars, 0)
DEFINE_GET_STRING_CHARS_TRAMPOLINE(jchar, String, GetStringCritical, 1)
DEFINE_RELEASE_STRING_CHARS_TRAMPOLINE(jchar, String, ReleaseStringChars)
DEFINE_RELEASE_STRING_CHARS_TRAMPOLINE(jchar, String, ReleaseStringCritical)
DEFINE_GET_STRING_REGION_TRAMPOLINE(jchar, GetStringRegion)

DEFINE_GET_STRING_LEN_TRAMPOLINE(GetStringUTFLength, 1)
DEFINE_GET_STRING_CHARS_TRAMPOLINE(jbyte, StringUTF, GetStringUTFChars, 0)
DEFINE_RELEASE_STRING_CHARS_TRAMPOLINE(const char, StringUTF, ReleaseStringUTFChars)
DEFINE_GET_STRING_REGION_TRAMPOLINE(char, GetStringUTFRegion)


 
DEFINE_JNIENV_IA64_TRAMPOLINE(NewString)
{
	JNIEnv	*env;
	jstring result;
	jchar	*bytes;
	jsize	len;
	uint32_t sp32;

	sp32	= btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env	= jni32_jnienv_demarshal(&sp32);
	assert(env != NULL);
	bytes = demarshal_ptr(&sp32);
	len	= btmix_pop_jint(&sp32);
	

	result = (*env)->NewString(env, bytes, len);

	__SET_RETVAL_OBJECT_LOCAL(jobject, result);
	return 0;
}

 
DEFINE_JNIENV_IA64_TRAMPOLINE(NewStringUTF)
{
	JNIEnv	*env;
	jstring result;
	char	*bytes;
	uint32_t sp32;

	sp32	= btmix_get_upcall_argp(ec32);
	assert(sp32 != 0);

	env	= jni32_jnienv_demarshal(&sp32);
	assert(env != NULL);
	bytes = demarshal_ptr(&sp32);
	

	result = (*env)->NewStringUTF(env, bytes);

	__SET_RETVAL_OBJECT_LOCAL(jobject, result);
	return 0;
}

