/* 
 * jni32_upcall.c
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Stanislav Polevic <stanislav.v.polevic@intel.com>
 * Copyright (C) 2005       Vasily Levchenko <vasily.v.levchenko@intel.com>
 * Copyright (C) 2005       Alexander Alexeev <alexander.n.alexeev@intel.com>
 * Copyright (C) 2005       Valery Ushakov <valery.e.ushakov@intel.com>
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
#include <stdint.h>
#include <string.h>
#include <assert.h>

#include <mme_internal.h>
#include <jni32_upcall.h>
#include <auxapp.h>


/*
 * Our handler for JNI upcalls from IA32 world.	The argument points
 * to the upcall arguments in the IA32 stack (not jni call arguments!
 * those are further down the stack in the next frame).
 */
int64_t HIDDEN
jni32_upcall(ia32el_context32 * ec32, jni32_trampoline_array_t jni32_trampolines, int32_t max_idx)
{
	jni32_trampoline_t tramp;
	int32_t idx;
	int64_t result;

	assert(ec32 != NULL);
	assert(jni32_trampolines != NULL);

	/* Index of the JNI function should be found 4 bytes below esp */
	copy_from_ia32_stack (&idx, I2P(ec32->esp + MME_UPCALL_IDX_OFFSET), sizeof (idx));
	assert(idx >= 0);

	DBPRINT (1, "idx=%d\n", idx);
	if (idx > max_idx || idx < 0) {
		DBPRINT (true, "illegal idx: %d\n", idx);
		return -1;
	}

	tramp = jni32_trampolines[idx];

	if (tramp == NULL) {
		DBPRINT (true, "null trampoline idx: %d\n", idx);
		return -1;
	}
	result = ((*tramp)(ec32));

	DBPRINT (0, "result=%ld\n", result);

	return result;
}

