/* 
 * jni32_upcall.h
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Stanislav Polevic <stanislav.v.polevic@intel.com>
 * Copyright (C) 2005       Vasily Levchenko <vasily.v.levchenko@intel.com>
 * Copyright (C) 2005       Alexander Alexeev <alexander.n.alexeev@intel.com>
 * Copyright (C) 2005       Valery Ushakov <valery.e.ushakov@intel.com>
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
#ifndef _JNI32UPCALL_H_
#define _JNI32UPCALL_H_

#include <ia32el_mme_api.h>


/* Marshalling, demarshalling and set return type helper macros */

/*
 * How to return call result to ia32.
 * Differs for each return type.
 */


#define __SET_RETVAL_PRIMITIVE(_jtype, _ret) do {\
	btmix_set_retval_ ## _jtype(ec32, _ret);\
} while (0)


#define __SET_RETVAL_PTR(_jtype, _ret) do {\
	_jtype *ret = _ret;\
	ptr32_t ptr32;\
	ptr32 = TRUNCATE_PTR (ret);\
	btmix_set_retval_ptr(ec32, ptr32);\
} while (0)

#define __SET_RETVAL_VOID(_jtype, _ret) do {\
	_ret;\
} while (0)

#define __DEFINE_SET_RETVAL_OBJECT(_jtype, _ret, _ref_type) do {\
	_jtype ret = _ret;\
	uint32_t ref32;\
	ref32 = jni32_marshal_ ## _ref_type ## _ref(ret);\
	assert(ref32 != (uint32_t)-1);\
	btmix_set_retval_ ## _jtype(ec32, ref32);\
} while (0)

#define __SET_RETVAL_OBJECT_LOCAL(_jtype, _ret)\
	__DEFINE_SET_RETVAL_OBJECT(_jtype, _ret, local)
#define __SET_RETVAL_OBJECT_GLOBAL(_jtype, _ret)\
	__DEFINE_SET_RETVAL_OBJECT(_jtype, _ret, global)
#define __SET_RETVAL_OBJECT_WEAK(_jtype, _ret)\
	__DEFINE_SET_RETVAL_OBJECT(_jtype, _ret, weak)

#define __SET_RETVAL_MID(_jtype, _ret) do {\
	_jtype ret = _ret;\
	uint32_t ptr32;\
	ptr32 = marshal_methodID(ret);\
	btmix_set_retval_ptr(ec32, ptr32);\
} while (0)

#define __SET_RETVAL_FID(_jtype, _ret) do {\
	_jtype ret = _ret;\
	uint32_t ptr32;\
	ptr32 = marshal_fieldID(ret);\
	btmix_set_retval_ptr(ec32, ptr32);\
} while (0)


#define __SET_RETVAL_void		__SET_RETVAL_VOID
#define __SET_RETVAL_jboolean		__SET_RETVAL_PRIMITIVE
#define __SET_RETVAL_jbyte		__SET_RETVAL_PRIMITIVE
#define __SET_RETVAL_jchar		__SET_RETVAL_PRIMITIVE
#define __SET_RETVAL_jshort		__SET_RETVAL_PRIMITIVE
#define __SET_RETVAL_jint		__SET_RETVAL_PRIMITIVE
#define __SET_RETVAL_jlong		__SET_RETVAL_PRIMITIVE
#define __SET_RETVAL_jfloat		__SET_RETVAL_PRIMITIVE
#define __SET_RETVAL_jdouble		__SET_RETVAL_PRIMITIVE
#define __SET_RETVAL_jobject		__SET_RETVAL_OBJECT_LOCAL



/*
 * How to pop value from stack.
 * Differs for each value type.
 */
#define __POP_VALUE_OBJECT(stack, value, jtype)\
do {\
	value = jni32_demarshal_ref(&stack);\
} while (0)

#define __POP_VALUE_PRIMITIVE(stack, value, jtype)\
do {\
	value = btmix_pop_ ## jtype (&stack);\
} while (0)

#define __POP_VALUE_jboolean		__POP_VALUE_PRIMITIVE
#define __POP_VALUE_jbyte		__POP_VALUE_PRIMITIVE
#define __POP_VALUE_jchar		__POP_VALUE_PRIMITIVE
#define __POP_VALUE_jshort		__POP_VALUE_PRIMITIVE
#define __POP_VALUE_jint		__POP_VALUE_PRIMITIVE
#define __POP_VALUE_jlong		__POP_VALUE_PRIMITIVE
#define __POP_VALUE_jfloat		__POP_VALUE_PRIMITIVE
#define __POP_VALUE_jdouble		__POP_VALUE_PRIMITIVE
#define __POP_VALUE_jobject		__POP_VALUE_OBJECT

/* Type of 64 bit handler for a JNI32 upcall */
typedef int64_t (*jni32_trampoline_t)(ia32el_context32 *);

/* Type of 64 bit handler array for a JNI32 upcall */
typedef int64_t (*jni32_trampoline_array_t[])(ia32el_context32 *);


/* btlib upcall handler for IA32 JNI calls */
extern int64_t jni32_upcall(ia32el_context32 *pbc32, jni32_trampoline_array_t jni32_trampolines, int32_t max_idx);


#define _UPCASE_OF_jboolean_TYPE	Boolean
#define _UPCASE_OF_jbyte_TYPE		Byte
#define _UPCASE_OF_jchar_TYPE		Char
#define _UPCASE_OF_jshort_TYPE		Short
#define _UPCASE_OF_jint_TYPE		Int
#define _UPCASE_OF_jlong_TYPE		Long
#define _UPCASE_OF_jfloat_TYPE		Float
#define _UPCASE_OF_jdouble_TYPE		Double
#define _UPCASE_OF_jobject_TYPE		Object
#define _UPCASE_OF_void_TYPE		Void

#define _UPCASE_OF_jsize_TYPE		Int


#define __XUPCASE_TYPE(x)	_UPCASE_OF_##x##_TYPE
#define __UPCASE_TYPE(x)	__XUPCASE_TYPE(x)



#define _CONCAT(a, b)	a ## b
#define CONCAT(a, b)	_CONCAT(a, b)

#define _CONCAT3(a, b, c) a##b##c     
#define CONCAT3(a, b, c) _CONCAT3(a, b, c)


#endif /* _JNI32UPCALL_H_ */

