/* 
 * mme_internal.h
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
#ifndef _MTL_INTERNAL_H
#define _MTL_INTERNAL_H

#include <stdio.h>
#include <unistd.h>
#include <stdint.h>
#include <syscall.h>
#include <stdbool.h>
#include <setjmp.h>
#include <assert.h>
#include <signal.h>
#include <string.h>
#include <link.h>
#include <jni.h>

#include <linux/limits.h>

#include <mme_jvm_api.h>
#include <ia32el_mme_api.h>

#define UNUSED __attribute__((unused))
#define HIDDEN __attribute__ ((visibility ("hidden")))
#define MME_THREAD_VAR __thread __attribute__ ((tls_model ("global-dynamic")))
#define I2P(n) (void*)(uintptr_t)(n)
#define OFFSETOF(field,struct_type) (uintptr_t)&(((struct_type*)0)->field)
#define IS_32BIT_VALUE(x) ((uint64_t)(x) == (0xffffffff & (uint64_t)(x)))
#define TRUNCATE_PTR(p) (ptr32_t)(uintptr_t)(p)
#define MME_THROW_EXCEPTION(msg) jni32_throw_error (1, msg)
#define ROUNDUP(v,b) (((v) + (b) - 1) & ~((b) - 1))
#define ROUNDDOWN(v,b)	((v) & ~((b) - 1))
/* hint for compiler */
#if __GNUC__ >= 3
#define __PREDICT_TRUE(cond) __builtin_expect((cond), !0)
#define __PREDICT_FALSE(cond) __builtin_expect((cond), 0)
#else
#define __PREDICT_TRUE(cond) (cond)
#define __PREDICT_FALSE(cond) (cond)
#endif

#define gettid() syscall ((int64_t)SYS_gettid)

#if MMEDEBUG
extern HIDDEN FILE * g_log_file;
#define DBPRINT(sw,format,args...) do {						\
	if (sw) {								\
		fprintf (g_log_file ? : stderr, "(%d) %s:%s - " format, 	\
			 (int32_t)gettid(), __FILE__, __FUNCTION__, ## args);	\
	}									\
} while (0)
#define DBCODE(sw,expr) if (sw) {expr;}
#else
#define DBPRINT(sw,format,args...)
#define DBCODE(sw,expr)
#endif

typedef uint32_t ptr32_t;

extern struct _mme_ia32el_exports_tag HIDDEN g_ia32el_exports;
extern uint64_t HIDDEN g_flags;
extern MME_THREAD_VAR ptr32_t t_downcall_stacktop;
extern struct ia32_runtime_fptr_table HIDDEN g_ia32_runtime;
extern sigjmp_buf g_thread_func_jmpbuf;

extern int32_t HIDDEN internal_ld_iterate_phdr (phdr_iter_callback jvm_callback, void * data);
extern int32_t HIDDEN mme_register_signal_handlers (void);
extern int32_t HIDDEN mme_unregister_signal_handlers (void);
extern bool HIDDEN change_sighand (int32_t signo, struct sigaction * sc);
extern ia32el_sighand_result HIDDEN internal_mme_signal_handler (int32_t signo, struct siginfo * si, struct sigcontext * sc);
extern int32_t HIDDEN do_downcallV (int64_t return_type, uint64_t * return_value, void *addr, char *param_list, ...) ;
extern int32_t HIDDEN do_downcall (bool is_jni, int64_t return_type, uint64_t * return_value, void *addr, char *param_list, va_list params);
extern int32_t HIDDEN is_nio_call(char *param_list, va_list params, uint64_t * return_value);
extern int32_t HIDDEN internal_mme_init (uint64_t flags, int64_t tls_offset);
extern ptr32_t HIDDEN malloc32 (uint32_t size);
extern ptr32_t HIDDEN realloc32(ptr32_t ptr, uint32_t size);
extern void HIDDEN free32 (ptr32_t ptr32);
extern int32_t HIDDEN internal_mme_unload (void);
extern int32_t HIDDEN before_thread_create (void **);
extern int32_t HIDDEN after_thread_create (mme_thread_init_flag flag, void *clone_params);
extern int32_t HIDDEN init_thread (bool);
int32_t HIDDEN internal_thread_term (void);
extern int32_t HIDDEN process_phdr_iter_callback (ia32el_context32 *);
extern int32_t HIDDEN do_ld_iterate_phdr (phdr_iter_callback callback, void * data);
extern int32_t HIDDEN internal_dump_exception_context (FILE *fp);
extern int32_t HIDDEN dump_ia32_context (FILE *fp, ia32el_context32 *);


#if FIXED_STUBS
/* stubs stuff */
#define STUBS_AREA_SIZE getpagesize()
extern void * g_stubs_area;
static inline void * get_stub_addr (uint32_t stub_type)
{
	/* the left shift is not really important - it's just an easy way
	 * to pick a place large enough to accomodate a stub */
	assert (g_stubs_area != NULL);
	return (stub_type << 4) + g_stubs_area;
}
#endif // FIXED_STUBS

/****************************************************************************/
/* copy 'size' bytes from 'src' to 'dest'. assuming 'dest' is 4 bytes aligned
 * and 'src' is well aligned. */
/****************************************************************************/
static inline void copy_to_ia32_stack (void * dest, void * src, uint64_t size)
{
	union {
		struct {
			uint32_t lo, hi;
		} u;
		uint64_t lu;
	} pc;

	DBPRINT (0, "dest=%p src=%p size=%lx\n", dest, src, size);

	switch (size) {
		case 4:
			assert ((uintptr_t)dest == ((uintptr_t)dest & ~(uintptr_t)0x3));
			assert ((uintptr_t)src == ((uintptr_t)src & ~(uintptr_t)0x3));
			*(uint32_t*)dest = *(uint32_t*)src;
			break;
		case 8:
			assert ((uintptr_t)dest == ((uintptr_t)dest & ~(uintptr_t)0x3));
			assert ((uintptr_t)src == ((uintptr_t)src & ~(uintptr_t)0x7));
			pc.lu = *(uint64_t*)src;
			*(uint32_t*)dest = pc.u.lo;
			*(uint32_t*)(dest + 4) = pc.u.hi;
			break;
		default:
			assert ( ! "unexpected size" );
			memcpy (dest, src, size);
	}
}
/****************************************************************************/
/* copy 'size' bytes from 'src' to 'dest'. assuming 'dest' is well aligned
 * and 'src' is 4 bytes aligned. */
/****************************************************************************/
static inline void copy_from_ia32_stack (void * dest, void * src, uint64_t size)
{
	union {
		struct {
			uint32_t lo, hi;
		} u;
		uint64_t lu;
	} pc;

	DBPRINT (0, "dest=%p src=%p size=%ld\n", dest, src, size);
	switch (size) {
		case 4:
			assert ((uintptr_t)dest == ((uintptr_t)dest & ~(uintptr_t)0x3));
			assert ((uintptr_t)src == ((uintptr_t)src & ~(uintptr_t)0x3));
			*(uint32_t*)dest = *(uint32_t*)src;
			break;
		case 8:
			assert ((uintptr_t)dest == ((uintptr_t)dest & ~(uintptr_t)0x7));
			assert ((uintptr_t)src == ((uintptr_t)src & ~(uintptr_t)0x3));
			pc.u.lo = *(uint32_t*)src;
			pc.u.hi = *(uint32_t*)(src + 4);
			*(uint64_t*)dest = pc.lu;
			break;
		default:
			assert ( ! "unexpected size" );
			memcpy (dest, src, size);
	}
}

#endif // _MTL_INTERNAL_H
