/* 
 * mme_signals.c
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
#include <signal.h>
#include <string.h>
#include <assert.h>
#include <stdbool.h>
#include <stdlib.h>

#include <mme_jvm_api.h>
#include <ia32el_mme_api.h>
#include <mme_internal.h>

typedef void (*sigaction_handler_t) (int32_t, siginfo_t *, void *);

int64_t g_tls_offset = 0;

static int32_t m_hw_signals[] = {SIGILL, SIGBUS, SIGFPE, SIGSEGV, /* MUST BE LAST ELEMENT */ 0};
static struct sigaction m_orig_handlers[SIGSEGV + 1];
static bool m_is_orig_handlers[SIGSEGV + 1] = {false};
extern HIDDEN void __mme_signal_handler (int32_t, struct siginfo *, struct sigcontext *);

int32_t HIDDEN mme_register_signal_handlers (void)
{
	int32_t * hwsp;
	int32_t ret;
	struct sigaction act;

#if DEBUG && 0
	/* make sure static vars are initialized to zero */
	char _dd[sizeof(m_orig_handlers)] = {0};
	assert (memcmp (_dd, m_orig_handlers, sizeof(m_orig_handlers) == 0));
#endif

	memset (&act, 0, sizeof (act));
	act.sa_sigaction = (sigaction_handler_t)__mme_signal_handler;
	act.sa_flags = SA_SIGINFO;
	sigfillset (&act.sa_mask);

	for (hwsp = &m_hw_signals[0]; * hwsp; hwsp ++) {
		assert ((int64_t)*hwsp < (int64_t)(sizeof(m_orig_handlers)/sizeof(m_orig_handlers[0])));
		ret = sigaction (*hwsp, &act, &m_orig_handlers[*hwsp]);
		if (ret < 0) {
			DBCODE (true, perror ("sigaction failed"));
			return -1;
		}
		m_is_orig_handlers[*hwsp] = true;
	}
	return 0;
}

int32_t HIDDEN mme_unregister_signal_handlers (void)
{
	int32_t * hwsp;
	int32_t ret, error = 0;

	for (hwsp = &m_hw_signals[0]; * hwsp; hwsp ++) {
		assert ((int64_t)*hwsp < (int64_t)(sizeof(m_orig_handlers)/sizeof(m_orig_handlers[0])));
		ret = m_is_orig_handlers[*hwsp] ?
			sigaction (*hwsp, &m_orig_handlers[*hwsp], NULL) :
			/* orig_handlers[0] can serve as the default handler */
			sigaction (*hwsp, &m_orig_handlers[0], NULL);
		if (ret < 0) {
			DBCODE (true, perror ("sigaction failed"));
			error = 1;
		}


	}

	return error;
}

ia32el_sighand_result HIDDEN internal_mme_signal_handler (int32_t signo, struct siginfo * si, struct sigcontext * sc)
{
	ia32el_sighand_result ret;
	int32_t syscall_retval;
        register void * __r13  asm("r13");
        void **__clst_storage;
        __clst_storage = (void **)((uintptr_t)__r13 + g_tls_offset);
        ret = false;
	/* this handler can be called on 2 circumstances -
	 * 1 - mme handles the signals, in which case the signal should
	 *     be forwareded to ia32el and only if ia32el does not handle
	 *     it then the signal should be forwarded to the orig hanlder (if exists)
	 * 2 - jvm handles the signals, in which case this the signal should be forwarded
	 *     to btlib and if it is not handled there - an indication should be returned
	 *     to the jvm.
	 *
	 * so in both cases - first thing to do is to call ia32el's signal handler.
	 */ 
        if (*__clst_storage != NULL) /* to be sure we are on btlibed thread*/
            ret = (*g_ia32el_exports.ia32el_signal_handler) (signo, si, sc);

	if (ret == MME_IA32_SIGNAL_HANDLED) {
		/* ia32el handled the signal - we're done */
	} else {
		/* ia32el did not handle this signal */
		if (g_flags & MME_MTL_SIG_HANDLE) {
			/* if mme handles the signals then this signal should be
			 * forwarded to the orig handler */
			if (m_is_orig_handlers[signo]) {
				(*m_orig_handlers[signo].sa_sigaction)(signo, si, sc);
				ret = MME_IA32_SIGNAL_HANDLED;
			} else {
				/* or - if orig handler does not exist - remove
				 * mme's handler and return. in hw exceptions this
				 * exception will be repeated and kill the process
				 * as expected */
				syscall_retval = sigaction (signo, &m_orig_handlers[0], NULL);
				if (syscall_retval < 0) {
					DBCODE (true, perror ("sigaction failed"));
					abort ();
				}
			}
		}
	}

	return ret;
}

bool HIDDEN change_sighand (int32_t signo, struct sigaction * sc)
{
	m_orig_handlers[signo] = *sc;
	m_is_orig_handlers[signo] = true;

	return true;
}
