/* 
 * mme_upcall.c
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
#include <string.h>
#include <stdbool.h>
#include <setjmp.h>

#include <ia32el_mme_api.h>
#include <mme_internal.h>
#include <jni32.h>
#include <jni32_javavm.h>
#include <jni32_jnivm.h>

int64_t HIDDEN mme_upcall (ia32el_context32 * ec32, uint32_t code)
{
	int64_t error = 0;

	DBPRINT (0, "eip=%x esp=%x code=%x\n", ec32->eip, ec32->esp, code);
#if MMEDEBUG & 0
	{
		int32_t j;
		for (j = 0; j < 8; j ++) {
			uint32_t * p = (uint32_t *)I2P (ec32->esp - 4 + j * 4);
			fprintf (g_log_file, "\t%p: %x\n", p, *p);
		}
	}
#endif

	switch (code) {
		case IA32EL_TRAP_PHDR_ITER:
			error = process_phdr_iter_callback (ec32);
			break;
		case IA32EL_TRAP_JNI_CALL:
			error = jni32_jnienv_upcall (ec32);
			break;
		case IA32EL_TRAP_JAVAVM_CALL:
			error = jni32_javavm_upcall (ec32);
			break;
		case IA32EL_TRAP_JNIVM_CALL:
			error = jni32_jnivm_upcall (ec32);
			break;
		case IA32EL_TRAP_THREAD_FUNC:
			siglongjmp (g_thread_func_jmpbuf, 1);
			break;
		default:
			DBPRINT (true, "unknown trap code:%x\n", code);
	}
	DBPRINT (0, "error=%ld\n", error);

	return error;
}






