/*
 * ia32_ipc.c
 *
 * wrapper functions for syscalls that deal with ipc. 
 * common to both internal and external wrappers.
 *
 */
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 * Copyright (C) 2005       Gordon Jin <gordon.jin@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#include <ia32x_utils.h>
#include "user32.h"


long HIDDEN remove_shm_region (unsigned long addr);




long HIDDEN
sys32_sendfile (int out_fd, int in_fd, __u32 poff32, unsigned int count)
{
	extern long sendfile (int, int, off_t *, size_t);
	int    ret;
	off_t  off64 = 0;
	int    off32;

	if (IS_VALID_PTR(I2P(poff32))) {
		if (g_safe_memcpy (&off32, I2P(poff32), sizeof(int)) < 0) {
			return -EFAULT;
		}
		off64 = (off_t)off32;
	}

	ret = SYSCALL (sendfile, (out_fd, in_fd, 
		       VALID_PTR(I2P(poff32), &off64), count));

	ASSERT_HIGH_ZERO(off64);
	off32 = (int)off64;
	if ((ret == 0) && (poff32 != 0)) {
		if (g_safe_memcpy (I2P(poff32), &off32, sizeof(int)) < 0) {
			return -EFAULT;
		}
	}

	return ret;
}

