/* 
 * ia32_networking.c
 *
 * wrapper functions for syscalls related with networking.
 * common for both internal and external wrappers.
 *
 */
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/


#include <ia32x_utils.h>

#include "user32.h"
#include "ia32_networking.h"



static int
get_epoll_event (struct epoll_event32 *epev, struct epoll_event *epev64)
{
	int err = 0;
	struct epoll_event32 epev32;

	if (g_safe_memcpy (&epev32, epev, sizeof(epev32)) < 0) {
		return -EFAULT;
	}
	epev64->events = epev32.events;
	bt_memcpy(&epev64->data, &epev32.data_high, sizeof(epev64->data));
	
	return err;
}

static int
put_epoll_event (struct epoll_event32 *epev, struct epoll_event *epev64)
{
	int err = 0;
	struct epoll_event32 epev32;

	epev32.events = epev64->events;
	bt_memcpy(&epev32.data_high, &epev64->data, sizeof(epev64->data));
	if (g_safe_memcpy (epev, &epev32, sizeof(epev)) < 0) {
		err = -EFAULT;
	}
	
	return err;
}

long HIDDEN
sys32_epoll_ctl(int epfd, int op, int fd, __u32 event)
{
	int retval;
	struct epoll_event epev64;

	if (get_epoll_event((struct epoll_event32 *)I2P(event), &epev64) < 0) {
		return -EFAULT;
	}

	retval = SYSCALL (epoll_ctl, (epfd, op, fd, &epev64));
	
	return retval;
}

long HIDDEN
sys32_epoll_wait(int epfd, __u32 events, int maxevents, int timeout)
{
	int retval;
	int i;
	struct epoll_event *pepev64;

	if (maxevents <= 0) {
		return -EINVAL;
	}
	pepev64 = bt_malloc(maxevents * sizeof(struct epoll_event));

	if (pepev64 == NULL) {
		return -ENOMEM;
	}
	retval = LONG_SYSCALL4 (epoll_wait, epfd, pepev64, maxevents, timeout);
	if (retval < 0) {
		bt_free(pepev64);
		return retval;
	}
	for (i = 0; i < retval; i++) {
		if (put_epoll_event(&(((struct epoll_event32 *)I2P(events))[i]), 
				    &pepev64[i]) < 0) {
			return -EFAULT;
		}				
	}	
	bt_free(pepev64);
	return retval;
}

