/* 
 * ia32toia64.h
 *
 * interface of libia32toia64
 *
 */
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/


#ifndef _IA32TOIA64_H
#define _IA32TOIA64_H

#ifndef __ASSEMBLY__

struct memory_region
{
	unsigned long address;
	unsigned long len;
	unsigned prot;
	int dumpable;
	unsigned flags;
};

#if !RELEASE
// user-mem-space-extended-flag.
extern int extended_user_mem_flag; 
#endif

extern long iti_init (int page_size, long (*p_safe_memcpy)(void *, void *, unsigned long));
extern long iti_set_brk (unsigned end_code, unsigned brk_value);
extern long iti_get_memory_access_permissions (void *addr, unsigned *prot, 
                                               void **start, unsigned long *len);
extern void iti_get_ia32toia64_version (char *version_str);
extern long iti_before_fork (void);
extern long iti_after_fork (long is_parent);
extern long iti_long_syscall_interface (long **long_syscall_flag_p, 
                                        unsigned long *before_break_inst_p);
extern int  iti_enumerate_memory_regions (struct memory_region *mem_regions_buffer, 
                                          int max_memory_regions);
extern long iti_personality_interface (long **personality_p);
extern long iti_alloc_kernel_page (unsigned int prot);
extern long iti_closest_mem_region (unsigned long address, struct memory_region *buf);

#endif

/* values of long_syscall_flag */
#define NOT_IN_SYSCALL		0
#define CHECK_QUE		1
#define AFTER_CHECK_QUE		2
#define SIG_PENDING		3
#define BEFORE_BREAK		4
#define SYSCALL_COMPLETED	5
#define AFTER_SYSCALL		6


// Flag:btl_extended_user_mem indicate whether the memory is extended or not.
// 1:0-4G;0:0-3G
#define USER_SPACE_3G 0xc0000000
#define USER_SPACE_4G 0x100000000
#if !RELEASE  
#define access_ok(addr,size)  \
	({ int __ret; \
		if (extended_user_mem_flag){\
			__ret = ((addr) + (size) <= USER_SPACE_4G);\
		} \
		else{ \
			__ret = ((addr) + (size) <= USER_SPACE_3G);\
		}\
		__ret; \
	})
#define USER_SPACE_SIZE (extended_user_mem_flag ? USER_SPACE_4G : USER_SPACE_3G)
#else
#define access_ok(addr, size) ((addr) + (size) <= USER_SPACE_4G)
#define USER_SPACE_SIZE 0x100000000
#endif


#endif // _IA32TOIA64_H

