/*
 * conversion table from IA-32 syscall id's into 64 bit syscalls/wrapper funcs
 *
 */
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
#include "user32_ia32_syscall_prototypes.h"
#include "user32_const_for_asm.h"
#include "user32_pathwalk.h"

#define SYS_DIRECT(name) {CT_DIRECT,SYS_##name}
#define SYS_LONG(name) {CT_LONG,SYS_##name}
#define SYS_WRAPPER(name) {CT_WRAPPER,(unsigned long)name}
#if NOKERNELALTROOT
#define SYS_PWS(name) {CT_WRAPPER,(unsigned long)pws_##name}
#else
#define SYS_PWS(name) {CT_DIRECT,SYS_##name}
#endif

 
struct  {
	long is_ia64_syscall;
	unsigned long data;
} ia32_2_ia64[] = {

/*  0 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/*  1 exit*/			SYS_DIRECT(exit),
/*  2 fork*/			SYS_WRAPPER(sys32_ni_syscall),
/*  3 read*/			SYS_LONG(read),
/*  4 write*/			SYS_LONG(write),
/*  5 open*/			SYS_PWS(open),
/*  6 close*/			SYS_DIRECT(close),
/*  7 waitpid*/			SYS_WRAPPER(sys32_waitpid),
/*  8 creat*/			SYS_PWS(creat),
/*  9 link*/			SYS_PWS(link),
/* 10 unlink*/			SYS_PWS(unlink),
/* 11 xecve*/			SYS_WRAPPER(sys32_execve),
/* 12 chdir*/			SYS_PWS(chdir),
/* 13 time*/			SYS_WRAPPER(sys32_time),
/* 14 mknod*/			SYS_PWS(mknod),
/* 15 chmod*/			SYS_PWS(chmod),
/* 16 lchown*/			SYS_PWS(lchown),
/* 17 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/* 18 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/* 19 lseek*/			SYS_WRAPPER(sys32_lseek),
/* 20 getpid*/			SYS_DIRECT(getpid),
/* 21 mount*/			SYS_PWS(mount),
/* 22 oldumount*/		SYS_WRAPPER(sys32_oldumount),
/* 23 setuid*/			SYS_DIRECT(setuid),
/* 24 getuid*/			SYS_DIRECT(getuid),
/* 25 stime*/			SYS_WRAPPER(sys32_stime),
/* 26 ptrace*/			SYS_WRAPPER(sys32_ptrace),
/* 27 alarm*/			SYS_WRAPPER(sys32_alarm),
/* 28 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/* 29 pause*/			SYS_WRAPPER(sys32_pause),
/* 30 utime*/			SYS_WRAPPER(sys32_utime),
/* 31 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/* 32 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/* 33 access*/			SYS_PWS(access),
/* 34 nice*/			SYS_WRAPPER(sys32_nice),
/* 35 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/* 36 sync*/			SYS_DIRECT(sync),
/* 37 kill*/			SYS_DIRECT(kill),
/* 38 rename*/			SYS_PWS(rename),
/* 39 mkdir*/			SYS_PWS(mkdir),
/* 40 rmdir*/			SYS_PWS(rmdir),
/* 41 dup*/			SYS_DIRECT(dup),
/* 42 pipe*/			SYS_WRAPPER(sys32_pipe),
/* 43 times*/			SYS_WRAPPER(sys32_times),
/* 44 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/* 45 brk*/			SYS_WRAPPER(sys32_brk),
/* 46 setgid*/			SYS_DIRECT(setgid),
/* 47 getgid*/			SYS_DIRECT(getgid),
/* 48 signal*/			SYS_WRAPPER(sys32_signal),
/* 49 geteuid*/			SYS_DIRECT(geteuid),
/* 50 getegid*/			SYS_DIRECT(getegid),
/* 51 acct*/			SYS_PWS(acct),
/* 52 umount*/			SYS_PWS(umount),
/* 53 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/* 54 ioctl*/			SYS_WRAPPER(sys32_ioctl),
/* 55 fcntl*/			SYS_WRAPPER(sys32_fcntl),
/* 56 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/* 57 setpgid*/			SYS_DIRECT(setpgid),
/* 58 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/* 59 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/* 60 umask*/			SYS_DIRECT(umask),
/* 61 chroot*/			SYS_PWS(chroot),
/* 62 ustat*/			SYS_WRAPPER(sys32_ustat),
/* 63 dup2*/			SYS_DIRECT(dup2),
/* 64 getppid*/			SYS_DIRECT(getppid),
/* 65 getpgrp*/			SYS_WRAPPER(sys32_getpgrp),
/* 66 setsid*/			SYS_DIRECT(setsid),
/* 67 sigaction*/		SYS_WRAPPER(sys32_sigaction),
/* 68 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/* 69 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/* 70 setreuid*/		SYS_DIRECT(setreuid),
/* 71 setregid*/		SYS_DIRECT(setregid),
/* 72 sigsuspend*/		SYS_WRAPPER(sys32_sigsuspend),
/* 73 sigpending*/		SYS_WRAPPER(sys32_sigpending),
/* 74 sethostname*/		SYS_DIRECT(sethostname),
/* 75 setrlimit*/		SYS_WRAPPER(sys32_setrlimit),
/* 76 old_getrlimit*/		SYS_WRAPPER(sys32_old_getrlimit),
/* 77 getrusage*/		SYS_WRAPPER(sys32_getrusage),
/* 78 gettimeofday*/		SYS_WRAPPER(sys32_gettimeofday),
/* 79 settimeofday*/		SYS_WRAPPER(sys32_settimeofday),
/* 80 getgroups16*/		SYS_WRAPPER(sys32_getgroups16),
/* 81 setgroups16*/		SYS_WRAPPER(sys32_setgroups16),
/* 82 old_select*/		SYS_WRAPPER(sys32_old_select),
/* 83 symlink*/			SYS_PWS(symlink),
/* 84 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/* 85 readlink*/		SYS_PWS(readlink),
/* 86 uselib*/			SYS_DIRECT(uselib),
/* 87 swapon*/			SYS_PWS(swapon),
/* 88 reboot*/			SYS_DIRECT(reboot),
/* 89 readdir*/			SYS_WRAPPER(sys32_readdir),
/* 90 mmap*/			SYS_WRAPPER(sys32_mmap),
/* 91 munmap*/			SYS_WRAPPER(sys32_munmap),
/* 92 truncate*/		SYS_PWS(truncate),
/* 93 ftruncate*/		SYS_DIRECT(ftruncate),
/* 94 fchmod*/			SYS_DIRECT(fchmod),
/* 95 fchown*/			SYS_DIRECT(fchown),
/* 96 getpriority*/		SYS_WRAPPER(sys32_getpriority),
/* 97 setpriority*/		SYS_DIRECT(setpriority),
/* 98 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/* 99 statfs*/			SYS_WRAPPER(sys32_statfs),
/*100 fstatfs*/			SYS_WRAPPER(sys32_fstatfs),
/*101 ioperm*/			SYS_WRAPPER(sys32_ni_syscall),
/*102 socketcall*/		SYS_WRAPPER(sys32_ni_syscall), /*implemented in ia32x*/
/*103 syslog*/			SYS_DIRECT(syslog),
/*104 setitimer*/		SYS_WRAPPER(sys32_setitimer),
/*105 getitimer*/		SYS_WRAPPER(sys32_getitimer),
/*106 newstat*/			SYS_WRAPPER(sys32_newstat),
/*107 newlstat*/		SYS_WRAPPER(sys32_newlstat),
/*108 newfstat*/		SYS_WRAPPER(sys32_newfstat),
/*109 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/*110 iopl*/			SYS_WRAPPER(sys32_ni_syscall),
/*111 vhangup*/			SYS_DIRECT(vhangup),
/*112 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/*113 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/*114 wait4*/			SYS_WRAPPER(sys32_wait4),
/*115 swapoff*/			SYS_PWS(swapoff),
/*116 sysinfo*/			SYS_WRAPPER(sys32_sysinfo),
/*117 ipc*/			SYS_WRAPPER(sys32_ni_syscall),
/*118 fsync*/			SYS_DIRECT(fsync),
/*119 sigreturn*/		SYS_WRAPPER(sys32_sigreturn),
/*120 clone*/			SYS_WRAPPER(sys32_clone),
/*121 setdomainname*/		SYS_DIRECT(setdomainname),
/*122 newuname*/		SYS_WRAPPER(sys32_newuname),
/*123 modify_ldt*/		SYS_WRAPPER(sys32_ni_syscall),
/*124 adjtimex*/		SYS_WRAPPER(sys32_adjtimex),
/*125 mprotect*/		SYS_WRAPPER(sys32_mprotect),
/*126 sigprocmask*/		SYS_WRAPPER(sys32_sigprocmask),
/*127 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/*128 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/*129 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/*130 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/*131 quotactl*/		SYS_DIRECT(quotactl),
/*132 getpgid*/			SYS_DIRECT(getpgid),
/*133 fchdir*/			SYS_DIRECT(fchdir),
/*134 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/*135 sysfs*/			SYS_WRAPPER(sys32_sysfs),
/*136 personality*/		SYS_WRAPPER(sys32_personality),
/*137 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/*138 setfsuid*/		SYS_DIRECT(setfsuid),
/*139 setfsgid*/		SYS_DIRECT(setfsgid),
/*140 llseek*/			SYS_WRAPPER(sys32_llseek),
/*141 getdents*/		SYS_WRAPPER(sys32_getdents),
/*142 select*/			SYS_WRAPPER(sys32_select),
/*143 flock*/			SYS_DIRECT(flock),
/*144 msync*/			SYS_WRAPPER(sys32_msync),
/*145 readv*/			SYS_WRAPPER(sys32_ni_syscall),
/*146 writev*/			SYS_WRAPPER(sys32_ni_syscall),
/*147 getsid*/			SYS_DIRECT(getsid),
/*148 fdatasync*/		SYS_DIRECT(fdatasync),
/*149 sysctl*/			SYS_WRAPPER(sys32_ni_syscall),
/*150 mlock*/			SYS_WRAPPER(sys32_mlock),
/*151 munlock*/			SYS_WRAPPER(sys32_munlock),
/*152 mlockall*/		SYS_DIRECT(mlockall),
/*153 munlockall*/		SYS_DIRECT(munlockall),
/*154 sched_setparam*/		SYS_DIRECT(sched_setparam),
/*155 sched_getparam*/		SYS_WRAPPER(sys32_sched_getparam),
/*156 sched_setscheduler*/	SYS_DIRECT(sched_setscheduler),
/*157 sched_getscheduler*/	SYS_DIRECT(sched_getscheduler),
/*158 sched_yield*/		SYS_DIRECT(sched_yield),
/*159 sched_get_priority_max*/	SYS_DIRECT(sched_get_priority_max),
/*160 sched_get_priority_min*/	SYS_DIRECT(sched_get_priority_min),
/*161 sched_rr_get_interval*/	SYS_WRAPPER(sys32_sched_rr_get_interval),
/*162 nanosleep*/		SYS_WRAPPER(sys32_nanosleep),
/*163 mremap*/			SYS_WRAPPER(sys32_mremap),
/*164 setresuid*/		SYS_DIRECT(setresuid),
/*165 getresuid16*/		SYS_WRAPPER(sys32_getresuid16),
/*166 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/*167 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/*168 poll*/			SYS_WRAPPER(sys32_poll),
/*169 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/*170 setresgid*/		SYS_DIRECT(setresgid),
/*171 getresgid16*/		SYS_WRAPPER(sys32_getresgid16),
/*172 prctl*/			SYS_WRAPPER(sys32_prctl),
/*173 rt_sigreturn*/		SYS_WRAPPER(sys32_rt_sigreturn),
/*174 rt_sigaction*/		SYS_WRAPPER(sys32_rt_sigaction),
/*175 rt_sigprocmask*/		SYS_DIRECT(rt_sigprocmask),
/*176 rt_sigpending*/		SYS_WRAPPER(sys32_rt_sigpending),
/*177 rt_sigtimedwait*/		SYS_WRAPPER(sys32_rt_sigtimedwait),
/*178 rt_sigqueueinfo*/		SYS_WRAPPER(sys32_rt_sigqueueinfo),
/*179 rt_sigsuspend*/		SYS_LONG(rt_sigsuspend),
/*180 pread*/			SYS_WRAPPER(sys32_pread),
/*181 pwrite*/			SYS_WRAPPER(sys32_pwrite),
/*182 chown*/			SYS_PWS(chown),
/*183 getcwd*/			SYS_DIRECT(getcwd),
/*184 capget*/			SYS_WRAPPER(sys32_capget),
/*185 capset*/			SYS_WRAPPER(sys32_capset),
/*186 sigaltstack*/		SYS_WRAPPER(sys32_sigaltstack),
/*187 sendfile*/		SYS_WRAPPER(sys32_sendfile),
/*188 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/*189 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/*190 vfork*/			SYS_WRAPPER(sys32_ni_syscall),
/*191 getrlimit*/		SYS_WRAPPER(sys32_getrlimit),
/*192 mmap2*/			SYS_WRAPPER(sys32_mmap2),
/*193 truncate64*/		SYS_WRAPPER(sys32_truncate64),
/*194 ftruncate64*/		SYS_WRAPPER(sys32_ftruncate64),
/*195 stat64*/			SYS_WRAPPER(sys32_stat64),
/*196 lstat64*/			SYS_WRAPPER(sys32_lstat64),
/*197 fstat64*/			SYS_WRAPPER(sys32_fstat64),
/*198 lchown*/			SYS_DIRECT(lchown),
/*199 getuid*/			SYS_DIRECT(getuid),
/*200 getgid*/			SYS_DIRECT(getgid),
/*201 geteuid*/			SYS_DIRECT(geteuid),
/*202 getegid*/			SYS_DIRECT(getegid),
/*203 setreuid*/		SYS_DIRECT(setreuid),
/*204 setregid*/		SYS_DIRECT(setregid),
/*205 getgroups*/		SYS_WRAPPER(sys32_getgroups),
/*206 setgroups*/		SYS_DIRECT(setgroups),
/*207 fchown*/			SYS_DIRECT(fchown),
/*208 setresuid*/		SYS_DIRECT(setresuid),
/*209 getresuid*/		SYS_WRAPPER(sys32_getresuid),
/*210 setresgid*/		SYS_DIRECT(setresgid),
/*211 getresgid*/		SYS_WRAPPER(sys32_getresgid),
/*212 chown*/			SYS_DIRECT(chown),
/*213 setuid*/			SYS_DIRECT(setuid),
/*214 setgid*/			SYS_DIRECT(setgid),
/*215 setfsuid*/		SYS_DIRECT(setfsuid),
/*216 setfsgid*/		SYS_DIRECT(setfsgid),
/*217 pivot_root*/		SYS_PWS(pivot_root),
/*218 mincore*/			SYS_DIRECT(mincore),
/*219 madvise*/			SYS_DIRECT(madvise),
/*220 getdents64*/		SYS_DIRECT(getdents64),
/*221 fcntl64*/			SYS_WRAPPER(sys32_fcntl64),
/*222 tux*/			SYS_WRAPPER(sys32_ni_syscall),
/*223 secutiry*/		SYS_WRAPPER(sys32_ni_syscall),
/*224 gettid*/			SYS_DIRECT(gettid),
/*225 readahead*/		SYS_DIRECT(readahead),
/*226 setxattr*/		SYS_PWS(setxattr),
/*227 lsetxattr*/		SYS_PWS(lsetxattr),
/*228 fsetxattr*/		SYS_DIRECT(fsetxattr),
/*229 getxattr*/		SYS_PWS(getxattr),
/*230 lgetxattr*/		SYS_PWS(lgetxattr),
/*231 fgetxattr*/		SYS_DIRECT(fgetxattr),
/*232 listxattr*/		SYS_PWS(listxattr),
/*233 llistxattr*/		SYS_PWS(llistxattr),
/*234 flistxattr*/		SYS_DIRECT(flistxattr),
/*235 removexattr*/		SYS_PWS(removexattr),
/*236 lremovexattr*/		SYS_PWS(lremovexattr),
/*237 fremovexattr*/		SYS_DIRECT(fremovexattr),
/*238 tkill*/			SYS_DIRECT(tkill),
/*239 sendfile64*/		SYS_WRAPPER(sys32_sendfile64),
/*240 futex*/			SYS_WRAPPER(sys32_ni_syscall),
/*241 sched_setaffinity*/	SYS_DIRECT(sched_setaffinity),
/*242 sched_getaffinity*/	SYS_DIRECT(sched_getaffinity),
/*243 set_thread_area*/		SYS_WRAPPER(sys32_ni_syscall),
/*244 get_thread_area*/		SYS_WRAPPER(sys32_ni_syscall),
/*245 io_setup*/		SYS_WRAPPER(sys32_io_setup),
/*246 io_destroy*/		SYS_WRAPPER(sys32_io_destroy),
/*247 io_getevents*/		SYS_WRAPPER(sys32_io_getevents),
/*248 io_submit*/		SYS_WRAPPER(sys32_io_submit),
/*249 io_cancel*/		SYS_WRAPPER(sys32_io_cancel),
/*250 fadvise64*/		SYS_DIRECT(fadvise64),
/*251 free_hugepages*/		SYS_WRAPPER(sys32_ni_syscall),
/*252 exit_group*/		SYS_DIRECT(exit_group),
/*253 lookup_dcookie*/		SYS_DIRECT(lookup_dcookie),
/*254 epoll_create*/		SYS_DIRECT(epoll_create),
/*255 epoll_ctl*/		SYS_WRAPPER(sys32_epoll_ctl),
/*256 epoll_wait*/		SYS_WRAPPER(sys32_epoll_wait),
/*257 remap_file_pages*/	SYS_WRAPPER(sys32_ni_syscall),
/*258 set_tid_address*/		SYS_DIRECT(set_tid_address),
/*259 timer_create*/		SYS_WRAPPER(sys32_timer_create),
/*260 timer_settime*/		SYS_WRAPPER(sys32_timer_settime),
/*261 timer_gettime*/		SYS_WRAPPER(sys32_timer_gettime),
/*262 timer_getoverrun*/	SYS_DIRECT(timer_getoverrun),
/*263 timer_delete*/		SYS_DIRECT(timer_delete),
/*264 clock_settime*/		SYS_WRAPPER(sys32_clock_settime),
/*265 clock_gettime*/		SYS_WRAPPER(sys32_clock_gettime),
/*266 clock_getres*/		SYS_WRAPPER(sys32_clock_getres),
/*267 clock_nanosleep*/		SYS_WRAPPER(sys32_clock_nanosleep),
/*268 statfs64*/		SYS_WRAPPER(sys32_ni_syscall/*sys32_statfs64*/),
/*269 fstatfs64*/		SYS_WRAPPER(sys32_fstatfs64),
/*270 tgkill*/			SYS_DIRECT(tgkill),
/*271 utimes*/			SYS_WRAPPER(sys32_utimes),
/*272 fadvise64_64*/		SYS_DIRECT(fadvise64),
/*273 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/* the three following system calls are not implemented at 
 * this stage because currently none of the linux kernels 
 * support them */
/*274 mbind*/			SYS_WRAPPER(sys32_ni_syscall),
/*275 et_mempolicy*/	        SYS_WRAPPER(sys32_ni_syscall),
/*276 set_mempolicy*/		SYS_WRAPPER(sys32_ni_syscall),
/*277 mq_open*/			SYS_WRAPPER(sys32_mq_open),
/*278 mq_unlink*/		SYS_DIRECT(mq_unlink),
/*279 mq_timedsend*/		SYS_WRAPPER(sys32_mq_timedsend),
/*280 mq_timedreceive*/		SYS_WRAPPER(sys32_mq_timedreceive),
/*281 mq_notify*/		SYS_WRAPPER(sys32_mq_notify),
/*282 mq_getsetattr*/		SYS_WRAPPER(sys32_mq_getsetattr),
/*283 ni_syscall*/		SYS_WRAPPER(sys32_ni_syscall),
/*284 waitid*/			SYS_WRAPPER(sys32_waitid),
/*285 setalroot*/		SYS_WRAPPER(sys32_ni_syscall), /*the syscall is not impelement by IA-32 kenerl now*/
/*286 add_key*/			SYS_WRAPPER(sys32_ni_syscall),
/*287 request_key*/		SYS_WRAPPER(sys32_ni_syscall),
/*288 keyctl*/			SYS_WRAPPER(sys32_ni_syscall),
/*289 ioprio_set*/		SYS_DIRECT(ioprio_set),
/*290 ioprio_get*/		SYS_DIRECT(ioprio_get),
/*291 inotify_init*/		SYS_DIRECT(inotify_init),
/*292 inotify_add_watch*/	SYS_DIRECT(inotify_add_watch),
/*293 inotify_rm_watch*/	SYS_DIRECT(inotify_rm_watch),
/*294 migrate_pages*/		SYS_WRAPPER(sys32_ni_syscall),
/*295 openat*/			SYS_DIRECT(openat),
/*296 mkdirat*/			SYS_DIRECT(mkdirat),
/*297 mknodat*/			SYS_DIRECT(mknodat),
/*298 fchownat*/		SYS_DIRECT(fchownat),
/*299 futimesat*/		SYS_WRAPPER(sys32_futimesat),
/*300 fstatat64*/		SYS_WRAPPER(sys32_newfstatat),
/*301 unlinkat*/		SYS_DIRECT(unlinkat),
/*302 renameat*/		SYS_DIRECT(renameat),
/*303 linkat*/			SYS_DIRECT(linkat),
/*304 symlinkat*/		SYS_DIRECT(symlinkat),
/*305 readlinkat*/		SYS_DIRECT(readlinkat),
/*306 fchmodat*/		SYS_DIRECT(fchmodat),
/*307 faccessat*/		SYS_DIRECT(faccessat),
/*308 pselect*/			SYS_WRAPPER(sys32_pselect),
/*309 ppoll*/			SYS_WRAPPER(sys32_ppoll),
/*310 unshare*/			SYS_DIRECT(unshare), 
/*311 set_robust_list*/		SYS_WRAPPER(sys32_ni_syscall),
/*312 get_robust_list*/		SYS_WRAPPER(sys32_ni_syscall),
/*313 splice*/			SYS_WRAPPER(sys32_ni_syscall),
/*314 sync_file_range*/		SYS_DIRECT(sync_file_range),
/*315 tee*/			SYS_DIRECT(tee),
/*316 vmsplice*/		SYS_WRAPPER(sys32_ni_syscall),
/*317 move_pages*/		SYS_WRAPPER(sys32_move_pages),
/*318 getcpu*/			SYS_WRAPPER(sys32_getcpu),
/*319 epoll_pwait*/		SYS_DIRECT(epoll_pwait),
/*320 utimensat*/		SYS_WRAPPER(sys32_utimensat),
/*321 signalfd*/		SYS_WRAPPER(sys32_ni_syscall),
/*322 timerfd_create*/		SYS_DIRECT(timerfd_create),
/*323 eventfd*/			SYS_DIRECT(eventfd),
/*324 fallocate*/		SYS_WRAPPER(sys32_fallocate),
/*325 timerfd_settime*/		SYS_WRAPPER(sys32_timerfd_settime),
/*326 timerfd_gettime*/		SYS_WRAPPER(sys32_timerfd_gettime),
/*327 signalfd4*/		SYS_WRAPPER(sys32_ni_syscall),
/*328 eventfd2*/		SYS_WRAPPER(sys32_ni_syscall),
/*329 epoll_create1*/		SYS_WRAPPER(sys32_ni_syscall),
/*330 dup3*/			SYS_DIRECT(dup3),
/*331 pipe2*/			SYS_WRAPPER(sys32_pipe2),
/*332 inotify_init1*/		SYS_WRAPPER(sys32_ni_syscall),
/* following system calls are not supported by IA-32 EL now */
/* NEXT LINE MUST BE THE LAST IN THE TABLE */
/* end of table */		{-1, 0}
};

