/*
 * user32_mqueue.c
 *
 * mqueue system calls
 *
 */
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Abigail Tarem <abigail.tarem@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#include <ia32x_utils.h>
#include "user32.h"


/* from  linux/mqueue.h */

struct mq_attr {
	long    mq_flags;
	long    mq_maxmsg;
	long    mq_msgsize;
	long    mq_curmsgs;
	long    __reserved[4];

};

struct mq_attr32 {
	int    mq_flags;
	int    mq_maxmsg;
	int    mq_msgsize;
	int    mq_curmsgs;
	int    __reserved[4];

};

long HIDDEN
sys32_mq_open (const char* u_name, int oflag, __mode_t32 mode, __u32 u_attr)
{
	struct mq_attr mq64;
	struct mq_attr32 mq32;
        
	if (IS_VALID_PTR(I2P(u_attr))) {
		if (g_safe_memcpy (&mq32, I2P(u_attr), sizeof(mq32)) < 0) {
                        return -EFAULT;
		}
	mq64.mq_flags = mq32.mq_flags;
	mq64.mq_maxmsg = mq32.mq_maxmsg;
	mq64.mq_msgsize = mq32.mq_msgsize;
	mq64.mq_curmsgs = mq32.mq_curmsgs;
	/*mq64.__reserved = mq32->__reserved;*/
	}

	return (int)SYSCALL (mq_open, ((char*)u_name, oflag, mode, 
		VALID_PTR(I2P(u_attr),&mq64)));


}


long HIDDEN
sys32_mq_timedsend (int mqdes, const char* u_msg_ptr, __size_t32 msg_len,  
					unsigned int msg_prio, const __u32 u_abs_timeout)
{
	struct timespec32 ts32;
	struct timespec ts64;
        
	if (IS_VALID_PTR(I2P(u_abs_timeout))) {
		if (g_safe_memcpy (&ts32, I2P(u_abs_timeout), sizeof(ts32)) < 0) {
                        return -EFAULT;
		}
		ts64.tv_sec = ts32.tv_sec;
		ts64.tv_nsec = ts32.tv_nsec;
	}
	return (int)LONG_SYSCALL5 (mq_timedsend, mqdes, u_msg_ptr, msg_len, msg_prio, 
		VALID_PTR(I2P(u_abs_timeout),&ts64));
}


int HIDDEN
sys32_mq_timedreceive (int mqdes, char* u_msg_ptr, __size_t32 msg_len, 
		       unsigned int* u_msg_prio, const __u32 u_abs_timeout)
{
	struct timespec32 ts32;
	struct timespec ts64;
        
	if (IS_VALID_PTR(I2P(u_abs_timeout))) {
		if (g_safe_memcpy (&ts32, I2P(u_abs_timeout), sizeof(ts32)) < 0) {
                        return -EFAULT;
		}
		ts64.tv_sec = ts32.tv_sec;
		ts64.tv_nsec = ts32.tv_nsec;
	}
	return (int)LONG_SYSCALL5 (mq_timedreceive, mqdes, u_msg_ptr, msg_len, u_msg_prio, 
		VALID_PTR(I2P(u_abs_timeout),&ts64));
	
}

long HIDDEN
sys32_mq_notify (int mqdes, const __u32 u_notification)
{
	struct sigevent se64;
	struct sigevent32 se32;

	if (IS_VALID_PTR(I2P(u_notification))) {
		if (g_safe_memcpy(&se32, I2P(u_notification), sizeof(se32)) < 0) {
			return -EFAULT;
		}
		se64.sigev_value.sival_int = se32.sigev_value.sival_int;
		se64.sigev_value.sival_ptr = I2P(se32.sigev_value.sival_ptr);
		se64.sigev_signo = se32.sigev_signo;
		se64.sigev_notify = se32.sigev_notify;
		bt_memcpy (se64._sigev_un._pad, se32._sigev_un._pad, sizeof(se64._sigev_un._pad));
	}

	return (int)SYSCALL (mq_notify, (mqdes, VALID_PTR(I2P(u_notification),&se64)));

}

long HIDDEN
sys32_mq_getsetattr (int mqdes, const __u32 u_mqstat, __u32 u_omqstat)
{
	int retval;
	struct mq_attr mq64;
	struct mq_attr omq64;
	struct mq_attr32 mq32;
        
	if (IS_VALID_PTR(I2P(u_mqstat))) {
		if (g_safe_memcpy (&mq32, I2P(u_mqstat), sizeof(mq32)) < 0) {
                        return -EFAULT;
		}
	mq64.mq_flags = mq32.mq_flags;
	mq64.mq_maxmsg = mq32.mq_maxmsg;
	mq64.mq_msgsize = mq32.mq_msgsize;
	mq64.mq_curmsgs = mq32.mq_curmsgs;
	/*mq64.__reserved = mq32->__reserved;*/
	}

	retval = SYSCALL (mq_getsetattr, (mqdes, VALID_PTR(I2P(u_mqstat),&mq64), 
		VALID_PTR(I2P(u_omqstat),&omq64)));

	if (retval < 0) return retval;
	
	if (IS_VALID_PTR(I2P(u_omqstat))) {
		struct mq_attr32 omq32;

		omq32.mq_flags = omq64.mq_flags;
		omq32.mq_maxmsg = omq64.mq_maxmsg;
		omq32.mq_msgsize = omq64.mq_msgsize;
		omq32.mq_curmsgs = omq64.mq_curmsgs;
		/*omq32.__reserved = omq64->__reserved;*/

		if (g_safe_memcpy (I2P(u_omqstat), &omq32, sizeof(omq32)) < 0)
			return -EFAULT;
	}
	return retval;
}

