/* 
 * btlib_main.c
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#include <ia32x_utils.h>

#include "btlib.h"
#include "btlib_sync.h"
#include "btload.src/os_dep_linux.h"
#include "btload.src/new_pe.h"
#include "btload.src/btload_pub.h"
#include "btlib_stats.h"
#include "btlib_ptemu.h"
#include "btlib_syscall.h"

#include <ia32toia64.h>

#if DEBUG
/* Config file and log file handles */
char btl_prt_cs = 0;
char btl_prt_api = 0;
char btl_prt_sig = 0;
char btl_prt_pt = 0;
char btl_prt_mem = 0;
char btl_prt_mem_region = 0;
char btl_prt_thread = 0;
char btl_prt_syscall = 0;
#endif				/* DEBUG */
#if ! RELEASE
char btl_prt_dll = 0;
char btl_no_dll_reuse = 0;
char btl_no_affinity = 0;
char btl_extended_user_mem = 1;
/* specify if always set READ_IMPLIES_EXEC */
char btl_always_rie = 0;
char btl_disable_smcopt = 0;
#endif

#define MAX_SWITCHES            16
#define MAX_SWITCH_LEN          1024
#define MAX_INI_BUF_SIZE        MAX_SWITCHES * MAX_SWITCH_LEN

/* globals */
unsigned btl_clst_glst_size = 0;	/* btgeneric's clst+glst size */
unsigned long btl_page_size = 0ul;	/* system page size */
void *btl_btg_load_address = (void *) BTGENERIC_LOAD_ADDRESS;
unsigned long btl_btg_size = 0ul;
char btl_ia32_bin_full_path[PATH_MAX] = { 0 };	/* full path to IA-32 executable binary */
char *btl_ia32_bin_name = NULL;	/* IA-32 executable binary name */
int btl_btg_initialized = 0;
btrans_binary_map_t g_application_info;	/* IA-32 application information */
char **g_envp;
unsigned long btl_num_threads = 1;

#if ALL_THREAD_AFFINITY
#if DEBUG
unsigned long btl_cycles_in_second;
#endif
unsigned long btl_cycles_between_ts_checks;
#endif


/* information for locating BTGeneric */
#define HOME		"HOME="
#define BTGENERIC_EFI	"ia32exec.bin"
char *g_search_path[] = {
	"/usr/local/bin",
	"/usr/bin",
	"/bin",
	"/usr/lib/ia32el",
	""
};
static char m_btg_filename[PATH_MAX] = { 0 };

#if !RELEASE
static char m_ini_file[PATH_MAX] = { 0 };
#endif
#define INI_FILE_NAME "Btrans.ini"
#define LOG_FILE_NAME "Btrans.log"

#define RLIM_NOFILE_EXT 2

/* function prototypes */
static void print_ver (char *btg_ver);
static long btlib_init (int, char **, char **, btrans_binary_map_t *, long *);
static long set_btgeneric_path (char *dest, unsigned long dest_size,
				char **envp);
#if ! RELEASE
static long read_config_file (char *argv0, char *btrans_ini_switches[]);
static void open_log_file (char *line, char *argv0, char *logdir);
static void print_version_to_log (void);
#endif
static long load_and_init_btgeneric (char **argv RELEASE_UNUSED, char **envp,
				     long is_ver, char btg_ver[PATH_MAX]);
static void report_initial_code_areas (btrans_binary_map_t * app_info);
static void extract_unicode_str (char *ver_str_p, char *ver_str,
				 unsigned long ver_str_size);
static void get_version_str (void *rsrc, unsigned long size, char *ver_str,
			     unsigned long ver_str_size);
static char *find_unicode_str (char *base_addr, unsigned long limit, char *str);
static int shift_argv (int argc, char *argv[]);

#if ALL_THREAD_AFFINITY
#if DEBUG
#include "btlib_asm.h"
#endif
#endif

unsigned long g_stack_size = 0ul;

/*****************************************************************************/
/*****************************************************************************/
int btlib_load_and_exec (unsigned long ptrace_flag, int argc, char *argv[],
			 char *envp[]);
int
btlib_load_and_exec (unsigned long ptrace_flag, int argc, char *argv[],
		     char *envp[])
{
	/* btl_is_ptraced should be initialized before the call to main_thread_init */
	btl_is_ptraced = ptrace_flag;
	return internal_load_and_exec (argc, argv, envp);
}

int
internal_load_and_exec (int argc, char *argv[], char *envp[])
{
	long retval;
	long is_ver = 0;
	char btg_ver[PATH_MAX];
	long personality;
	unsigned long mhz;

#if (DEBUG && 0)
	{
		int k;
		for (k = 0; k < argc; k++) {
			DBPRINT (1, "\targv[%d]='%s'\n", k, argv[k]);
		}
	}
#endif
	/* 
	 * argv[0] is the translator's name
	 * argv[1] is full path to the translated process
	 * argv[2] is the original argv[0]
	 */
	if (argc < 2) {
		retval = 0;
		goto out;
	}
	g_log_file = -1;

	/* check validity of arguments */
	if ((argv[1][0] == '-') && (argv[1][1] == 'v')) {
		if (argv[1][2] != '\0') {
			retval = -EINVAL;
			goto out;
		}
		is_ver = 1;
	}

	btl_page_size = SYSCALL (getpagesize, ());

	/* initialize the IA-32 syscall wrapper library - it is needed 
	 * in load_elf32_binary */
	retval = iti_init ((int) btl_page_size, safe_memcpy);
	if (retval < 0) {
		DBPRINT (TRUE, "ia32toia64_init failed\n");
		return retval;
	}

	/* load and initialize btgeneric */
	retval = load_and_init_btgeneric (argv, envp, is_ver, btg_ver);

	if (is_ver) {
		print_ver ((retval < 0) ? NULL : btg_ver);
		retval = 0;
		goto out;
	}
	if (retval < 0) {
		DBPRINT (TRUE, "load_and_init_btgeneric failed\n");
		retval = -ENOEXEC;
		notify_error (NO_BTGENERIC);
		goto out;
	}

	/* prepare the tsd of the main thread - don't emit any debug
	 * message before this point */
	retval = btlib_main_thread_init_tsd ();
	if (retval < 0) {
		goto out;
	}

	/* initialize btlib */
	retval = btlib_init (argc, argv, envp, &g_application_info, &personality);
	if (retval < 0) {
		DBPRINT (TRUE, "btlib_init failed\n");
		goto out;
	}


	/* prepare the current thread for running the IA-32 application */
	retval = btlib_main_thread_init (&g_application_info, personality);
	if (retval < 0) {
		goto out;
	}

	/* report to btgeneric about the initial code areas 
	 * (binary and dynamic loader) */
	report_initial_code_areas (&g_application_info);

	if (!g_is_mme_mode) {
		/* remove first two arguments from argv so 'ps -ef' will show the
		 * proess name and not the translator's name */
		shift_argv (argc, argv);
	}
	/* silent the misaligned-access messages */
	retval = SYSCALL (prctl, (PR_SET_UNALIGN, (unsigned long) PR_UNALIGN_NOPRINT));

#if DEBUG
	if (IS_SYS_ERROR (retval)) {
		DBPRINT (TRUE, "prctl failed:%ld\n", retval);
	}
#endif

#if PTEMU_ENABLED

	DBPRINT (btl_prt_pt, "btl_is_ptraced=%ld r13=%p blst=%p clst=%p\n",
		 btl_is_ptraced, __r13, cur_thread_data, cur_thread_clst);
	/* if this process is traced then suspend yourself, simulate the behavior when 
	   a ptraced process call execve */
	if (btl_is_ptraced != IS_PTRACED_NO) {
		assert (cur_thread_data->tid == SYSCALL (gettid, ()));
		/* set execution policy of debuggee */
		{
			EMULATION_POLICY ep = {
				/* memory order should not be changed */
				memory_order_policy:USE_ORDERED_LD_ST,
				/* don't optimize away changes in eflag */
				strict_state_policy:STATE_POLICY_DEBUGEE,
			};
			BTGENERIC_SET_EMULATION_POLICY (&ep);
			assert (ep.strict_state_policy == STATE_POLICY_DEBUGEE);
			/* we don't need flush instruction cache here because there 
			   are no translations yet */
		}

		/* suspend */
		cur_thread_data->tia_signal.siginfo.si_signo = SIGTRAP;
		retval = btlib_ptemu_sig0_selfkill (TRUE);
		sys_err_chk (retval, "kill");
	}
#endif

#ifndef INITIAL_IA32_STACK_SIZE
#define INITIAL_IA32_STACK_SIZE	(MAX_ARG_PAGES*IA32_PAGE_SIZE)
#endif	
	/* disable smc prot on stack, it should be the same the the stack in setup_elf32_stack*/
	assert (g_stack_size >= INITIAL_IA32_STACK_SIZE);
	btlib_disable_smc_opt((IA32_STACK_TOP - g_stack_size), g_stack_size); 

	if (g_is_mme_mode) {
		/* as btlib cannot change affinity when running in mme
		 * mode - need to enforce strict memory ordering */
		EMULATION_POLICY ep = {
			/* memory order should not be changed */
			memory_order_policy:USE_ORDERED_LD_ST,
			/* don't optimize away changes in eflag */
			strict_state_policy:POLICY_UNCHANGED,
		};
		BTGENERIC_SET_EMULATION_POLICY (&ep);
		btl_did_ordered_code = 1;
	}

#if ALL_THREAD_AFFINITY
	mhz = btlib_get_cpu_mhz ();
#if DEBUG
	btl_cycles_in_second = mhz * 1000000L;
	btl_start_time = __btlib_read_timestamp_counter ();
#endif
	btl_cycles_between_ts_checks = 
		(unsigned long) (mhz * 1000000 * FRACTION_OF_SECOND_BETWEEN_THREAD_STATE_CHECKS);
#endif
	/* enter the IA-32 process */
	btl_btg_initialized = 1;
       /* check the initalization */
#ifdef DEBUG
	{
		unsigned long curbrk;
		curbrk = SYSCALL (brk, (0ul));
		DBPRINT(btl_prt_mem, "current brk = 0x%lx\n", curbrk);
		assert(!IS_BTGENERIC_MEM_ADDR(curbrk));
	}
#endif

	(void) btrans_jump ();

	DBPRINT (!g_is_mme_mode, "SHOULD NOT GET HERE !!!\n");
	assert (g_is_mme_mode);

	retval = 0;
out:
	return retval;
}

/*****************************************************************************/
/*
 * static int * btlib_init(char ** envp)
 *
 * Purpose: initialize btlib 
 *
 * Input:   pointer to stacked arguments (as passed to main)
 * Output:  ia32stack pointer
 *
 */
/*****************************************************************************/
static long
btlib_init (int argc, char **argv, char **envp,
	    btrans_binary_map_t * app_info UNUSED, long *personality UNUSED)
{
	long retval;

	/* init the sync module */
	btlib_init_sync_module ();

	/* init the memory module */
	btlib_init_memory_module ();

	/* init the signal module */
	btlib_init_signal_module ();

	/* init thread module */
	btlib_init_thread_module ();

#if PTEMU_ENABLED
	/* init ptrace-emulation module */
	btlib_init_ptemu_module ();
#endif

	/* 1. Initialize globals . */
	bt_memset (app_info, 0, sizeof (*app_info));

	/* 2. load the IA-32 binary */
	assert (argc > 1);
	retval = load_elf32_binary (argc, argv, envp, app_info, personality);
	if (retval < 0) {
		return retval;
	}
#if (DEBUG & 0)
	fprintf (g_log_file, "app_info:\n");
	fprintf (g_log_file, "\tstart_brk=%x\n", app_info->start_brk);
	fprintf (g_log_file, "\tstart_code=%x\n", app_info->start_code);
	fprintf (g_log_file, "\tend_code=%x\n", app_info->end_code);
	fprintf (g_log_file, "\tstart_data=%x\n", app_info->start_data);
	fprintf (g_log_file, "\tend_data=%x\n", app_info->end_data);
	fprintf (g_log_file, "\tstart_stack=%x\n", app_info->start_stack);
	fprintf (g_log_file, "\tbrk=%x\n", app_info->brk);
	fprintf (g_log_file, "\tentry_point=%x\n", app_info->entry_point);
	fprintf (g_log_file, "\targ_start=%x\n", app_info->arg_start);
	fprintf (g_log_file, "\targ_end=%x\n", app_info->arg_end);
	fprintf (g_log_file, "\tenv_start=%x\n", app_info->env_start);
	fprintf (g_log_file, "\tenv_end=%x\n", app_info->env_end);
#endif				/* DEBUG */

	/* set the brk values in the IA-32 syscall wrapper library */
	retval = iti_set_brk (app_info->end_code, app_info->brk);
	if (retval < 0) {
		DBPRINT (TRUE, "set_brk_values failed\n");
		return -1;
	}

	/* below func uses IA-32 syscall wrapper, so put it here */
	retval = btlib_init_syscall_module ();
	if (retval < 0) {
		DBPRINT (TRUE, "btlib_init_syscall_module failed\n");
		return -1;
	}

	return 0;
}

/*****************************************************************************/
/*
 * int btrans_jump
 *
 * Purpose: Jump into BTGeneric_Run
 *
 * Input: thread->longjmp_status indicates why we are here:
 *          RUN = just jump to BTGeneric
 *          SIGNAL = call the signal raising function
 *          DELIVER = call the signal delivery function
 *
 * Output: returns -1 if btgeneric is not initialized, 0 otherwise
 *
 */
/*****************************************************************************/
int
btrans_jump ()
{
	int status;
	long retval;
	jmp_buf btgeneric_run_context __attribute__ ((aligned (16)));
	jmp_buf *previous_context;

	assert (cur_thread_data->tid == SYSCALL (gettid, ()));

	if (!btl_btg_initialized) {
		assert (!"btgeneric is not initialized");
		return -1;
	}

	previous_context = cur_thread_data->btgeneric_run_context;
	cur_thread_data->btgeneric_run_context = &btgeneric_run_context;
	DBPRINT (0, "ctxt save %p -> %p\n",
		 previous_context, btgeneric_run_context);

	status = sigsetjmp (*cur_thread_data->btgeneric_run_context, 0);

	/* get the thread's specific data (in case we got here from __siglongjmp) */
	assert (cur_thread_data->tid == SYSCALL (gettid, ()));

	DBPRINT (0, "status=%d ptrace=%lx\n", status, get_cur_thread_ptrace ());

	STATCODE (btl_stats, btlib_update_stat (BTL_STAT_BTGJUMP, COUNT, 0ul));

	if (status == 0) {
		cur_thread_data->is_run_context_ready = cur_thread_data->tid;
	}
	cur_thread_data->in_nonblocking_syscall = FALSE;
	/* 
	 * this is the debugee's part of the algorithm for sync_self_canonization.
	 * look at the header of this function in btlib_ptemu.c
	 */
	if (get_cur_thread_ptrace () & PTEMU_PTRACED) {
		int cont_signo;

		assert (!g_is_mme_mode);
		if (get_cur_thread_ptrace () & PTEMU_SELF_CANONIZE) {
			int semid;
			struct sembuf semops[2];

			(*cur_thread_ptrace ()) &= ~PTEMU_SELF_CANONIZE;

			DBPRINT (btl_prt_pt, "self canonization\n");

			/* we should get here only thru BtlReenter */
			assert (status == RUN);

			/* Open the semaphore (allocated by the debugger) */
			semid = SYSCALL (semget, (cur_thread_data->self_canon_semkey, 2, 0600));
			sys_err_chk ((long) semid, "semget");

			/* Wake the debugger */
			semops[0].sem_num = SEMREADY;

			/* TODO: 1 should be enough, but it's not */
			semops[0].sem_op = 2;
			semops[0].sem_flg = 0;
			retval = SYSCALL (semop, (semid, semops, 1));

			/* wait for the debugger */
			DBPRINT (btl_prt_pt, "before wait\n");
			semops[0].sem_num = SEMRESUME;
			semops[0].sem_op = -1;
			semops[0].sem_flg = SEM_UNDO;
			/* don't let signals interrupt us here */
			do {
				retval = SYSCALL (semop, (semid, semops, 1));
			} while (retval == -EINTR);
			sys_err_chk (retval, "semop");
			DBPRINT (btl_prt_pt, "after wait\n");

			/* Remove the semaphore set */
			retval = SYSCALL (semctl, (semid, 0, IPC_RMID, (union semun) 0));
			sys_err_chk (retval, "semctl");
		}
		/* get the signal to continue with */
		cont_signo = GET_LO_DWORD (get_cur_thread_ptrace ());
		SET_LO_DWORD (cur_thread_ptrace(), 0);
		DBPRINT (btl_prt_pt, "cont_signo=%d\n", cont_signo);
		if (cont_signo != 0) {
			if (cont_signo == PTEMU_SIG0) {
				retval = btlib_ptemu_sig0_selfkill (TRUE);
				sys_err_chk (retval, "suspend self");
			} else {
				btlib_prepare_cont_signo (cont_signo);
				status = SIGNAL;
			}
		}

		btlib_check_segment (cur_thread_data->bc32);
	}

	switch (status) {
	case 0:
	case RUN:
		DBPRINT (0, "eip=%x esp=%x ebp=%x\n",
			 cur_thread_data->bc32->eip,
			 cur_thread_data->bc32->esp,
			 cur_thread_data->bc32->ebp);
#ifdef ALL_THREAD_AFFINITY
		cur_thread_data->is_in_simulation_session = 1;
		if (SHOULD_TRACK_THREAD_SCLBLTY) {
			btlib_thread_scalability_analysis ();
		}
#endif
		assert (!btlib_is_inside_lock (cur_thread_data));
		FINISH_CONT_PROCESS ();
		BTGENERIC_RUN ();
		break;
	case DELIVER:
		btlib_deliver_signals ();
		break;
	case SIGNAL:
		btlib_raise_signal ();
		break;
	case IA32_SYSCALL:
		btlib_system_call ();
		break;
	case TRAP:
		btlib_emulate_trap ();
		break;
	case RETURN:
		assert (g_is_mme_mode);
		cur_thread_data->btgeneric_run_context = previous_context;
		return 0;
	default:
		assert (!"unknown status");
	}
	assert (!"should never get here");
	return -2;
}

/*****************************************************************************/
/*
 * set_btgeneric_path:
 * search ia32exec.bin in order: predefined path, HOME, current dir
 */
/*****************************************************************************/
static long
set_btgeneric_path (char *dest, unsigned long dest_size, char **envp UNUSED)
{
	long retval = -1;
	struct stat stat_buf[3];	/* 3 due to different struct defs in the old headers */
	char buff[PATH_MAX];
	char **search_path;

	assert (sizeof (buff) >= dest_size);

	/* assign here to avoid unused warning when SYSBTG=0 */
	search_path = g_search_path;

	if (envp != (char **) NULL) {
		g_envp = envp;
	}
#if SYSBTG
	while (bt_strnlen (*search_path, (unsigned long) PATH_MAX) != 0) {
		bt_strncpy (buff, *search_path, sizeof (buff) - 1);
		bt_strncat (buff, "/" BTGENERIC_EFI,
			    MIN (sizeof (buff) - bt_strlen (buff) - 1,
				 bt_strlen (BTGENERIC_EFI) + 1));
		retval = SYSCALL (stat, (buff, stat_buf));

		if (retval == 0) {
			bt_strncpy (dest, buff, dest_size - 1);
#if !RELEASE
			bt_snprintf (m_ini_file, sizeof (m_ini_file) - 1,
				     "%s/" INI_FILE_NAME, *search_path);
#endif
			break;
		} else {
			retval = -1;
			search_path++;
		}
	}
#else
	/* try current directory first */
	assert (bt_strlen ("./" BTGENERIC_EFI) < sizeof (buff) - 1);
	bt_strncpy (buff, "./" BTGENERIC_EFI, sizeof (buff) - 1);
	retval = SYSCALL (stat, (buff, stat_buf));
	if (retval == 0) {
		bt_strncpy (dest, buff, dest_size - 1);
		goto out;
	}

	/* try to look for the HOME environment variable */
	size_t home_len = bt_strnlen (HOME, (unsigned long) PATH_MAX);

	if (envp != (char **) NULL) {
		/* Go through the environment strings until we find "HOME=" */
		while (*envp) {
			if (bt_strncmp (*envp, HOME, home_len) == 0) {
				break;
			} else {
				envp++;
			}
		}

		/*
		 * If *envp is not NULL, then we found HOME - 
		 * create btgeneric's path
		 */
		if (*envp) {
			unsigned long allowed_len = sizeof (buff) - 
				bt_strnlen ("/" BTGENERIC_EFI, (unsigned long) NAME_MAX) - 1;
			unsigned long actual_len =
				bt_strnlen (*envp + home_len, allowed_len + 1);

			assert ((long) allowed_len > 0);
			if (actual_len < allowed_len) {
				bt_strncpy (buff, *envp + home_len, allowed_len);
				assert (buff[actual_len] == '\0');

				bt_strncat (buff, "/" BTGENERIC_EFI,
					    MIN (sizeof (buff) - bt_strlen (buff) - 1,
						 bt_strlen (BTGENERIC_EFI) + 1));
				retval = SYSCALL (stat, (buff, stat_buf));
				if (retval == 0) {
					bt_strncpy (dest, buff, dest_size - 1);
				}
			}
		}
	}

      out:
	/* ini file is in current directory */
#if !RELEASE
	if (retval == 0) {
		bt_snprintf (m_ini_file, sizeof (m_ini_file) - 1, "./" INI_FILE_NAME);
	}
#endif

#endif
	DBPRINT (0, "m_btg_filename='%s'\n", m_btg_filename);
	return retval;
}

#if ! RELEASE
/*****************************************************************************/
/* read the configuration file (Btrans.ini) */
/*****************************************************************************/
static long
read_config_file (char *argv0 UNUSED, char *btrans_ini_switches[])
{
	long ret;
	char IniBuffer[MAX_INI_BUF_SIZE] = "";
	long btrans_ini_switches_num = 0;
	int bt_IniFile = -1;
	static char btrans_ini_switches_array[MAX_SWITCHES][MAX_SWITCH_LEN];
	char logdir[PATH_MAX] = { 0 };
	char *p = IniBuffer, *q;

	DBPRINT (0, "m_ini_file='%s'\n", m_ini_file);

	bt_IniFile = SYSCALL (open, (m_ini_file, O_RDONLY, 0777));

	if (bt_IniFile <= 2) {
		DBPRINT (FALSE, "Can't open Btrans.ini file\n");
		return 0;
	}

	ret = SYSCALL (read, (bt_IniFile, (void *) IniBuffer,
			      (size_t) MAX_INI_BUF_SIZE - 1));

	if (IS_SYS_ERROR (ret)) {
		DBPRINT (TRUE, "Can't read Btrans.ini file\n");
		return 0;
	}

	*(p + ret) = '\0';
	while (*p != '\0') {
		/* skip all the new line chars */
		q = bt_strchr (p + 1, '\n');
		if (q != NULL) {
			/* handle ini files created on NT */
			if (*(q - 1) == '\r') {
				q--;
			}
			while ((*q == '\n') || (*q == '\r')) {
				*q++ = '\0';
			}
		}

		/* 
		 * the log option gets special treatment because it's btlib who
		 *  needs to open the log file
		 */
		if (bt_strncmp (p, "logdir", 6ul) == 0) {
			p += 6;
			if ((*p == '=') && (*(p++) != '\0')) {
				bt_strncpy (logdir, p, sizeof (logdir) - 1);
			}
		} else if (bt_strncmp (p, "log", 3ul) == 0) {
			open_log_file (p, argv0, logdir);
#ifdef STAT
		} else if (bt_strncmp (p, "btl_stats", 9ul) == 0) {
			btl_stats = 1;
#endif
		} else if (bt_strncmp (p, "btl_disable_smcopt", 16ul) == 0) {
                        btl_disable_smcopt = 1;
		} else if (bt_strncmp (p, "btl_no_dll_reuse", 16ul) == 0) {
                        btl_no_dll_reuse = 1;
                } else if (bt_strncmp (p, "btl_not_extended_user_mem", 24ul) == 0) {
			fprintf(g_log_file, "btl_not_extended_user_mem : IA-32 user memory space is limitted in 3G\n");
			btl_extended_user_mem = 0;
			extended_user_mem_flag = 0; 
		} else if (bt_strncmp (p, "btl_prt_dll", 11ul) == 0) {
                        DBCODE (TRUE, btl_prt_dll = 1);
		} else if (bt_strncmp (p, "btl_print_cs", 12ul) == 0) {
			DBCODE (TRUE, btl_prt_cs = 1);
		} else if (bt_strncmp (p, "affinity=no", 11ul) == 0) {
			btl_no_affinity = 1;
		} else if (bt_strncmp (p, "always_rie", 10ul) == 0) {
			btl_always_rie = 1;
		} else if (bt_strncmp (p, "btl_print_api", 13ul) == 0) {
			DBCODE (TRUE, btl_prt_api = 1);
		} else if (bt_strncmp (p, "btl_print_mem_region", 20ul) == 0) {
			DBCODE (TRUE, fprintf(g_log_file, "btl_print_mem_region : print memory region info for debug\n"));
			DBCODE (TRUE, btl_prt_mem_region = 1);
		} else if (bt_strncmp (p, "btl_print_mem", 13ul) == 0) {
			DBCODE (TRUE, fprintf(g_log_file, "btl_print_mem : print memory info for debug\n"));
			DBCODE (TRUE, btl_prt_mem = 1);
		} else if (bt_strncmp (p, "btl_print_ptrace", 16ul) == 0) {
			DBCODE (TRUE, btl_prt_pt = 1);
		} else if (bt_strncmp (p, "btl_print_sig", 13ul) == 0) {
			DBCODE (TRUE, btl_prt_sig = 1);
		} else if (bt_strncmp (p, "btl_print_thread", 16ul) == 0) {
			DBCODE (TRUE, btl_prt_thread = 1);
		} else if (bt_strncmp (p, "btl_print_syscall", 17ul) == 0) {
			DBCODE (TRUE, btl_prt_syscall = 1);
		} else if (*p != '\0' && *p != ';') {
			/* 
			 * Add next control directive to the 
			 * pseudo-btrans_ini_switches array 
			 */
			assert (bt_strlen (p) < MAX_SWITCH_LEN - 1);
			*btrans_ini_switches_array[1 + btrans_ini_switches_num] = '-';
			bt_strncpy (btrans_ini_switches_array [1 + btrans_ini_switches_num] + 1, p,
				    (size_t) MAX_SWITCH_LEN - 1);
			btrans_ini_switches_num++;
			assert (btrans_ini_switches_num < MAX_SWITCHES);
		}

		/* Next directive */
		if (q) {
			p = q;
		} else {
			break;
		}
	}
	SYSCALL (close, (bt_IniFile));

	if (btrans_ini_switches_num > 0) {
		int t;
		for (t = 1; t <= btrans_ini_switches_num; t++) {
			DBPRINT (0, "%d is %s\n", t,
				 btrans_ini_switches_array[t]);
			btrans_ini_switches[t] = (char *) &(btrans_ini_switches_array[t]);
		}
	}
	return btrans_ini_switches_num + 1;
}

/*****************************************************************************/
/* parse the "log" line in the Btrans.ini file
 * input - the beginning of the line (points at "log")
 * output - pointer to the beginning of the next line
 */
/*****************************************************************************/
static void
open_log_file (char *line, char *argv0, char *logdir)
{
	char *p = line;
	int tmp_log_file;
	char log_file_name[PATH_MAX] = { 0 };
	char *lfnp = log_file_name;
	unsigned long len;
	unsigned long chars_left = sizeof (log_file_name) - 1;

	/* add the logdir */
	if (*logdir != '\0') {
		bt_strncpy (lfnp, logdir, chars_left);
		len = bt_strlen (lfnp);
		lfnp += len;
		chars_left -= len;
		if (*lfnp != '/') {
			*lfnp = '/';
			lfnp++;
			chars_left--;
			assert (chars_left > 0);
		}
	}

	p += 3;
	/* log required - prepare log file name */
	if (*p == '=' && *(p + 1) != '\0') {
		/* Name present as well */
		p++;
		assert (bt_strlen (p) < MAX_SWITCH_LEN - 1);
		if (bt_strncmp (p, "<pid>", 5ul) == 0) {
			bt_snprintf (lfnp, chars_left, "log.%d", SYSCALL (gettid, ()));
		} else if (bt_strncmp (p, "<name>", 6ul) == 0) {
			char *pa = argv0;
			char *pl = pa;
			while (pa > (char *) 1) {
				pl = pa;
				pa = bt_strchr (pa, '/') + 1;
			}
			bt_snprintf (lfnp, chars_left, "log.%s", pl);
		} else if (bt_strncmp (p, "<name-pid>", 10ul) == 0) {
			char *pa = argv0;
			char *pl = pa;
			while (pa > (char *) 1) {
				pl = pa;
				pa = bt_strchr (pa, '/') + 1;
			}
			bt_snprintf (lfnp, chars_left, "log.%s-%d", pl,
				     SYSCALL (gettid, ()));
		} else {
			bt_strncpy (lfnp, p, chars_left);
		}
	} else {
		/* else the name remains the default */
		len = MIN (chars_left, bt_strlen (LOG_FILE_NAME));
		bt_strncat (lfnp, LOG_FILE_NAME, len);
	}

	DBPRINT (0, "log_file_name='%s'\n", log_file_name);

	/* open the file */
	tmp_log_file = SYSCALL (open, (log_file_name, O_CREAT | O_TRUNC | O_WRONLY, 0644));
	if (IS_SYS_ERROR (tmp_log_file)) {
		DBPRINT (TRUE, "Can't create LOG file %s:%d\n",
			 log_file_name, tmp_log_file);
	} else {
		g_log_file = tmp_log_file;
	}
}
#endif

/*****************************************************************************/
/* load and init btgeneric.
 * includes reading the ini file and initialization of the debug globals.
 *
 * if is_ver != 0 then only extract the version string from btgeneric and
 * return
 */
/*****************************************************************************/
static long
load_and_init_btgeneric (char **argv RELEASE_UNUSED, char **envp, long is_ver,
			 char btg_ver[PATH_MAX])
{
	char *btrans_ini_switches[MAX_SWITCHES] = { 0 };
	API_TABLE_TYPE *btrans_api_table_start;
	BT_STATUS_CODE bt_retval;
	long btrans_ini_switches_num = 0;
	long retval;
	unsigned i;
	unsigned long rsrc[2] = { 0ul, 0ul };	/* size and location of rsrc section */
	API_VERSION cur_api_version;
	__s32 index_of_accepted_api_version;
	void *p_btg_get_api_info;
	API_TABLE_TYPE api_table;

	/* load BTGeneric and get its API table start address. */
	retval = set_btgeneric_path (m_btg_filename, sizeof (m_btg_filename), envp);
	if (retval != 0) {
		DBPRINT (TRUE, "set_btgeneric_path failed\n");
		return -1;
	}

	/*
	 * If Btrans.ini configuration file is present, parse it and set
	 * ia32exec.bin debug parameters (BTGeneric_Debug_Settings)
	 */
	bt_memset ((void *) btrans_ini_switches, 0,
		   MAX_SWITCHES * sizeof (char *));
#if ! RELEASE
	btrans_ini_switches_num = read_config_file (argv[1], btrans_ini_switches);
	print_version_to_log ();
#endif

	/* initialize memory region manager*/
#if RELEASE
	init_mregion_managers();
#else
	init_mregion_managers(btl_extended_user_mem);
#endif

	retval = load_pe_binary (m_btg_filename,
				 &btl_btg_load_address,
				 &p_btg_get_api_info, &btl_btg_size, rsrc);

	if (retval != 0) {
		DBPRINT (TRUE, "load_pe_binary returned (%lx).\n", retval);
		return -1;
	}

	/* look for the version string in btgeneric */
	get_version_str ((void *) rsrc[0], rsrc[1], btg_ver,
			 (unsigned long) PATH_MAX);
	if (is_ver) {
		return retval;
	}

	cur_api_version.major_version = MAJOR_VERSION_NUMBER;
	cur_api_version.minor_version = MINOR_VERSION_NUMBER;

	bt_retval = (*(PTR_GET_API_INFO_FUNC) p_btg_get_api_info) ((API_VERSION
								    *)
								   (&cur_api_version),
								   1,
								   (__s32
								    *)
								   (&index_of_accepted_api_version),
								   (API_TABLE_TYPE
								    **)
								   (&btrans_api_table_start));

	if (bt_retval != BT_STATUS_SUCCESS) {
		DBPRINT (TRUE,
			 "\nCpuProcessInit: BTGenericGetAPIInfo(...) returned "
			 "bt_retval != BT_STATUS_SUCCESS\n");
		return -1;
	}

	/* perform the API table initialization */
	for (i = 0; i < NO_OF_BTGENERIC_APIS; i++) {
		btl_place_holder_table[i] = 
			GET_PLABEL_PTR_FROM_API_TABLE ((*btrans_api_table_start), i);
	}

	/* initialize BTGeneric placeholder table and get glst size from BTGeneric */
	api_table = &btl_api_table_elements[0];
	bt_retval = BTGENERIC_START (&cur_api_version,
				     &api_table,
				     btl_btg_load_address,
				     (void *) (btl_btg_size +
					       btl_btg_load_address),
				     g_clstp_offset,
				     (unsigned *) &btl_clst_glst_size);
	if (bt_retval != BT_STATUS_SUCCESS) {
		DBPRINT (TRUE, "BTGENERIC_START failed bt_retval=%d\n", bt_retval);
		return -1;
	}
/*The btl_clst_glst_size returned from BTGENERIC_START only included glst size, so adding the context32(clst) size and alignment*/
    btl_clst_glst_size = ROUNDUP (btl_clst_glst_size+sizeof(BTGENERIC_IA32_CONTEXT)+ctx_align_for_glst, __alignof__ (btlib_thread_data_t));
	DBPRINT (0,
		 "load_and_init_btgeneric: btl_clst_glst_size=%u<%x> \n",
		 btl_clst_glst_size, btl_clst_glst_size);

	bt_retval = BTGENERIC_DEBUG_SETTINGS (btrans_ini_switches_num,
					      btrans_ini_switches);
	if (bt_retval != BT_STATUS_SUCCESS) {
		DBPRINT (TRUE, "BTGENERIC_DEBUG_SETTINGS failed:%d\n",
			 bt_retval);
		return -1;
	}

	return 0;
}

/*****************************************************************************/
/*
 * Report to BTGeneric the location and dimensions of the loaded 
 * modules
 */
/*****************************************************************************/
static void
report_initial_code_areas (btrans_binary_map_t * app_info)
{
	/* the dynamic loader */
	if (app_info->dl_base_addr != 0) {
		DBPRINT (0, "dl_base_addr=%x dl_size=%x\n",
			 app_info->dl_base_addr, app_info->dl_size);
		BTGENERIC_FLUSH_IA32_INSTRUCTION_CACHE (U2P (app_info->dl_base_addr),
							app_info->dl_size,
							BT_FLUSH_ALLOC);
	}
	/* the binary */
	BTGENERIC_FLUSH_IA32_INSTRUCTION_CACHE (U2P (app_info->start_code),
						app_info->end_code - app_info->start_code + 1,
						BT_FLUSH_ALLOC);
}

/*****************************************************************************/
/*****************************************************************************/
static char *
find_unicode_str (char *base_addr, unsigned long limit, char *str)
{
	long status = 0;
	char *base_p = base_addr;
	char *start_unicode_str = NULL;
	char *str_p = str;

	while (1) {
		if ((unsigned long) (base_p - base_addr) > limit) {
			/* limit exceeded */
			return NULL;
		}

		if (status == 0) {
			if ((*base_p == *str_p) && (*(base_p + 1) == 0)) {
				status = 1;
				start_unicode_str = base_p;
				str_p++;
				base_p += 2;
			} else {
				base_p++;
			}
		} else if (status == 1) {
			if ((*base_p != *str_p) || (*(base_p + 1) != 0)) {
				status = 0;
				start_unicode_str = NULL;
				str_p = str;
				base_p++;
			} else if (*str_p == '\0') {
				/* bingo */
				break;
			} else {
				str_p++;
				base_p += 2;
			}
		}
	}
	return start_unicode_str;
}

/*****************************************************************************/
/*****************************************************************************/
static void
extract_unicode_str (char *ver_str_p, char *ver_str,
		     unsigned long ver_str_size RELEASE_UNUSED)
{
	unsigned long size = 0;
	while (*ver_str_p != 0) {
		assert (*(ver_str_p + 1) == 0);
		size++;
		assert (size < ver_str_size);
		*ver_str = *ver_str_p;
		ver_str_p += 2;
		ver_str += 1;
	}
	*(ver_str + size) = '\0';
}

/*****************************************************************************/
/*****************************************************************************/
static void
get_version_str (void *rsrc, unsigned long size, char *ver_str,
		 unsigned long ver_str_size)
{
	char str[] = "FileVersion";
	char *ver_str_p = NULL;

	ver_str_p = find_unicode_str (rsrc, size, str);
	assert (ver_str != NULL);

	ver_str_p += 2 * bt_strlen (str);
	while (*ver_str_p == '\0') {
		ver_str_p++;
	}

	extract_unicode_str (ver_str_p, ver_str, ver_str_size);
}

/*****************************************************************************/
/* remove first two arguments from argv. */
/*****************************************************************************/
static int
shift_argv (int argc, char *argv[])
{
	char *end_of_args = argv[argc - 1] + bt_strlen (argv[argc - 1]) + 1;
	char *new_end_of_args = (argc > 2 ? end_of_args - (argv[2] - argv[0])
				 : argv[0]);

	/* shift the args 2 positions to the left, so argv[2] becomes argv[0] */
	if (argc > 2)
		bt_memmove (argv[0], argv[2], (size_t) (end_of_args - argv[2]));
	/* reset the rest of the strings */
	bt_memset (new_end_of_args, 0,
		   (size_t) (end_of_args - new_end_of_args));
	return 0;
}

/*****************************************************************************/
/* print the versions of ia32el components */
/*****************************************************************************/
static void
print_ver (char *btg_ver)
{
	char verstr[PATH_MAX];

	if (btg_ver) {
		fprintf (stdout, "IA32X Generic   :\tversion=%s\tpath=%s\n",
			 btg_ver, m_btg_filename);
	} else {
		fprintf (stdout, "IA32X Generic   :\tnot found\n");
	}
	fprintf (stdout, "IA32X OS-Wrapper:\tversion=%d.%d.%s",
		 MAJOR_VERSION_NUMBER, MINOR_VERSION_NUMBER, BUILD_NO);
	iti_get_ia32toia64_version (verstr);
	fprintf (stdout, ".%s", verstr);
	lcs_get_ia32x_utils_version (verstr);
	fprintf (stdout, ".%s", verstr);
	fprintf (stdout, ".%s\n", MODE);
}

#ifndef RELEASE
static void
print_version_to_log ()
{
	char verstr[PATH_MAX];
	fprintf (g_log_file, "BTLib build number: %d.%d.%s",
		 MAJOR_VERSION_NUMBER, MINOR_VERSION_NUMBER, BUILD_NO);
	iti_get_ia32toia64_version (verstr);
	fprintf (g_log_file, ".%s", verstr);
	lcs_get_ia32x_utils_version (verstr);
	fprintf (g_log_file, ".%s\n", verstr);
}
#endif

