/* 
 * btlib_mme.c
 * implementation of MME interface
 */
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
#include <btlib.h>
#include <kernel_subst64.h>
#define __IA32EL__
#define bool  _Bool
#include <ia32el_mme_api.h>
#include <ia32toia64.h>
#include <btlib_asm.h>

/*int g_is_mme_mode = 0;*/
uint64_t btl_mme_flags = 0;
static struct _mme_api_table_tag m_mme_api;
extern long g_clstp_offset;
extern size_t __default_stack_size;

/* MME API functions */
/*****************************************************************************/
/*****************************************************************************/
ia32el_retval ia32el_mme_version (char version[IA32EL_MME_VER_STR_LEN])
{
	bt_strncpy (version, IA32EL_MME_VER_MAJOR "." IA32EL_MME_VER_MINOR, 
		       (uint64_t)IA32EL_MME_VER_STR_LEN - 1);
	return IA32EL_SUCCESS;	
}

/*****************************************************************************/
/*****************************************************************************/
ia32el_retval ia32el_mme_init (uint64_t flags,
		struct _mme_api_table_tag * mme_api,
		int64_t tls_offset)
{
/* it is the responsibility of the caller to make sure
 * this function is called in an MT safe manner */
	btl_mme_flags = flags;
	m_mme_api = *mme_api;
	g_clstp_offset = tls_offset;

	g_is_mme_mode = 1;

	DBPRINT (0, "\n");

	return IA32EL_SUCCESS;
}

/*****************************************************************************/
/*****************************************************************************/
ia32el_retval ia32el_mme_load_and_exec (int32_t argc, char ** argv, char ** envp)
{
	int ret;
	char ** ext_argv = NULL;
	int32_t j;

	/* argv[0] holds the full path to the binary */
	if (argc < 1) {
		DBPRINT (TRUE, "no argument given\n");
		return IA32EL_FAILURE;
	}
	/* add 2 args needed for internal_load_and_exec */
	ext_argv = bt_malloc (sizeof(char *) * (3 + argc));
	if (ext_argv == NULL) {
		DBPRINT (TRUE, "bt_malloc failed\n");
		return IA32EL_FAILURE;
	}
	ext_argv[0] = "libia32x.so";
	ext_argv[1] = argv[0]; /* dup argv[0] */
	for (j = 0; j <= argc; j ++) {
		ext_argv[j+2] = argv[j]; /* j==argc -> argv[j]==NULL */
	}

	/* load and exec */
	ret = internal_load_and_exec (argc + 2, ext_argv, envp);
	DBPRINT (0, "ret=%d\n", ret);
	DBPRINT (ret == 0, "ebx=%x ecx=%x\n", cur_thread_data->bc32->ebx, cur_thread_data->bc32->ecx);
	return ret ? IA32EL_FAILURE : IA32EL_SUCCESS;
}

/*****************************************************************************/
/*****************************************************************************/
ia32el_retval ia32el_mme_ia32_context_get (ia32el_context32 * bc32)
{

	BT_STATUS_CODE btg_retval;

	bc32->composition_bits = (uint32_t)-1;/*full context*/
	btg_retval = BTGENERIC_IA32_CONTEXT_GET (NULL, bc32);

	return (btg_retval == BT_STATUS_SUCCESS) ? IA32EL_SUCCESS : IA32EL_FAILURE;
}

/*****************************************************************************/
/*****************************************************************************/
ia32el_retval ia32el_mme_run (ia32el_context32 * bc32)
{
	BT_STATUS_CODE btg_retval;
	int ret;


	DBPRINT (1, "eip=%x esp=%x\n", bc32->eip, bc32->esp);
	bc32->composition_bits = (uint32_t)-1;/*full context*/
	btg_retval = BTGENERIC_IA32_CONTEXT_SET (NULL, bc32);
	if (btg_retval != BT_STATUS_SUCCESS) {
		DBPRINT (TRUE, "ia32_context_set failed\n");
		return IA32EL_FAILURE;
	}


#if DEBUG & 0
	/* print stack */
	{
		unsigned * p = U2P (bc32->esp);
		int k, l;

		DBPRINT (1, "stack=\n");
		for (k = 0; k < 4; k ++) {
			fprintf (g_log_file, "\t%p: ", p + k * 4);
			for (l = 0; l < 4; l ++) {
				fprintf (g_log_file, "%08x ", *(p + k * 4 + l));
			}
			fprintf (g_log_file, "\n");
		}

	}
#endif

	ret = btrans_jump ();
	if (ret < 0) {
		DBPRINT (TRUE, "btrans_jump failed\n");
		return IA32EL_FAILURE;
	}

	return IA32EL_SUCCESS;
}

/*****************************************************************************/
/*****************************************************************************/
ia32el_sighand_result ia32el_mme_signal_handler (int32_t signo, struct siginfo * si, struct sigcontext * sc)
{
	int ret;
	ia32el_sighand_result retval;
	unsigned long int fault_ip;
	void *region;

	DBPRINT (0, "signo=%d si_signo=%d\n", signo, si->si_signo);
	ret = btlib_signal_handler (signo, si, sc);

	if (!ret) {
		fault_ip = sc->sc_ip;
		if ( !find_mregion(MREGION_MANAGER_BTGENERIC, (unsigned long)fault_ip, &region)
		     && (fault_ip > BTLIB_MAPPING_BASE)
		     && (fault_ip < BTLIB_MAPPING_BASE + 0x100000000)) {
			retval = MME_IA32_NATIVE_EXCEPTION;
		} else {
			retval = MME_IA32_SIGNAL_NOT_HANDLED;
		}
	} else {
		retval = MME_IA32_SIGNAL_HANDLED;
	}
	DBPRINT (0, "retval=%d\n", retval);
	return retval;
}
/*****************************************************************************/
/*****************************************************************************/
ia32el_retval ia32el_mme_thread_term (void)
{
	/* free BTLib thread stack, which was deallocated in __btlib_thread_exit */
	if (cur_thread_data->ia32_thread_exited == 1) {
		SYSCALL (munmap, ((unsigned long)cur_thread_data->stackblock, __default_stack_size));
	}

	return IA32EL_SUCCESS;
}
/*****************************************************************************/
/*****************************************************************************/
void * ia32el_mme_memory_map (uint32_t addr, uint32_t size, uint32_t prot)
{
	long retval;
	void * retaddr;
	BTGENERIC_IA32_CONTEXT bc32;
	unsigned ia32_page_no;

	ia32_page_no = ROUNDUP (size, IA32_PAGE_SIZE) / IA32_PAGE_SIZE;
	DBPRINT (0, "addr=%p size=%x prot=%x ia32_page_no=%d\n",
			addr, size, prot, ia32_page_no);

	IA32_SYSCALL_PARAM1 (&bc32) = addr;
	IA32_SYSCALL_PARAM2 (&bc32) = ia32_page_no;
	IA32_SYSCALL_PARAM3 (&bc32) = prot;
	IA32_SYSCALL_PARAM4 (&bc32) = MAP_PRIVATE | MAP_ANON;
	IA32_SYSCALL_PARAM5 (&bc32) = (unsigned)-1;
	IA32_SYSCALL_PARAM6 (&bc32) = 0;

	retval = btlib_ia32_mmap2 (&bc32);
	retaddr = (void*)retval;
	DBPRINT (0, "retval=%lx retaddr=%p\n", retval, retaddr);
	return retaddr;
}
/*****************************************************************************/
/*****************************************************************************/
int32_t ia32el_mme_memory_unmap (void * addr, uint32_t size)
{
	int32_t ret;
	DBPRINT (0, "addr=%p size=%x\n", addr, size);
	ret = ILP32_WRAPPER2 (IA32_SYSCALL_MUNMAP, addr, size);
	DBPRINT (0, "ret=%d\n", ret);
	return ret;
}

/*****************************************************************************/
/*****************************************************************************/
ia32el_retval ia32el_mme_complete_parent_thread_create (void * clone_params, int64_t retval, int64_t *cpid)
{
	int64_t child_pid;


	DBPRINT (1, "clone_params=%p retval=%ld\n", clone_params, retval);
	child_pid = btlib_complete_mme_parent_thread_create (clone_params, retval);
	if (child_pid < 0) {
		return IA32EL_FAILURE;
	}
	after_syscall_returns ((int)IA32_SYSCALL_CLONE, child_pid, 0);
	*cpid = child_pid;
	return IA32EL_SUCCESS;
}
/*****************************************************************************/
/*****************************************************************************/
ia32el_retval ia32el_mme_complete_child_init (void * clone_params)
{
	void * ret;
	DBPRINT (1, "clone_params=%p\n", clone_params);
	ret = clone_main (clone_params);
	DBPRINT (1, "ret=%p\n", ret);
	return ret ? IA32EL_FAILURE : IA32EL_SUCCESS;
}

/*****************************************************************************/
/*****************************************************************************/
ia32el_retval ia32el_mme_query_memory_permission (void *addr, int64_t *perm)
{
	int64_t pperm;
	void *start;
	unsigned long size;

	pperm = btlib_memory_query_permissions (addr, &start, &size);
	if (pperm < 0) {
		assert (!"btlib_memory_query_permissions failed");
		*perm = 0;
		return IA32EL_FAILURE;
	}
	*perm = pperm;
	return IA32EL_SUCCESS;
}

/* HELPER FUNCTIONS */

/*****************************************************************************/
/*****************************************************************************/
void btlib_mme_trap (BTGENERIC_IA32_CONTEXT * bc32)
{
	unsigned * trap_code_p, trap_code;

	/* get the trap code from the context */
	trap_code_p = U2P (bc32->eip + 2);
	bt_memcpy (&trap_code, trap_code_p, sizeof (trap_code));

	DBPRINT (0, "trap_code=%x eip=%x esp=%x\n", trap_code, bc32->eip, bc32->esp);
	DBPRINT (0, "eax=%x ebx=%x ecx=%x edx=%x\n",
			cur_thread_data->bc32->eax,
			cur_thread_data->bc32->ebx,
			cur_thread_data->bc32->ecx,
			cur_thread_data->bc32->edx);


	switch (trap_code) {
		case IA32EL_TRAP_AUX_APP_READY:
		case IA32EL_TRAP_RETURN_FROM_RUN:
			DBPRINT (0, "jump to run_context\n");
			longjmp (*cur_thread_data->btgeneric_run_context, RETURN);
			break;
		case IA32EL_TRAP_JNI_CALL:
		case IA32EL_TRAP_JAVAVM_CALL:
		case IA32EL_TRAP_THREAD_FUNC:
		case IA32EL_TRAP_PHDR_ITER:
		case IA32EL_TRAP_JNIVM_CALL:
			(void)(*m_mme_api.mme_upcall) ((ia32el_context32*)bc32, trap_code);
			break;
		default:
			DBPRINT (TRUE, "unknown trap_code:%lx\n", trap_code);
			assert ( ! "should not get here" );
	}
}



/*****************************************************************************/
/*****************************************************************************/
long btlib_mme_is_ia64_thread (void * clone_params)
{
	long ret;
	DBPRINT (1, "clone_params=%p\n", clone_params);
	ret = m_mme_api.mme_thread_at_clone (clone_params);
	DBPRINT (1, "ret=%ld\n", ret);
	return ret;
}

/*****************************************************************************/
/*****************************************************************************/
void btlib_mme_run_jvm_handler(siginfo_t *siginfo)
{
	m_mme_api.mme_run_jvm_handler((void *)siginfo);
	return;
}

