/*  
 * btlib_pending_que.h
 *  
 *
 * 
*/
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#ifndef BTLIB_PENDINTG_QUE_H
#define BTLIB_PENDINTG_QUE_H

#include "btlib_signal.h"

typedef struct _sig_pending_tag {
	unsigned next;
	unsigned long time_stamp;
	siginfo_t info;
        int syscall_id;
} sig_pending;

typedef struct signal_pending_que_t {
	sig_pending *que;
	unsigned long size;
	unsigned first_pending;
	unsigned last_pending;	/* index of the last entry in the used list */
	unsigned first_free;
	unsigned total_pending;	/* total sum of signals in the que */
	sigset32_t pending_set;
	unsigned char pending_sigs[_NSIG + 1];	/* signo is the index to this array
						 * so entry [0] is never used and
						 * the array's size should be one 
						 * more than the number of signals */
} signal_pending_que;

#define INVALID_ENTRY (unsigned)-1

inline static long btlib_que_is_empty (signal_pending_que *pque);
/******************************************************************************/
/* return 0 if the que is surely not empty 
 * return 1 if the que is possibly emtpy */
/******************************************************************************/
inline static long
btlib_que_is_empty (signal_pending_que *pque)
{
	sigset32_t ss;
	
	if ((pque->total_pending) == 0) {

		/* we can not assert isemptyset32 (&ss, WORDS_IN_MASK)
		 * because one signal may arrive after the condition statement
		 */
		return 1;
	}

	ss = pque->pending_set;

	/* not include all signals which is actually  blocked */
	sig_andnot_mask (&ss, &cur_thread_data->sigmask, WORDS_IN_MASK);

	return isemptyset32 (&ss, WORDS_IN_MASK) ? 1 : 0;

}

#endif  /* BTLIB_PENDINTG_QUE_H */

