/*  
 * btlib_thread_data.h
 *  
 * data structure for thread 
 * 
*/
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#ifndef _BTLIB_THREAD_DATA_H
#define _BTLIB_THREAD_DATA_H

#include <ia32x_utils.h>
#include "btlib_segment.h"
#include "btrans.inc/ia32exec_os_api.h"

/* This is the thread specific data struct that BTLib maintains. */

struct signal_handler_table;
struct signal_pending_que_t;
struct _debugee_t;


typedef struct _suspend_t{

        int suspender_tid;
        long semaphore;
        int canonize_result;

}suspend_t;


typedef struct tia_signal_t {
	siginfo_t siginfo;
	unsigned int trapno;
} tia_signal_t;

#define SAFECPY_PFF 0
#define SAFECPY_ON	1 /*the bit is set by safe memory copy function*/
#define SAFECPY_REDO 2 /*failed due to smc, the bit is set in signal handler*/

#define SHOULD_IGNORE_SIGNAL(curthread) ((((curthread)->safe_memcpy_status) & SAFECPY_ON) == SAFECPY_ON)
#define SAFECPY_NEED_REDO(curthread) ((((curthread)->safe_memcpy_status) & SAFECPY_REDO) == SAFECPY_REDO) 

typedef struct btlib_thread_data_t {

	/* pointer to the next/prev thread */
	struct btlib_thread_data_t *next;
	struct btlib_thread_data_t *prev;

	unsigned long __r13;

	/* pointer to the IA-32 context inside btgeneric */
	BTGENERIC_IA32_CONTEXT *bc32;

	/* top of 64 bit stack, NULL for the 1st thread */
	void *stackblock;
	/* size of stack */
	unsigned long user_stack_size;
	/* for mme mode */
	pthread_t pthread_h;
	unsigned long ia32_thread_exited;

	/* last utilization measured for this thread */
	unsigned long last_utlization;	
	/* table of IA-32 signal handlers */
	struct signal_handler_table *sighandlers;

	/* the TLS descriptor, e.g., the GDT descriptor */
	desc_struct tls_array[GDT_USER_ENT_NUM];

	/* linked list of threads debugged by the current thread */
	struct _debugee_t *debugee_list;
	struct _suspend_t suspend_data;

	/* is eax changed after the point it was saved in debuggee */
	int is_eax_changed;

	/* pending signals for the IA-32 proc */
	struct signal_pending_que_t *pending_que;

	/* long system call handling */
	long *long_syscall_flag;

	/* thread personality */
	long *personality;

	/* 32 bit tidptr fields passed to clone */
	int * child_tidptr;
	int * parent_tidptr;

	/* info of sync signals */
	tia_signal_t tia_signal;

	/* alt stack pointer and size */
	stack32_t alt_stack;

	/* 
	 * this field is set upon entering a syscall and reset
	 * when exiting from it. it is mandatory that the field
	 * is cleared whenever we get to btlib_raise_signal while
	 * not inside a syscall
	 */
	int syscall_id;

	/* thread's tid */
	int tid;

	sigset32_t sigmask;

	/* 
	 * used for checking whether the current thread finished initialization.
	 * should be set after the first call to btrans_jump
	 */
	int is_run_context_ready;

	/* whether the thread exited normaly */
	int is_exited;

	/* 
	 * sem key for the sync algorithm for self canonization. set
	 * by the debugger in btlib_ptemu.c:sync_self_canonization()
	 */
	int self_canon_semkey;

	/* 
	 * indicate whether the thread is currently using the special
	 * signal handler for safe_memcpy
	 */
	int safe_memcpy_status;

	/* 
	 * when do_longjmp is used for returning to the first nesting level
	 * the status of the jump is placed in this var
	 */
	int longjmp_status;
#if DEBUG
	/* a bitmap of all locks inside btlib that the current thread is holding */
	int locks;
#endif

	/* context of the first nesting level, from which we call btgeneric_run */
	jmp_buf safecopy_jmpbuf __attribute__ ((aligned (16)));

	/* context of the first nesting level, from which we call btgeneric_run */
	jmp_buf *btgeneric_run_context;

#ifdef ALL_THREAD_AFFINITY
	/* this affinity mask is set only by explicit call to 32-setaffinity */
	cpu_set_t explicit_affinity_mask;
#endif	/* ALL_THREAD_AFFINITY */

	/* 1 if thread is currently suspended on the 64 bit level due to ptrace */
	int is_suspended;

	/* 1 if thread is currently waiting for the suspension lock (BTL_LOCK_IND_SUSPENSION) */
	int is_self_suspended;

	/* 1 if thread is in simulation session */
	int is_in_simulation_session;

	/* if the thread is currently in a non-blocking syscall */
	int in_nonblocking_syscall;

 	/* if the thread is in critical section */
 	int in_critical_section;

	/* used for debuggee request debugger change debug register for debuggee */
	int set_reg_addr;

	__u64 set_reg_value;

} btlib_thread_data_t;

#define IS_FREE(thread) (thread->tid == 0)
#define MAX_STACK_SIZE  100 * 1024 * 1024

#endif	// _BTLIB_THREAD_DATA_H

