/*
 * mem_regions.h
 *
 * prototypes of function to handle the list of memory regions that were 
 * mapped by the application
 *
 */
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 * Copyright (C) 2005       Eric Lin <eric.lin@intel.com>
 * Copyright (C) 2005       Bruce Chen <bruce.chen@intel.com>
 * Copyright (C) 2005       Dror Shenkar <dror.shenkar@intel.com>
 * Copyright (C) 2005       Xiaolan Huang <xiaolan.huang@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/


#ifndef _MEM_REGIONS_H
#define _MEM_REGIONS_H

#include "list.h"

/* Base addresses for code and memory allocation (above 4GB) and BTGeneric load address */
#define BTLIB_MAPPING_BASE		0x400000000
#define BTLIB_MEMORY_BASE		0x500000000
#define BTGENERIC_MEM_BASE		BTLIB_MAPPING_BASE

#define BTGENERIC_LOAD_ADDRESS		(BTLIB_MAPPING_BASE + 0x00000000)
#define INITIAL_CODE_ADDRESS		(BTGENERIC_MEM_BASE + 0x10000000)
#define INITIAL_DATA_ADDRESS		(BTGENERIC_MEM_BASE + 0x20000000)
#define INITIAL_DEFAULT_ADDRESS		(BTGENERIC_MEM_BASE + 0x30000000)
#define INITIAL_BTLIB_ADDRESS		(BTGENERIC_MEM_BASE + 0x50000000)

#define BTGENERIC_MEMORY_MASK		0xfffffffff0000000ul
#define BTGENERIC_MEMORY_SIZE		0x0000000100000000ul
#define BTGENERIC_MAX_ADDRESS		(BTGENERIC_MEM_BASE + BTGENERIC_MEMORY_SIZE)

#define IS_BTGENERIC_MEM_ADDR(addr)     ( addr >= BTGENERIC_MEM_BASE && \
                			  addr <  BTGENERIC_MAX_ADDRESS )


#define IA32_KERNEL_CODE		0xC0000000
typedef enum{
	MREGION_MANAGER_IA32=0,
	MREGION_MANAGER_BTGENERIC,
	MREGION_MANAGER_NUM,
} MREGION_MANAGER_ID;

/* attributes of a memory region */
struct mem_regions_manager_t;
typedef struct mem_regions_manager_t * mem_regions_manager;

#if RELEASE
extern long HIDDEN init_mregion_managers(void);
#else 
extern long HIDDEN init_mregion_managers(int extended_user_mem);
#endif


extern long HIDDEN lock_mregion_manager(MREGION_MANAGER_ID id);
extern long HIDDEN unlock_mregion_manager(MREGION_MANAGER_ID id);
extern long HIDDEN init_mregion_manager_after_fork(MREGION_MANAGER_ID id, long isparent);

extern long HIDDEN find_mregion (MREGION_MANAGER_ID id, unsigned long addr, void **p_region);
extern long HIDDEN remove_mregion (MREGION_MANAGER_ID id, unsigned long start, unsigned long len);
extern long HIDDEN insert_mregion (MREGION_MANAGER_ID id, unsigned long start, unsigned long len, unsigned prot, unsigned flags);
extern long HIDDEN is_memory_maps_changed(MREGION_MANAGER_ID id);
extern long HIDDEN reset_memory_maps(MREGION_MANAGER_ID id);

extern long HIDDEN is_inside_mregions (MREGION_MANAGER_ID id, unsigned long start, unsigned long len);
extern long HIDDEN get_mregion_prot (MREGION_MANAGER_ID id, unsigned long addr, unsigned *prot, unsigned long *start, 
                                    unsigned long *len, unsigned *flags);
extern long HIDDEN set_mregion_prot (MREGION_MANAGER_ID id, unsigned long start, unsigned long len, unsigned prot);
extern unsigned long HIDDEN find_free_mregion (MREGION_MANAGER_ID id, unsigned long start, unsigned long len, unsigned flags);
extern unsigned long HIDDEN find_free_mregion_below(MREGION_MANAGER_ID id, unsigned long start, unsigned long len, unsigned flags, unsigned long maxaddr);

extern list_handle HIDDEN get_first_mregion(MREGION_MANAGER_ID id);

extern unsigned long HIDDEN get_mregion_startaddr(list_handle region);
extern unsigned long HIDDEN get_mregion_endaddr(list_handle region);
extern list_handle HIDDEN get_next_mregion(list_handle region);
extern list_handle HIDDEN get_prev_mregion(list_handle region);

#ifdef DEBUG
extern void HIDDEN print_mregions(mem_regions_manager manager);
#endif
#endif //  _MEM_REGIONS_H
