/* 
 * mutex_internal.h
 *
 * details for simple mutex operations 
 *
 */
 /*
 *
 * This file has been derived from GNU libc. Please see COPYING.LIBC
 * for more information
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#ifndef _MUTEX_INTERNAL_H
#define _MUTEX_INTERNAL_H
#include "../include/mutex.h"

typedef union
{	 
	struct 
	{	 
		int __lock;
		unsigned int __count;
		unsigned long __owner;
	} __data;
	char __size[__SIZEOF_MUTEX_T];
	unsigned long pthread_h;
	long int __align;
} mutex_t;


#define SYS_futex __NR_futex
#ifndef FUTEX_WAIT
#define FUTEX_WAIT (0)
#endif
#ifndef FUTEX_WAKE
#define FUTEX_WAKE (1)
#endif

#define lll_futex_clobbers \
  "out5", "out6", "out7",                             \
  /* Non-stacked integer registers, minus r8, r10, r15.  */           \
  "r2", "r3", "r9", "r11", "r12", "r13", "r14", "r16", "r17", "r18",          \
  "r19", "r20", "r21", "r22", "r23", "r24", "r25", "r26", "r27",          \
  "r28", "r29", "r30", "r31",                             \
  /* Predicate registers.  */                             \
  "p6", "p7", "p8", "p9", "p10", "p11", "p12", "p13", "p14", "p15",       \
  /* Non-rotating fp registers.  */                       \
  "f6", "f7", "f8", "f9", "f10", "f11", "f12", "f13", "f14", "f15",       \
  /* Branch registers.  */                            \
  "b6", "b7",                                     \
  "memory"

#define lll_futex_wait(ftx, val) lll_futex_timed_wait (ftx, val, 0)

#define lll_futex_timed_wait(ftx, val, timespec) \
  ({                                          \
     register long int __o0 asm ("out0") = (long int) (ftx);            \
     register long int __o1 asm ("out1") = FUTEX_WAIT;                \
     register int __o2 asm ("out2") = (int) (val);                \
     register long int __o3 asm ("out3") = (long int) (timespec);         \
     register long int __r8 asm ("r8");                       \
     register long int __r10 asm ("r10");                     \
     register long int __r15 asm ("r15") = SYS_futex;                 \
                                          \
     __asm __volatile ("break %7;;"                       \
               : "=r" (__r8), "=r" (__r10), "=r" (__r15),         \
             "=r" (__o0), "=r" (__o1), "=r" (__o2), "=r" (__o3)   \
               : "i" (0x100000), "2" (__r15), "3" (__o0), "4" (__o1), \
             "5" (__o2), "6" (__o3)                   \
               : "out4", lll_futex_clobbers);                 \
     __r10 == -1 ? -__r8 : __r8;                          \
  })

#define lll_futex_wake(futex, nr) \
  ({                                          \
     register long int __o0 asm ("out0") = (long int) (futex);            \
     register long int __o1 asm ("out1") = FUTEX_WAKE;                \
     register int __o2 asm ("out2") = (int) (nr);                 \
     register long int __r8 asm ("r8");                       \
     register long int __r10 asm ("r10");                     \
     register long int __r15 asm ("r15") = SYS_futex;                 \
                                          \
     __asm __volatile ("break %6;;"                       \
               : "=r" (__r8), "=r" (__r10), "=r" (__r15),         \
             "=r" (__o0), "=r" (__o1), "=r" (__o2)            \
               : "i" (0x100000), "2" (__r15), "3" (__o0), "4" (__o1), \
             "5" (__o2)                       \
               : "out3", "out4", lll_futex_clobbers);             \
     __r10 == -1 ? -__r8 : __r8;                          \
  })



#define lll_wait_tid(tid)                       \
    do                                          \
        {                                       \
            __typeof (tid) __tid;               \
            while ((__tid = (tid)) != 0)        \
                lll_futex_wait (&(tid), __tid); \
        }                                       \
        while (0)




int mutex_module_init (void);

#endif // _MUTEX__INTERNALH
