/* 
 * setjmp.h
 *
 * definitions for longjmp
 *
 */
 /*
 *
 * This file has been derived from GNU libc. Please see COPYING.LIBC
 * for more information
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#ifndef _SETJMP_H
#define _SETJMP_H

#include "../ia32x_utils.h"

/**************** from sysdeps/unix/sysv/linux/ia64/bits/setjmp.h ****************/
/* User code must not depend on the internal representation of jmp_buf. */
#define _JBLEN  70
/* the __jmp_buf element type should be __float80 per ABI... */
typedef long __jmp_buf[_JBLEN] __attribute__ ((aligned (16))); /* guarantees 128-bit alignment! */



/**************** from setjmp/setjmp.h ****************/
/* Calling environment, plus possibly a saved signal mask.  */
typedef struct __jmp_buf_tag {
	/* NOTE: The machine-dependent definitions of `__sigsetjmp'
	 * assume that a `jmp_buf' begins with a `__jmp_buf' and that
	 * `__mask_was_saved' follows it.  Do not move these members 
	 * or add others before it.  */ 
	__jmp_buf __jmpbuf;         /* Calling environment.  */
	int __mask_was_saved;       /* Saved the signal mask?  */
	k_sigset_t __saved_mask;    /* Saved signal mask.  */
} jmp_buf[1];

extern int sigsetjmp(jmp_buf buf, int savemask);
extern int setjmp(jmp_buf buf);

extern int litesigsetjmp(jmp_buf buf);

inline static void litelongjmp (jmp_buf env, int val);
inline static void longjmp(jmp_buf buf, int val);

extern int g_log_file;
inline static void
litelongjmp (jmp_buf env, int value)
{
	extern void __litelongjmp(__jmp_buf buf, int val);
	/* Call the machine-dependent function to restore machine state.  */
	__litelongjmp (env[0].__jmpbuf, value ?: 1);
}

/* Set the signal mask to the one specified in ENV, and jump
   to the position specified in ENV, causing the setjmp
   call there to return VAL, or 1 if VAL is 0.  */
inline static void
longjmp (jmp_buf env, int value)
{

	extern void __longjmp(__jmp_buf buf, int val);
	extern void abort ();
	if (env[0].__mask_was_saved) {
		long r;
		/* Restore the saved signal mask.  */
		r = SYSCALL (rt_sigprocmask, (SIG_SETMASK,
					&env[0].__saved_mask,
				  	(void *) 0ul,
					sizeof (k_sigset_t)));
		sys_err_chk (r, "sigprocmask failed");
	}

	/* Call the machine-dependent function to restore machine state.  */
	__longjmp (env[0].__jmpbuf, value ?: 1);
}


#endif // _SETJMP_H
