/* 
 * ia32x_loader.h
 * header file for the custom loader.
 *
 */
 /*
 *
 * This file has been derived from GNU libc. Please see COPYING.LIBC
 * for more information
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
#ifndef _IA32X_LOADER_H
#define _IA32X_LOADER_H

#include <ia32x_utils.h>
#include "utils.h"

#define BTLIB_TLS_SIZE 32
#define SYMBOL_NAME_SIZE 64


struct _fdesc_tag {
	u64 ip, gp;
};
struct _btlib_entry_points_tag {
	char name[SYMBOL_NAME_SIZE];
	struct _fdesc_tag ** fdesc;
};

extern struct _btlib_entry_points_tag g_btlib_entry_points[];

#if GDB_INTERFACE
struct link_map {
	/* These first few members are part of the protocol with the debugger.
	 * This is the same format used in SVR4.	*/

	Elf64_Addr l_addr;	/* Base address shared object is loaded at.	*/
	char *l_name;		/* Absolute file name object was found in.	*/
	Elf64_Dyn *l_ld;	/* Dynamic section of the shared object.	*/
	struct link_map *l_next, *l_prev; /* Chain of loaded objects.	*/
};
struct r_debug {
	int r_version;		/* Version number for this protocol.	*/
	struct link_map *r_map;	/* Head of the chain of loaded objects.	*/
	/* This is the address of a function internal to the run-time linker,
	 * that will always be called when the linker begins to map in a
	 * library or unmap it, and again when the mapping change is complete.
	 * The debugger can set a breakpoint at this address if it wants to
	 * notice shared object mapping changes.	*/
	Elf64_Addr r_brk;
	enum {
		/* This state value describes the mapping change taking place when
		 * the `r_brk' address is called.	*/
		RT_CONSISTENT,		/* Mapping change is complete.	*/
		RT_ADD,			/* Beginning to add a new object.	*/
		RT_DELETE		/* Beginning to remove an object mapping.	*/
	} r_state;
	Elf64_Addr r_ldbase;	/* Base address the linker is loaded at.	*/
};
#endif

extern struct link_map g_maps;
extern Elf64_Addr g_load_addr;

extern __thread void * t_clstp  __attribute__ ((tls_model ("initial-exec")));
extern __thread unsigned long t_ptrace __attribute__ ((tls_model ("initial-exec")));

extern u64 g_tls_offset;

extern void _dl_debug_state (void);
extern int load_btlib (int fd);
extern void * __tls_get_addr (u64 m, u64 offset);
extern void __gmon_start__ (void);
extern void * memset(void *s, int c, unsigned long n);
extern void * memcpy(void *dest, const void *src, unsigned long n);
extern u64 * get_loader_ptrace_flag_addr (void);
#endif // _IA32X_LOADER_H

