/*  
 * loader_tls.c
 *  
 * allocate and init thread local storage needed by gcc
 * 
*/
 /*
 *
 * This file has been derived from GNU libc. Please see COPYING.LIBC
 * for more information
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
/* tls stuff */
/* based on __libc_setup_tls in sysdeps/generic/libc-tls.c */

#include <ia32x_utils.h>
#include "utils.h"
#include "ia32x_loader.h"
#include "loader_tls.h"

static unsigned long m_tls_template_size = 0;
static void * m_tls_template = NULL;
static unsigned long m_tls_size = 0;
static unsigned long m_tls_align = 0;
/*
 * layout of the tls is
 *
 *                     r13
 *              PRE-   |
 *               TCB   |   TCB       TSD
 *            |--------|----|----|------------------|
 *                       |       ^
 *                       |       |
 *                       |       --------|
 *                       |               |
 *            	         ------>|------|------|
 *               	              DTV
 *
 */
/*****************************************************************************/
/*****************************************************************************/
void get_tls_static_info (unsigned long *sizep, unsigned long *alignp)
{
	if (sizep != NULL) {
		*sizep = m_tls_size;
	}
	if (alignp != NULL) {
		*alignp = m_tls_align;
	}
}
/*****************************************************************************/
/*****************************************************************************/
long cleanup_initial_tls (void * r13)
{
	long retval;
	retval = SYSCALL (munmap, ((unsigned long)r13, m_tls_size));
	return retval;
}
/*****************************************************************************/
/*****************************************************************************/
long allocate_initial_tls (Elf64_Phdr * tls_phdr)
{
	tcbhead_t * tcb;
	long retval;

	/* find the size and alignment of tls section */
	m_tls_align = tls_phdr->p_align;
	m_tls_template = (void *)tls_phdr->p_vaddr + g_load_addr;
	m_tls_template_size = tls_phdr->p_filesz;
	m_tls_size = tls_phdr->p_memsz + 	/* size of btlib's tls */
		     sizeof (t_clstp) +		/* size of loader's tls */
		     sizeof (t_ptrace) +	/* size of ptrace field */  
		     sizeof (tcbhead_t) +	/* size of tcb */
		     m_tls_align + __alignof__ (tcbhead_t); /* space needed for alignment */

	/* allocate a minimal tls - this memory will never get
	 * deallocated. it's not a big thing since it's only
	 * done for the first thread */

	tcb = (void *)SYSCALL (mmap, (0UL, m_tls_size, PROT_READ | PROT_WRITE, 
			       MAP_PRIVATE | MAP_ANON, -1, 0));
	if (IS_SYS_ERROR (tcb)) {
		return (long)tcb;
	}
	DBPRINT (0, "addr=%p size=%lx\n", tcb, m_tls_size);
	retval = init_tls (tcb);
	if (IS_SYS_ERROR (retval)) {
		return retval;
	}

	DBPRINT (0, "tcb=%p\n", tcb);
	
	/* set r13 to point at the tcb */
	__asm__ __volatile__ ("add r13=r0, %0;;\n"::"r"(tcb));

	return 0;
}

void cleanup_tls (void * r13 UNUSED)
{
	/* nothing to do */
}

/*****************************************************************************/
/* copy the template tls to the given tls area */
/*****************************************************************************/
long init_tls (void * r13 UNUSED)
{
	long retval = 0;
	/* we really have nothing to do here : 
	 * 1) tls_template is all zero - as was asserted in function set_tls
	 *    of the loader.
	 * 2) the memory allocated for tls should be all zero - asserted here
	 * 3) btlib is the only module loaded, so dtv is not needed
	 */
#if DEBUG
	char * p;
	unsigned long j;

	p = r13;
	for (j = 0; j < m_tls_size; j ++, p ++)
		assert (*p == 0);
#endif

	assert ((((u64)r13 + g_tls_offset) % m_tls_align) == 0);

	return retval;
}


