/* 
 * selinux.c
 * minimal libselinux API.
 *  
 */
//End of file description

/*
 *	INTEL CORPORATION PROPRIETARY INFORMATION
 *
 *	This software is supplied under the terms of a license
 *	agreement or nondisclosure agreement with Intel Corporation
 *	and may not be copied or disclosed except in accordance with
 *	the terms of that agreement.
 *	Copyright (c) 1991-1998	Intel Corporation.
 *
 */
#include <ia32x_utils.h>
#include "utils.h"
#include "selinux.h"

#define INITCONTEXTLEN 255
#define  XATTR_NAME_SELINUX "security.selinux"

/*
* get the security context of the current thread
* return 0 on success
*/
int bt_getcon(security_context_t *context)
{
	int fd;
	size_t size;
	ssize_t ret;
	fd = SYSCALL (open, ("/proc/self/attr/current", O_RDONLY,0));
	if (fd < 0)
		return -1;
	size = INITCONTEXTLEN + 1;
	*context = zmalloc( size);
	if (!(*context)) {
		ret = -1;
		goto out;
	}
	bt_memset( *context, 0, size);

	ret = SYSCALL (read, (fd, *context, size - 1));
	if (ret <= 0){
		ret = -1;
		goto out2;
	}
	ret = 0;
	goto out;
      out2:
	zfree( (*context) );
      out:
	SYSCALL (close,(fd));
	return ret;
}
/*
* get the security context of the current thread
* return 0 on success
*/

int bt_setcon(security_context_t context)
{
	int fd;
	ssize_t ret;
	fd = SYSCALL(open, ("/proc/self/attr/current", O_RDWR,0));
	if (fd < 0) {
		return -1;
	}
	if (context)
		ret = SYSCALL(write, (fd, context, bt_strlen(context) + 1));
	else
		ret = SYSCALL(write, (fd, NULL, 0));	/* clear */
	SYSCALL(close,(fd));
	if (ret < 0)
		return -1;
	else
		return 0;
}

void bt_freecon( security_context_t context){
	zfree( context );
}

/*
* get the security context of a file
*/
int bt_getfilecon(const char *path, security_context_t * context)
{
	size_t size;
	ssize_t ret;

	size = INITCONTEXTLEN + 1;
	*context = zmalloc(size);
	if (!(*context))
		return -1;
	bt_memset( *context, 0, size);
	ret = SYSCALL( getxattr, ((char*)path, XATTR_NAME_SELINUX, *context, size - 1));
	//ret = SYSCALL(open, ((char*)path, 0, 0));
	if (ret < 0)
		zfree( (*context) );
	return ret;
}

/* 
*  Compute a labeling decision and set *newcon to refer to it.
*  Caller must free via freecon. 
*/

int bt_security_compute_create(security_context_t scon,
				security_context_t tcon,
				security_class_t tclass,
				security_context_t *newcon)
{
	size_t size;
	int fd, ret;


	fd = SYSCALL(open,("/selinux/create", O_RDWR,0));
	if (fd < 0){
		return -1;
	}

	size = INITCONTEXTLEN + 1;
	*newcon = zmalloc(size);
	if (!(*newcon)) {
		ret = -1;
		goto out;
	}
	bt_memset(*newcon,0,size);


	bt_snprintf(*newcon, size, "%s %s %u", scon, tcon, tclass);
	ret = SYSCALL(write,(fd, *newcon, bt_strlen(*newcon)));
	if (ret < 0)
		goto out2;
	bt_memset(*newcon,0,size);
	ret = SYSCALL(read,(fd, *newcon, size - 1));
	if (ret < 0)
		goto out2;
	ret = 0;
	goto out;
      out2:
	zfree(*newcon);
      out:
	SYSCALL(close,(fd));
	return ret;
}




