/* 
 * utils.c
 * implementation of basic util functions.
 *
 */
 /*
 *
 * This file has been derived from GNU libc. Please see COPYING.LIBC
 * for more information
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
#include "utils.h"


struct _zmalloc_tag {
	void * addr;
	u64 size;
};
#define MALLOC_AREA_NUM 10
static struct _zmalloc_tag m_zmalloc_areas[MALLOC_AREA_NUM] = {{0,0}};

void * zmalloc (u64 size)
{
	int j;
	u64 p, len;

	for (j = 0; j < MALLOC_AREA_NUM; j ++) {
		if (m_zmalloc_areas[j].addr == NULL) {
			break;
		}
	}
	assert (j < MALLOC_AREA_NUM);


	len = ROUNDUP (size, getpagesize());
	p = SYSCALL (mmap, (0, len,
				PROT_READ|PROT_WRITE, MAP_ANON|MAP_PRIVATE, -1, 0));
	if (IS_SYS_ERROR (p)) {
		DBPRINT (1, "zmalloc failed:%ld\n", p);
		return NULL;
	}
	DBPRINT (0, "addr=%lx size=%lx\n", p, len);

	m_zmalloc_areas[j].addr = (void*)p;
	m_zmalloc_areas[j].size = len;
	DBPRINT (0, "zmalloc: addr=%lx size=%lx\n", p, len);

	return m_zmalloc_areas[j].addr;
}

void zfree (void * addr)
{
	long ret;
	u64 j;
	for (j = 0; j < MALLOC_AREA_NUM; j ++) {
		if (m_zmalloc_areas[j].addr == addr) {
			break;
		}
	}
	assert (j < MALLOC_AREA_NUM);

	ret = SYSCALL (munmap, ((u64)m_zmalloc_areas[j].addr, m_zmalloc_areas[j].size));
	assert ( ! IS_SYS_ERROR (ret) );
}




	
